#
# Cookbook Name:: omnibus-chef-manage
# Attributes:: default
#
# Copyright:: Copyright (c) 2013-2015 Chef Software, Inc.
# License:: Apache License, Version 2.0
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

# The attributes that are user-configurable for this app live in the
# Manage::Config module and are documented there. They are loaded by the config
# recipe.

## Enterprise
#
# These is used by the enterprise-chef-common cookbook
default['enterprise']['name'] = 'chef-manage'
default['chef-manage']['sysvinit_id'] = 'CM' # For Chef Manage


## Base directory attributes
default['chef-manage']['chef_base_path'] = "/opt/opscode"
default['chef-manage']['install_path'] = "/opt/#{node['enterprise']['name']}"
default['chef-manage']['var_path'] = "/var/opt/#{node['enterprise']['name']}"
default['chef-manage']['config_path'] = "#{node['chef-manage']['var_path']}/etc"
default['chef-manage']['log_path'] = "/var/log/#{node['enterprise']['name']}"
default['chef-manage']['tmp_path'] = "#{node['chef-manage']['var_path']}/tmp"

# Service-specific options
#
# The values for log_rotation file_maxbytes (10MB) and num_to_keep are just
# arbitrary sane defaults that match what Chef server uses.
default['chef-manage']['web']['log_directory'] = "#{node['chef-manage']['log_path']}/web"
default['chef-manage']['web']['log_rotation']['file_maxbytes'] = 104857600
default['chef-manage']['web']['log_rotation']['num_to_keep'] = 10

default['chef-manage']['worker']['log_directory'] = "#{node['chef-manage']['log_path']}/worker"
default['chef-manage']['worker']['log_rotation']['file_maxbytes'] = 104857600
default['chef-manage']['worker']['log_rotation']['num_to_keep'] = 10

default['chef-manage']['redis']['log_directory'] = "#{node['chef-manage']['log_path']}/redis"
default['chef-manage']['redis']['log_rotation']['file_maxbytes'] = 104857600
default['chef-manage']['redis']['log_rotation']['num_to_keep'] = 10

## Services
#
# These control whether or not the individual services needed to run the
# application are enabled.
# (these are needed *before* the config recipe loads, so they are defined here.
# They are also defined in libraries/config.rb)
default['chef-manage']['services']['web']['enable'] = true
default['chef-manage']['services']['worker']['enable'] = true

## Webapp
#
# These attributes control the Unicorn application server that runs the webapp.
# Most of these are defined in libraries/config.rb, but some are defined
# dynamically here.
#
# The number of worker processes to run on the machine.
# See http://unicorn.bogomips.org/Unicorn/Configurator.html#method-i-worker_processes
#
# This can be overridden in the config, but is set to the number of CPUs on the
# system here.
default['chef-manage']['webapp']['worker_processes'] = node['cpu']['total'].to_i

## SSL
#
# Configure the SSL client that Manage uses for communicating with the Infra Server API
# As of Chef Manage 3.0, we now support verifying the SSL certificate of the Infra Server
# For compatibility reasons we default to :verify_none at this time
# These are required to be set if Chef Infra Server has mtls enabled.
default['chef-manage']['ssl_verify_mode'] = :verify_none
default['chef-manage']['trusted_certs_dir'] = ""
default['chef-manage']['ssl_client_cert'] = ""
default['chef-manage']['ssl_client_key'] = ""
# Additional optional mtls settings
default['chef-manage']['ssl_ca_file'] = ""
