#
# Cookbook Name:: omnibus-chef-manage
# Recipe:: default
#
# Copyright:: Copyright (c) 2013-2015 Chef Software, Inc.
# License:: Apache License, Version 2.0
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

# Previous versions would put the cache path in
# /var/chef-manage/embedded/cookbooks, which would cause a bunch of warnings
# on reconfigure. If that is not our cache path (which is now defined in
# omnibus/config/software/chef-manage-cookbooks.rb), delete it.
directory "#{node['chef-manage']['install_path']}/embedded/cookbooks/local-mode-cache" do
  action :delete
  recursive true
  only_if do
    Chef::Config.cache_path !=
      "#{node['chef-manage']['install_path']}/embedded/cookbooks/local-mode-cache"
  end
end

private_chef_addon 'chef-manage' do
  action :nothing
end.run_action(:create)

# We're gonna check if webui1 is enabled, and bail if it is
# since we run on the same port
if node['private_chef']['opscode-webui']['enable']
  Chef::Application.fatal!("opscode-webui is enabled on this node, not installing chef-manage.")
end

# Remove the previous services
component_runit_supervisor "opscode-manage" do
  ctl_name "opscode-manage-ctl"
  install_path "/opt/opscode-manage"
  sysvinit_id "CM"
  action :delete
end

include_recipe 'enterprise::runit'
include_recipe 'omnibus-chef-manage::config'
include_recipe 'omnibus-chef-manage::redis'

# Do a sanity check to make sure both SAML and LDAP not enabled at the same time
if node['chef-manage']['ldap']['enabled'] && node['chef-manage']['saml']['enabled']
  Chef::Application.fatal!("Both SAML and LDAP auth are enabled at the same time - please enable only one of those auth types.")
end

class Chef::Resource::RunitService
  include Manage::Helpers
end

app = node['chef-manage']

app['services'].each do |service, options|
  recipe = "omnibus-chef-manage::#{service}"
  recipe += '_disable' unless options[:enable]
  include_recipe recipe
end

include_recipe 'omnibus-chef-manage::nginx'

# Because these symlinks get removed during the postrm
# of the chef-manage and opscode-manage packages, we should
# ensure that they're always here.
%w[ chef-manage-ctl opscode-manage-ctl ].each do |bin|
  link "/usr/bin/#{bin}" do
    to "/opt/chef-manage/bin/#{bin}"
  end
end

# Dump out a json file will all current attributes
file "#{app['config_path']}/chef-manage-running.json" do
  owner app['user']['username']
  group 'root'
  mode '0600'
  content Chef::JSONCompat.to_json_pretty(
    'chef-manage' => node['chef-manage'].to_hash,
    'run_list' => node.run_list,
  )
end
