#
# Cookbook Name:: omnibus-chef-manage
# Recipe:: development
#
# Copyright (c) 2015 Chef Software, Inc., All Rights Reserved.

# TODO: Allow setting version or installing from local packages
chef_server_ingredient "chef-server-core" do
  notifies :reconfigure, "chef_server_ingredient[chef-server-core]"
end

deb_path = node['chef-manage']['development']['chef_manage_deb_package_path']
rpm_path = node['chef-manage']['development']['chef_manage_rpm_package_path']

# TODO: Allow installing from packagecloud
package "chef-manage" do
  case node['platform_family']
  when 'debian'
    provider Chef::Provider::Package::Dpkg
    source deb_path
    only_if { deb_path }
  when 'rhel'
    provider Chef::Provider::Package::Rpm
    source rpm_path
    only_if { rpm_path }
  end
end

# Symlink the Test Kitchen cookbooks into the embedded cookbooks directory
Dir["/tmp/kitchen/cookbooks/*"].each do |dir|
  directory "/opt/chef-manage/embedded/cookbooks/#{File.basename(dir)}" do
    recursive true
    action :delete
  end

  link "/opt/chef-manage/embedded/cookbooks/#{File.basename(dir)}" do
    to dir
  end
end

# Write out a manage config to exercise some of the options
# TODO: Make this more robust, so we can test different configurations
directory '/etc/chef-manage'

file '/etc/chef-manage/manage.rb' do
  content <<-EOF
    events.port 12001
    redis.port 12002
    webapp.port 19462
    support_email_address 'test@test.com'
  EOF
end

execute "chef-manage-ctl reconfigure" do
  only_if { File.exist? "/usr/bin/chef-manage-ctl" }
end
