#
# Cookbook Name:: omnibus-chef-manage
# Recipe:: nginx
#
# Copyright:: Copyright (c) 2013-2015 Chef Software, Inc.
# License:: Apache License, Version 2.0
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

include_recipe 'omnibus-chef-manage::config'

app = node['chef-manage']
nginx_config_path = "#{node['private_chef']['nginx']['dir']}/etc"

service 'nginx' do
  control_cmd = "env SVDIR=/opt/opscode/sv #{app['chef_base_path']}/embedded/bin/sv"
  provider Chef::Provider::Service::Simple
  supports :restart => true, :status => true
  restart_command "#{control_cmd} restart nginx"
  action :nothing
end

%w{upstreams internal external}.each do |file|
  template "#{nginx_config_path}/addon.d/#{app['nginx_addon_prefix']}-chef-manage_#{file}.conf" do
    source "nginx/#{file}.conf.erb"
    owner 'root'
    group 'root'
    mode '0644'
    variables({
      :events_port   => app['events']['port'],
      :webapp_port   => app['webapp']['port'],
      :helper        => Object.new.extend(Manage::Helpers)
    })
    notifies :restart, 'service[nginx]' if OmnibusHelper.should_notify?('nginx')
  end

  # These used to be called "opscode-manage" instead. Delete the old ones.
  file "#{nginx_config_path}/addon.d/#{app['nginx_addon_prefix']}-opscode-manage_#{file}.conf" do
    action :delete
    notifies :restart, 'service[nginx]' if OmnibusHelper.should_notify?('nginx')
  end
end

# Configuration if we're running on a port or host different than the one the
# API is using
if use_standalone_nginx_server?(node)
  standalone_action = :create
else
  standalone_action = :delete
end

template "#{nginx_config_path}/nginx.d/manage.conf" do
  source 'nginx/standalone.conf.erb'
  action standalone_action
  notifies :restart, 'service[nginx]' if OmnibusHelper.should_notify?('nginx')
end
