#
# Cookbook Name:: omnibus-chef-manage
# Recipe:: webapp
#
# Copyright:: Copyright (c) 2013-2015 Chef Software, Inc.
# License:: Apache License, Version 2.0
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

include_recipe 'omnibus-chef-manage::config'

app = node['chef-manage']

unicorn_config ::File.join(app['config_path'], 'unicorn.rb') do
  listen app['webapp']['listen'] => {
    :backlog     => app['webapp']['backlog'],
    :tcp_nodelay => app['webapp']['tcp_nodelay']
  }
  mode '0600'
  owner app['user']['username']
  preload_app true
  worker_processes app['webapp']['worker_processes']
  worker_timeout app['webapp']['worker_timeout']

  after_fork """
  Sidekiq.configure_server do |config|
    config.redis = { :url => '#{app['redis']['url']}', :namespace => 'sidekiq' }
  end
  Sidekiq.configure_client do |config|
    config.redis = { :url => '#{app['redis']['url']}', :namespace => 'sidekiq' }
  end
  """

  notifies :restart, "component_runit_service[web]"
end

app_path = ::File.join(app['install_path'], 'embedded/service/chef-manage')
config_ru = ::File.join(app_path, 'config.ru')

template config_ru do
  source 'config.ru.erb'
  mode '0644'
  owner app['user']['username']
  notifies :restart, "component_runit_service[web]"
end

component_runit_service "web" do
  package node['enterprise']['name']
end

ruby_block 'update marketplace signup sentinel atime' do
  block do
    FileUtils.touch(node['chef-manage']['marketplace']['sentinel']['file'])
  end

  only_if { node['chef-manage']['marketplace']['sign_up']['enabled'] }
  action :run
end
