#
# Cookbook:: private_chef_addon
# Resource:: default
#
# Copyright:: Copyright (c) 2013-2015 Chef Software, Inc.
# License:: Apache License, Version 2.0
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
unified_mode true

action :create do
  load_private_chef_attributes
  create_base_directories
  create_user
  create_log_directories
end

action_class do
  def create_base_directories
    %w( install var config tmp log ).each do |type|
      directory node[new_resource.name]["#{type}_path"] do
        owner 'root'
        group 'root'
        mode '0775'
      end
    end
  end

  def create_log_directories
    services.each do |name, _options|
      directory ::File.join(node[new_resource.name]['log_path'], name) do
        user node[new_resource.name]['user']['username']
        mode '0700'
        recursive true
      end
    end
  end

  # Set user attributes and create user
  def create_user
    node.default[new_resource.name]['user']['username'] = 'opscode'
    node.default[new_resource.name]['user']['shell'] = '/usr/sbin/nologin'
    node.default[new_resource.name]['user']['home'] = '/opt/opscode/embedded'

    user node[new_resource.name]['user']['username'] do
      system true
      shell node[new_resource.name]['user']['shell']
      home node[new_resource.name]['user']['home']
    end

    group node[new_resource.name]['user']['username'] do
      members [node[new_resource.name]['user']['username']]
    end
  end

  def load_private_chef_attributes
    private_chef = if ::File.exist?('/etc/opscode/chef-server-running.json')
                     JSON.parse(IO.read('/etc/opscode/chef-server-running.json'))
                   # Fall back to an empty hash if the file does not exist
                   else
                     { 'private_chef' => {} }
                   end
    node.consume_attributes({ 'private_chef' => private_chef['private_chef'] })
  end

  def services
    node[new_resource.name]['services']
  end
end
