# Chef Management Console

* [Develop](#getting-started-with-development)
* [Build](#building-omnibus-packages)
* [Release](../../doc/releasing.md)
* [Deploy](../../doc/deploy.md)

## Getting started with development

**Note** These docs assume you are under the src/chef-manage directory. The paths here are treating src/chef-manage as the base.

This will run Chef Management Console a.k.a Chef Manage, a.k.a Manage, locally on your workstation in development mode.

If you only want to run Chef Manage to view and use it and not for development work, it is easier to install it in a dev-vm and access it that way.

These directions assume you are running these commands from the top level directory of this repository.

### Dependencies

* Ruby 2.6.6
* As of 3/2021 and thin 1.7.2 you'll need to set the following bundler config on macOS 11 and later `bundle config build.thin --with-cflags="-Wno-error=implicit-function-declaration"`
* A development VM
  * DVM (Preferred)

     The [chef-server repository](https://github.com/chef/chef-server) now includes a local development environment for running Chef server code in development mode. It is called [dvm](https://github.com/chef/chef-server/tree/main/dev). This can be used to run a Chef server to run Manage against in development mode. dvm is replacing [dev-vm](https://github.com/chef/opscode-dev-vm), but may not yet have all the features of dev-vm.

     To use dvm to run Manage, get a working dvm set up by following the [readme in the chef-server repo under the dev directory](https://github.com/chef/chef-server/blob/main/dev/README.md).

     Once you have dvm up and running, you need to get few values from the private-chef-secrets.json of the Chef server and into your chef Manage repo. Follow these steps for that:

       * Log onto the dvm: ```vagrant ssh```
       * Switch to the root user: ```sudo -i```
       * Copy the private-chef-secrets.json to the /vagrant directory: ```cp /etc/opscode/private-chef-secrets.json /vagrant```
       * Now copy the values of `secret` and `salt` from `chef-server/dev/private-chef-secrets.json` (put there by Vagrant) and update them in the chef-manage repo in the file: `chef-manage/src/chef-manage/dev/private-chef-secrets.json`
       * You need to get the private key from the file: `webui_priv.pem` under directory `/etc/opscode/`. Copy the content and replace the newlines with `\n`. You will need to add it in your chef-manage repo as mentioned in the subsequent step.
       * Log off the dvm by typing ```exit``` twice (once to exit the root user, and again to exit the ssh session).
       * Edit the `private-chef-secrets.json` file under chef-manage and update the value for `credentials -> chef-server -> webui_key -> value` as the private key you copied from the vagrant box in the previous step.
       * It looks something like this:
           ```
           "credentials": {
             "chef-server": {
               "webui_key": {
                 "type": "Veil::Credential",
                 "name": "webui_key",
                 "group": "chef-server",
                 "value": "<< WEBUI_PRIVATE_KEY goes here with newlines replaced with \n >>",
                 "version": 0,
                 "length": false,
                 "frozen": true
               }
             }
           }
           ```
       * Also make sure you change the value of `length` key from any fixed number to `false`.


    You will then need to point Manage at the dvm server, which operates on a different IP by default from dev-vm. At the time of this writing, that IP is 192.168.33.100

    Put these lines into the config/settings.local.yml file in your chef-manage repository (create the file if it doesn't exist):

     ```
     platform:
       url: https://192.168.33.100/
     ```

    You will also need redis running. It is probably easiest to run redis locally. See below in the redis section for how to do so. If you want you could potentially use the redis in the dvm by following the instructions for using the dev-vm redis and adapting any dev-vm instructions to be dvm specific instead.

     dvm does not currently preload any users, so once you have Manage up and running you'll have to sign up a new user to have a test user if you haven't already used `chef-server-ctl user-create` to make one.

* Redis

    Redis is required, as it is used by the session store, background job service (sidekiq), and the event service. There are several ways you can get a running redis server to use:

    * Run redis locally. Install redis via Homebrew: ```brew install redis```  Once it is installed, you can open a new terminal window or tab and type ```redis-server```. If the output looks okay, you're good to proceed in another window or tab. Simply hit ctrl-c in the window or tab where redis is running to stop it. Or you can use the commands that homebrew recommends when you installed redis, or you can alternatetly do ```brew tap homebrew/services``` and ```brew services start redis```.

    * (Deprecated) Expose the redis server running inside your dev-vm instance. SSH into the dev-vm, open ```/etc/opscode/chef-server.rb``` and add ```redis_lb['bind'] = '0.0.0.0'```. Then run ```sudo chef-server-ctl reconfigure``` to write out the new config. You will have edit ```config/settings.local.yml``` and set something like:

          ```yaml
          redis:
            url: redis://api.opscode.piab:16379/0
          ```

          If this fails and when you start Manage you get an error about failing to connect to redis, then on the dev-vm, open up /etc/opscode/chef-server-running.json and search for redis. Check the port that redis is running on and ensure that it is set to the same value in your Manage config. If everything is set properly and Manage still fails to connect to redis, then it is probably best to fall back to running redis locally. While it shouldn't be needed, because redis was set to bind to 0.0.0.0 by the change above, the Chef server does use a password for connecting to redis under normal circumstances. You could try setting this by setting ```redis_lb['password'] = passwordyouwant``` in the ```/etc/opscode/chef-server.rb``` file on the dev-vm and then running ```sudo chef-server-ctl reconfigure``` on the dev-vm to set it. Then add a line with ```password: 'passwordyouwant'``` to the redis section of ```config/settings.local.yml``` for Manage that you modified earlier. Restart Manage and see if this works to connect to the dev-vm redis server. If this still fails, the options at this point are debug further or fall back to using a local redis.

        Note that the name of the config file, ctl command, and redis may all be different if you're running a version of Enterprise Chef before 12 (```private-chef.rb```, ```opscode-chef-ctl```, and possibly ```redis['bind']``` instead of ```redis_lb['bind']```).

* Node.js

    Node.js is used for the building of assets used by Chef Manage and for running the javascript tests. It is not a requirment to actually run Chef Manage, but you'll want it installed for doing development work on Chef Manage.

    Node can be installed via Homebrew with ```brew install node``` or if you prefer you can use a node version manager, like [nvm](https://github.com/creationix/nvm).

## Run Chef Manage in Development Mode

Ensure that all the dependencies described above are installed, that your development VM is running, and that you've copied over the private-chef-secrets.json file as described above.

From your chef-manage checkout, run:

  ```bin/rails server```

Load up ```http://localhost:3000/``` and you should have a running Manage instance. If you get an error about failing to connect to Redis, then you likely do not have Redis running locally.

If you get errors during sign up or login that Manage is having trouble contacting the server or you see it spinning unable to verify if your username or other information that you input is valid, then you likely either input the platform url wrong or failed to copy over the `private-chef-secrets.json`. Check those and try again before assuming you have a bug. A 401 error indicates the `pivotal` user and credentials


### Cookbooks development

Cookbooks used by Manage are in the cookbooks directory:

* chef-manage: Used for building packages and other useful development and
    deployment tasks.
* omnibus-chef-manage: The cookbook that is used inside of the Omnibus packages.
    Its default recipe is run when you do `chef-manage-ctl reconfigure`.

Each of these cookbooks has its own README test and development environments.

## Configuration

The app uses [RailsConfig](https://github.com/railsjedi/rails_config) for its configuration settings. The default configuration for all environments is in config/settings.yml. You can override any of these settings in config/settings.local.yml. When the app is deployed to preprod, prod, or as a package, config/settings/production.yml will be written to override the settings for production use.

## Browser Support

We officially support these browsers and versions:

* Internet Explorer >= 9
* Firefox >= 20
* Chrome >= 26
* Safari >= 6

There is a warning modal that pops up when you have an unsupported version. The versions listed are [specified in this file](https://github.com/opscode/chef-manage/blob/main/app/assets/javascripts/views/application/modals/browser_version.coffee).

## Development Tasks

### Build Process

The build process will transform things like scripts and templates from source
to the artifacts that will be used when running. Things created must be
committed along with the source changes in order for everything to work.

Ensure the build and test dependencies are installed:

```npm install```

To run the build:

```npm run build```

To watch and build files continuously:

```npm run watch-build```

To watch both the build and the tests run:

```npm run watch```

## Testing

### Ruby

To run the Ruby tests:

```./bin/rake spec```

### JavaScript

To run the JavaScript tests:

```./bin/rake spec:javascripts``` to run all the tests once
```./bin/rake spec:javascripts:watch``` to run the test server and watch for file changes

**Note** that it is javascripts with an s, not javascript.

## MailCatcher

[MailCatcher](http://mailcatcher.me/) is a useful tool that can  be used to intercept and inspect email sent from the app in development mode. It is installed when running bundle install, so it ready for use. Run ```bundle exec mailcatcher``` and go to http://localhost:1080 to see messages. If you experience problems with rails hanging on sent mail, running mailcatcher not as a background process with ```bundle exec mailcatcher -f``` may fix them.

## Loading Sample Data

There are rake tasks load to randomly generated data for resources. The ordering is important for most of these (roles use cookbooks, nodes' run lists use roles, etc.) so run them in the order provided here. `rake -T chef` for a full list, including optional arguments.

In this example the user is `applejack` and the organization is `ponyville`. The 3rd argument to the cookbooks task is `user\_key`, which can be found in the user's .chef directory within the users directory of the [opscode-dev-vm](https://github.com/opscode/opscode-dev-vm)

```
rake chef:data:cookbooks[applejack,ponyville,../opscode-dev-vm/users/applejack/.chef/applejack.pem]
rake chef:data:roles[applejack,ponyville]
rake chef:data:nodes[applejack,ponyville]
rake chef:data:data_bags[applejack,ponyville]
rake chef:data:data_bag_items[applejack,ponyville]
```

Run the tasks above (if you have a working dev-vm). Those resources will now be available.

## Reporting

Note: These directions currently assume you're using dev-vm and have not been updated for working with dvm.

If you need to work with reporting, the best way to set it up is to let the dev-vm set it up for you when you start it.

First download the reporting package you need from either the [reporting section of the download site](https://downloads.chef.io/reporting/) or download a nightly/development package from CI.

Copy the downloaded package to your dev-vm directory. Ensure that the following environment variable is set:

OC\_REPORTING\_INSTALLER=\<path to the reporting package\>

Start the dev-vm as normal. In your local Chef Manage checkout, in the config directory, create the dark_launch_features.json if it does not already exist. Add ```{ "reporting": true }``` to the file, if it is not already present. Refresh Chef Manage in your browser. You should now see the Reports tab.

If you need to generate reporting data, use the [dev-vm-chef-client](https://github.com/chef/dev-vm-chef-client) repo to setup a client that can run chef-client runs against your dev-vm. This will generate reporting data in Chef Manage, if reporting is enabled.

 It may also be possible to use basho_bench to generate large amounts of data. The reporting basho_bench setup is found [here](https://github.com/chef/oc_reporting/tree/main/bench). Follow the readme there; however, be aware that the project isn't always actively maintained and might have bit rotted.

## Building Omnibus Packages

[Omnibus](https://github.com/chef/omnibus) packages are automatically built from
the latest Git main on our
[internal Jenkins infrastructure](https://wilson.ci.opscode.us) (requires VPN
connection.)

To build a package locally you can use the [Test Kitchen](http://kitchen.ci/)
build-\* suites. From the cookbooks/chef-manage directory:

```shell
kitchen converge build-ubuntu-1004 --log-level=debug
kitchen converge build-centos-510 --log-level=debug
```

These will use [Vagrant](https://www.vagrantup.com/) virtual machines to build
packages which will end up in the pkg/ directory.

*NOTE: If you experience unexpected behavior with Bundler gems after doing a
build, `rm -rf .bundle/config vendor/bundle`. The build process bundles
different RubyGems for different platforms and could cause confusion.*

See the [build documentation](../../doc/build.md) for more information on building
packages.

## Interactive ClojureScript development

We're experimenting with endpoints of the form
`/organizations/:organization_id/search/` using some new code in ClojureScript
using the reagent wrapper around react. There is support for live
reloading when you edit some source like
`chef-manage/cljs/src/cljs/reagent_manage/core.cljs`.

To enable it, add `cljs_dev: true` to your
`config/settings/development.local.yml` (creating that file if necessary) and run

```shell
cd cljs
rlwrap lein figwheel
```

and this will generate `chef-manage/public/js/reagent-manage-dev.js`, after which
`/organizations/:organization_id/search/` should show some interactive widgets.

For production, make sure `cljs_dev: false` is in your `config/settings/development.local.yml`, and run
```shell
cd cljs
lein deploy
```

to generate `chef-manage/public/js/reagent-manage.js`. This file is
tracked by git and should be checked in when making changes to
clojurescript source.

To run unit tests, first install PhantomJS by running:

```shell
brew install phantomjs
```

and then run:

```shell
lein cljsbuild test
```

Tests can also be run continuously in the browser directly by running

```shell
rlwrap lein figwheel-test
```

instead of the usual `lein figwheel`. Whether tests are passing or not can be
viewed at `http://localhost:3449/`, and live-update the same as the rest of the code.

To get a cljs repl up with node, make sure you have node.js installed and in
your path, and then run:

```shell
rlwrap lein repl
```

from the cljs sub-directory. Once inside the repl it is necessary to
explicitly require project code for it to be visible, e.g.

```clojure
(require 'reagent-manage.core)
```

or

```clojure
(require '[reagent-manage.core :as manage])
```

### Inserting Welcome Panel Using Cljs

When you first navigate to the new search interface, you are greeted by a welcome panel that briefly explains the new search feature built in clojurescript. Since using clojurescript is new in manage, here is a short description of how to insert a simple welcome panel using cljs.

The code for the welcome panel is located in:

`chef-manage/cljs/src/cljs.reagent_manage/search_interface.js.`

Approximately at the end of the file there is a function `defn form`. In that function, there is a convenience function called panels/perma-hide which modifies the state of the web application to hide the panel. The state includes the idea of a welcome message and whether or not it is visible. That function is wrapped in a conditional. If the state of the welcome message is true or visible, then show the div that is below. If not, then show an empty span.

## Using the Docker Development Environment

The first step is to make sure you have Docker and Docker Compose installed. To do so, please follow
the instructions [here](https://docs.docker.com/compose/install/).

## Integrating Eloqua marketing tool with the application
Chef manage has the option to post the lead to form-<form_id>, for example 902, as and when there is a sign up on application.\
Configuration changes which are required to submit data to eloqua on user sign up are :

1. Settings.yml  -\
  enabled: true
  host: "https://****.****.eloqua.com"\
  url: "https://****.****.eloqua.com/api/REST/2.0/data/form/"\
  username: "****"\
  password: "****"\
  form_id: "<form_id>"

2. eloqua.env  -\
    ELQ_URL= <URL as mentione in Settings.yml/host">\
    ELQ_FORM_ID=902\
    ELQ_USER= <user as mentioned in settings.yml/username>\
    ELQ_PWD= <password as mentioned in settings.yml/password>\
    ELQ_CUSTOMER_GUID= <field_id as per form_structure>\
    EMAIL= <field_id as per form_structure>\
    FNAME= <field_id as per form_structure>\
    LNAME= <field_id as per form_structure>\
    COMPANY_NAME= <field_id as per form_structure>\
    ELEC_MSG_OPT_OUT= <field_id as per form_structure>\
    PRODUCT= <field_id as per form_structure>\
    SFDCCampaignID= <field_id as per form_structure>\
    SFDCCampaignStatus= <field_id as per form_structure>\
    LEAD_SOURCE= <field_id as per form_structure>\
    LEAD_SOURCE_DETAIL= <field_id as per form_structure>\
    OFFER_TITLE= <field_id as per form_structure>\
    ELQ_CUSTOMER_GUID= <field_id as per form_structure>\
    PROD_INTEREST= <field_id as per form_structure>\
    ACCEPTED_TERMS= <field_id as per form_structure>\

    Its important to mention all the field ids properly otherwise Form at eloqua end will not identify the value passed along.\
    For example Email Field id in form-902 is 21326.

    We also need to mention some default values for those are mandatory for a succesfull form submission, please note values mentioned below are only for example purposes and will vary depending on final form structure by eloqua :

    COMPANY_NAME_DEFAULT= ABCD TEST\
    ELEC_MSG_OPT_OUT_DEFAULT= 0\
    PRODUCT_DEFAULT= CHEF\
    SFDCCampaignID_DEFAULT= 7022E000001Frv6QAD\
    SFDCCampaignStatus_DEFAULT= Responded\
    LEAD_SOURCE_DEFAULT= Hosted Chef Trial\
    LEAD_SOURCE_TITLE_DEFAULT= Trial - SaaS\
    OFFER_TITLE_DEFAULT= Registered for Hosted Chef Trial\
    ELQ_CUSTOMER_GUID_DEFAULT= 00000000-0000-0000-0000-000000000000

  These changes are important to make sure all the fields are properly populated and sent to eloqua.

### Limitations
This development environment is in heavy beta, and has the following limitations:
1. Does not currently support Clojure development
2. Limitations in Docker for Mac Beta make it extremely slow.
3. Has not been tested thoroughly.

### Usage

Run the following command:

    docker-compose up -d

This command will bring up the Redis and Chef Server in the background. To stop these services, run:

    docker-compose stop

If you'd like to view the logs for these services, run:

    docker-compose logs -f

Then, in your `settings.local.yml`, use these values:

    redis:
      url: redis://127.0.0.1:16379/0


Once you've done that, you can run `bin/rails server` as you normally would.


## Additional Documentation

* [The Ark Framework](../../doc/ark.md)
* [Events Service](../../doc/events.md)
* [Pattern Library](../../doc/pattern-library.md)
* [Using an AD/LDAP Server](../../doc/ldap.md)

## Big TODOs

### Bulk APIs

Currently, we are using search for pagination. This has a side effect of exposing a giant security hole in that search is not protected under permissions. This means, users without "Read" access to an object can view its contents without permission.

We want to be able to prevent users from clicking on objects they do not have permissions on in the list view to avoid having to handle a butt-load of 403s in every widget in every object pane in the app.

To this end, we added a hack into rails controller ```ChefSolrSearchableObjectController::index``` method that queries every object (as the non-solr searchable endpoint does) and catches the 403 in the same way as ```ChefObjectController```. Once there are bulk API endpoints, we should not even need ```ChefSolrSearchableObjectController``` and should be able to remove this hack.

We would also wanna remove the "_id" convention from the base backbone model, as we would now be able to have the name of the object from the bulk API.
