define (require) ->
  Backbone = require('backbone')
  _        = require('underscore')

  require('utils/backbone_validation')

  helpSpan = ($el) ->
    mode = if $el.is(':checkbox')
      'block'
    else
      'inline'
    $label = $el.parents('label')
    $base = if $label.length
      # Put the help span outside the label
      $label
    else
      $el
    $help = $base.nextAll(".help-#{mode}")
    unless $help.length
      $help = $base.after("<span class=\"help-#{mode}\"></span>").nextAll(".help-#{mode}")
    $help

  _.extend Backbone.Validation.callbacks,
      # Gets called at the start of validation
      validating: (view, attr) ->
        $el = @reverseBindings[attr]
        $help = helpSpan($el)
        unless $help.find('.fa-spinner').length
          $help.prepend('<i class="fa fa-spinner fa-spin fa-large"></i>')

      valid: (view, attr, selector) ->
        $el = @reverseBindings[attr]
        return unless $el # Don't error on valid fields that aren't part of this form
        $help = helpSpan($el)
        $help.find('.fa-spinner,.error-message').remove()
        $el.parents('.control-group').removeClass('error')

      invalid: (view, attr, error, selector) ->
        $el = @reverseBindings[attr]
        $help = helpSpan($el)
        $help.find('.fa-spinner').remove()
        $message = $help.find('.error-message')
        unless $message.length
          $message = $help.prepend('<span class="error-message"></span>').find('.error-message')
        $message.text(error)
        $el.parents('.control-group').addClass('error')

  Backbone.View::bindValidation = ->
    # Copy validation config from the view if present
    @model.validation ?= @validation
    # Set up model binding
    @initializing = true
    @stickit()
    @initializing = false
    # Set up validation
    Backbone.Validation.bind(@)
    # On change, validate. We track changed attrs ourselves because the debounce
    # means model.changed will usually be empty by the time validation happens.
    changed = {}
    onChangeInner = (model) ->
      model.validate(changed)
      for key of changed
        delete changed[key]
    onChangeInner = _.debounce(onChangeInner, 200)
    onChange = (model) ->
      _.extend(changed, model.changed)
      onChangeInner(model)
    @model.on('change', onChange, @)

    throw Error(
      "The view must have a bindings object property to bind validation"
    ) unless @bindings
    @$(_.keys(@bindings).join(',')).change()
