define (require) ->
  Backbone = require('b')
  Paginator = require('backbone.paginator')
  SlickBack = require('slickback')
  _ = require('underscore')
  aclHelper = require('helpers/acl')
  config = require('config')
  urlsMixin = require('collections/mixin/urls')

  class BasePaginator extends Paginator.requestPager
    _.extend(@prototype, SlickBack.Collection.prototype)
    _.extend(@prototype, aclHelper)
    _.extend(@prototype, urlsMixin)

    searchFilter: '*:*'  # default Solr query string
    searchSortOrder: null # default sort order
        # Other nontrivial sort orders are entailed by string values
        # like "name", "-name" (descending), "ipaddress",
        # "-ipaddress", etc.

    reachedEnd: false

    paginator_ui:
      firstPage: 0
      currentPage: 0
      perPage: 20
    paginator_core:
      # Backbone.Paginator's default for this is 25000, which, when search
      # requests take a long time, can cause the client to close the connection
      # and the request fail with an error.
      #
      # Instead of using the 25s default, use a user-supplied value from
      # configuration options or default to 120s, which has proved to be
      # sufficient in the cases where this was an issue.
      timeout: config.backbone_paginator_timeout || 120000
      dataType: 'json'
      url: -> @url()
    server_api:
      page: -> @currentPage
      count: -> @perPage
      filter: -> @searchFilter
      sort: -> @searchSortOrder

    initialize: (models, options) ->
      SlickBack.Collection::initialize.call(@, arguments)
      Paginator.requestPager::initialize.call(@, arguments)
      @organization = options?.organization
      @chefUrl ?= @webuiUrl # Default the Chef URL fragment to match the Webui one
      @queryStatus =
        resting: true
        initQuery: false
        queryingPageDown: false
        queryinyPageUp: false

    sync: (method, model, options) ->
      # Update query status for new request
      @queryStatus.resting = false
      if @length == 0
        options.queryInitial = true
        @queryStatus.initQuery = true
      # TODO: this won't work for backscroll :(((
      else
        options.queryDown = true
        @queryStatus.queryingPageDown = true
      # Setup callbacks
      success = options.success
      options.success = =>
        @queryStatus.resting = true
        @queryStatus.initQuery = false
        @queryStatus.queryingPageDown = false
        @queryStatus.queryinyPageUp = false
        success?(arguments...)
      error = options.error
      options.error = =>
        @queryStatus.resting = true
        @queryStatus.initQuery = false
        @queryStatus.queryingPageDown = false
        @queryStatus.queryinyPageUp = false
        error?(arguments...)
      # Run the XHR
      super(method, model, options)

    parse: (response) ->
      @totalPages = Math.floor(response.total / @perPage)
      # The total given in the JSON response may not be accurate in all cases.
      # If our response is less than the per-page amount, a more accurate
      # number will be the actual length of the result set, so use that.
      if response.results.length < @paginator_ui.perPage &&
         response.total != response.results.length
        @totalRecords  = response.results.length
      else
        @totalRecords  = response.total
      response.results

    urlWithQueryParams: ->
      # currently, this is the only argument that most collections have
      if @searchFilter != "*:*"
        @url() + '?filter=' + encodeURIComponent(@searchFilter)
      else
        @url()

    # Interface with slickgrid for the purposes of spinner magic
    getItemMetadata: (row) ->
      model = @at(row)
      if not model.get('isSpinner')
        if not model.get('forbidden')
          return null
        else
          {
            selectable: false,
            focusable: false,
            cssClasses: 'forbidden-row muted'
          }
      else
        {
          columns: {
            0: {
              colspan: '*',
              formatter: (row, cell, value, columnDef, dataContext) -> ''
              selectable: false,
            },
          },
          cssClasses: 'slick-spinner-row',
          selectable: false,
          focusable: false
        }
