define (require) ->
  Accordion = require('views/controls/accordion')
  Backbone = require('b')
  DeleteModal = require('views/controls/modals/delete')
  ObjectTabs = require('views/application/object_tabs')
  SidebarSection = require('views/application/sidebar/section')
  _ = require('underscore')
  i18n = require('utils/i18n')
  router = require('router')

  class BaseController extends Backbone.Layout
    # Note: by convention, subclasses that are using actions should have
    # an endpoint field equal to the chef API endpoint it wants to hit AND
    # a collection string equal to collection type class. For example,
    # if you want to have a corntroller for environments that deletes extend this
    # class and set @id = environments, that way your delete endpoint will look like
    # "#{@endpoint}/#{environment_id} and send a delete message on @collection_type
    #
    # The reason these are different is the endpoint is "data" for data_bags
    # Override these
    endpoint: 'base'
    collection_type: 'base'
    sidebarSectionClass: SidebarSection
    sidebarViewClass: Accordion
    listViewClass: Backbone.LayoutView
    objectViewClass: ObjectTabs
    deleteModalClass: DeleteModal
    messages:
      label: (n) -> i18n.ngettext('Object', 'Objects', n ? 1)
      selectEmpty: -> i18n.gettext('Please select an object')
      selectMulti: -> i18n.gettext('%d objects selected')
      searchMessage: i18n.gettext("Search Objects")
      showingNObjects: (n) -> i18n.sprintf(i18n.ngettext('Showing <strong>%d object</strong>', 'Showing <strong>%d objects</strong>', n ? 1), n)  # embedded so as to make string localizable
      showingAllObjects: -> T("Showing <strong>All Objects</strong>")
      searchingObjects: -> T("Searching objects...")
    actions: []
    split_pane: true
    children: []
    wrapRoutes: true

    # Method decorator to override the controller-level wrapRoutes setting
    # Use like "route_name: @wrapRoute(true/false) -> <method body>"
    @wrapRoute: (value) -> (method) ->
      method.wrapRoutes = value
      method

    initialize: ->
      @app = @options.app
      @selection = @app?.selection || []
      @parentController = @options.parentController
      childOptions = _.clone(@options)
      childOptions.parentController = @
      @childrenInstances = for child in @children
        new child(childOptions)
      @bindRoutes()
      @sidebarView = _.once(@sidebarView)

    sidebarView: ->
      if @options.parentController?
        return @options.parentController.sidebarView()
      sections = for child in @childrenInstances
        if typeof child.checkAccess == 'function' && !child.checkAccess()
          continue
        new child.sidebarSectionClass({selection: @selection, label: child.messages.label(2), actions: child.actions, controller: child, app: @app})
      new @sidebarViewClass({sections: sections})

    getCollection: (org, params) ->
      org[@id] ? new Backbone.Collection

    url: (org) =>
      if @childrenInstances.length > 0
        for child in @childrenInstances
          if child.checkAccess(org)
            return child.url(org)
        org.url() # Fallback
      else
        org ||= @app?.organizations?.last()
        collection = @getCollection(org)
        if typeof collection.url != 'function'
          throw Error("#{collection.constructor.name} collection has no url method")
        @getCollection(org).url()

    checkAccess: (org) ->
      org ||= @app?.organizations?.last()
      if @childrenInstances.length > 0
        _.any(@childrenInstances, (child) -> child.checkAccess(org))
      else if not org
        false
      else
        collection = @getCollection(org)
        return true if !collection?.chefUrl
        org.get('access')[collection.chefUrl].read

    dispatchAction: (actionName) ->
      actionFn = @["action_#{actionName}"]
      if actionFn
        actionFn.call(@)

    bindRoutes: ->
      for route, name of _.result(@, 'routes')
        routeName = "#{@id}:#{name}"
        routeFn = @["route_#{name.split('.', 1)[0]}"] # Trim anything after the first . a la jQuery event scoping
        if not routeFn?
          # No callback
          router.route(route, routeName)
        else
         if routeFn.wrapRoutes ? @wrapRoutes
            # Force a closure wrapper
            routeFn = do (routeFn) ->
              (orgName, args..., params) ->
                params = {} if not params
                org = @app.organizations.getCurrent(orgName)
                unless @checkAccess(org)
                  return @app.rootRoute()
                @collection = @getCollection(org, params)
                @collection.params = params

                @selection.reset(routeFn.apply(@, arguments) ? [], {silent: true})
                @app.applyController(@)
                @selection.trigger('reset navigate')
          router.route(route, routeName, _.bind(routeFn, @))

    # Default route callbacks
    route_index: (org, params) ->
      []

    route_show: (org, id, tabId, params) ->
      @currentTabId = tabId
      model = @collection.get(id)
      if model? then [model] else []

    # Actions that occur in every class
    action_delete: ->
      new @deleteModalClass({collection: @selection, app: @app, messages: @messages, controller: @, endpoint: @endpoint, collection_type: @collection_type})
