define (require) ->
  BaseController = require('controllers/base')
  ClientListView = require('views/clients/list')
  ClientObjectTabsView = require('views/clients/object_tabs')
  ClientDetailsView = require('views/clients/details')
  ClientDeleteModal = require('views/clients/modals/delete')
  T = require('utils/i18n.gettext')
  i18n = require('utils/i18n')
  ClientCreateModal = require('views/clients/modals/create')
  ExtremeWarningModal = require('views/controls/modals/extreme_warning_modal')
  PermissionsView = require('views/controls/permissions')

  class ClientsController extends BaseController
    id: 'clients'
    endpoint: 'clients'
    collection_type: 'clients'
    listViewClass: ClientListView
    objectViewClass: ClientObjectTabsView
    deleteModalClass: ClientDeleteModal
    messages:
      label: (n) -> i18n.ngettext('Client', 'Clients', n ? 1)
      selectEmpty: -> T('Please select a client')
      selectMulti: (n) -> T(i18n.sprintf('%d clients selected', n))
      delete: (n) -> i18n.ngettext('Delete Client', 'Delete Clients', n ? 1)
      deleteSingle: T('Deleting %s')
      deleteLong: (n) -> i18n.ngettext('You are about to delete %d client:', 'You are about to delete %d clients:', n ? 1)
      searchMessage: T("Search Clients")
      showingNObjects: (n) -> i18n.sprintf(i18n.ngettext('Showing <strong>%d client</strong>', 'Showing <strong>%d clients</strong>', n ? 1), n)
      showingAllObjects: -> T("Showing <strong>All Clients</strong>")
      searchingObjects: -> T("Searching clients...")

    routes:
      'organizations/:org/clients': 'index'
      'organizations/:org/clients/:id': 'show'
      'organizations/:org/clients/:id/:tabId': 'show'

    tabs: [
      {id: 'details', label: T('Details'), viewClass: ClientDetailsView},
      {id: 'permissions', label: T('Permissions'), viewClass: PermissionsView},
    ]
    actions: [
      {id: 'create', label: T('Create'), always: true, dropdown: false},
      {id: 'delete', label: T('Delete'), multi: true}
    ]

    action_create: ->
      new ClientCreateModal({ app: @app, controller: @ })
