define (require) ->
  Backbone            = require('b')
  BaseController      = require('controllers/base')
  CookbookListView    = require('views/cookbooks/list')
  CookbookDetailsView = require('views/cookbooks/details')
  CookbookContentView = require('views/cookbooks/content')
  PermissionsView     = require('views/controls/permissions')
  DeleteModal         = require('views/controls/modals/delete')
  T                   = require('utils/i18n.gettext')
  i18n                = require('utils/i18n')
  CookbookObjectTabs  = require('views/cookbooks/object_tabs')

  class CookbooksController extends BaseController
    id: 'cookbooks'
    endpoint: 'cookbooks'
    collection_type: 'cookbooks'
    objectViewClass: CookbookObjectTabs
    messages:
      label: (n) -> i18n.ngettext('Cookbook', 'Cookbooks', n ? 1)
      selectEmpty: -> T('Please select a cookbook')
      selectMulti: -> T('%d cookbooks selected')
      delete: (n) -> i18n.ngettext('Deleting %d cookbook', 'Deleting %d cookbooks', n ? 1)
      deleteSingle: T('Deleting %s')
      deleteLong: (n) -> i18n.ngettext('You are about to delete %d cookbook:', 'You are about to delete %d cookbooks:', n ? 1)
      showingAllObjects: -> T("Showing <strong>All Cookbooks</strong>")

    listViewClass: CookbookListView
    tabs: [
        {id: 'details', label: T('Details'), viewClass: CookbookDetailsView},
        {id: 'content', label: T('Content'), viewClass: CookbookContentView},
        {id: 'permissions', label: T('Permissions'), viewClass: PermissionsView},
    ]

    routes:
      'organizations/:org/cookbooks': 'index'
      'organizations/:org/cookbooks/:id': 'showVersion'
      'organizations/:org/cookbooks/:id/:version': 'showVersion'
      'organizations/:org/cookbooks/:id/:version/:tabId': 'showVersion.tab'

    route_showVersion: (org, id, version, tabId) ->
      @currentTabId = tabId
      cookbook = @collection.get(id)
      if cookbook?
        version ||= cookbook.get('current_version')
        cookbook.set('displayVersion', version)
        cookbook.getCookbookVersion(version).fetch()
        [cookbook]
      else
        []
