define (require) ->
  BaseController = require('controllers/base')
  Cookbooks = require('collections/cookbooks')
  CreateEnvironmentModal = require('views/environments/modals/create')
  Environment = require('models/environment')
  EnvironmentAttributesView = require('views/environments/attributes')
  EnvironmentDetailsView = require('views/environments/details')
  EnvironmentListView = require('views/environments/list')
  PermissionsView = require('views/controls/permissions')
  T = require('utils/i18n.gettext')
  i18n = require('utils/i18n')

  class EnvironmentsController extends BaseController
    id: 'environments'
    endpoint: 'environments'
    collection_type: 'environments'
    messages:
      label: (n) -> i18n.ngettext('Environment', 'Environments', n ? 1)
      selectEmpty: -> T('Please select an environment')
      selectMulti: (n) -> i18n.sprintf(T('%d environments selected'), n)
      delete: (n) -> i18n.ngettext('Delete Environment', 'Delete Environments', n ? 1)
      deleteSingle: T('Deleting %s')
      deleteLong: (n) -> i18n.ngettext('You are about to delete %d environment:', 'You are about to delete %d environments:', n ? 1)
      searchMessage: T("Search Environments")
      showingNObjects: (n) -> i18n.sprintf(i18n.ngettext('Showing <strong>%d environment</strong>', 'Showing <strong>%d environments</strong>', n ? 1), n)
      showingAllObjects: -> T("Showing <strong>All Environments</strong>")
      searchingObjects: -> T("Searching environments...")

    listViewClass: EnvironmentListView
    tabs: [
      {id: 'details', label: T('Details'), viewClass: EnvironmentDetailsView},
      {id: 'attributes', label: T('Attributes'), viewClass: EnvironmentAttributesView, viewArgs: { controller: @ }},
      {id: 'permissions', label: T('Permissions'), viewClass: PermissionsView},
    ]
    actions: [
      {id: 'create', label: T('Create'), always: true, dropdown: false},
      {id: 'delete', label: T('Delete'), multi: true},
    ]

    routes:
      'organizations/:org/environments': 'index'
      'organizations/:org/environments/:id': 'show'
      'organizations/:org/environments/:id/:tabId': 'show.tab'

    action_create: ->
      new CreateEnvironmentModal({
        app: @app,
        model: new Environment(),
        cookbooks: new Cookbooks([], { organization: @app.currentOrg })
      })
