define (require) ->
  BaseController       = require('controllers/base')
  GroupCreateModalView = require('views/groups/modals/create')
  GroupListView        = require('views/groups/list')
  GroupMembersView     = require('views/groups/members')
  GroupPermissionsView = require('views/groups/permissions')
  T                    = require('utils/i18n.gettext')
  i18n                 = require('utils/i18n')

  class GroupsController extends BaseController
    id: 'groups'
    endpoint: 'groups'
    collection_type: 'groups'
    messages:
      label: (n) -> i18n.ngettext('Group', 'Groups', n ? 1)
      selectEmpty: -> T('Please select a group')
      selectMulti: (n) -> i18n.sprintf(T('%d groups selected'), n)
      searchMessage: T("Search Groups")
      showingAllObjects: -> T("Showing <strong>All Groups</strong>")
      delete: (n) -> i18n.ngettext('Delete Group', 'Delete Groups', n ? 1)
      deleteSingle: T('Deleting %s')
      deleteLong: (n) -> i18n.ngettext('You are about to delete %d group:', 'You are about to delete %d groups:', n ? 1)

    listViewClass: GroupListView

    tabs: [
      {id: 'members', label: T('Members'), viewClass: GroupMembersView},
      {id: 'group_permissions', label: T('Permissions'), viewClass: GroupPermissionsView},
    ]

    actions: [
      {id: 'create', label: T('Create'), always: true, dropdown: false},
      {
        id: 'delete'
        label: T('Delete')
        multi: true
        disabled_when: =>
          systemGroups = ['admins', 'billing-admins', 'clients', 'users']
          selectedGroups = window.app.selection.pluck('name')
          _.any(selectedGroups, (group) => _.contains(systemGroups, group))
        disabled_message: 'System groups cannot be deleted'
      },
    ]

    routes:
      'organizations/:org/groups': 'index'
      'organizations/:org/groups/:id': 'show'
      'organizations/:org/groups/:id/:tabId': 'show.tab'

    action_create: ->
      new GroupCreateModalView(app: @app)
