define (require) ->
  AttributesModal = require('views/controls/modals/attributes')
  AttributesView = require('views/controls/attributes')
  Backbone = require('b')
  BaseController = require('controllers/base')
  Client = require('models/client')
  EditRunListModal = require('views/controls/modals/edit_run_list')
  ManageTagsModal = require('views/nodes/modals/manage_tags')
  NodeDeleteModal = require('views/nodes/modals/delete')
  NodeDetailsView = require('views/nodes/details')
  NodeListView = require('views/nodes/list')
  PermissionsView = require('views/controls/permissions')
  RekeyModal = require('views/controls/modals/rekey')
  T = require('utils/i18n.gettext')
  i18n = require('utils/i18n')

  class NodesController extends BaseController
    id: 'nodes'
    endpoint: 'nodes'
    collection_type: 'nodes'
    messages:
      editRunList: T('Edit Node Run List')
      label: (n) -> i18n.ngettext('Node', 'Nodes', n ? 1)
      selectEmpty: -> T('Please select a node')
      selectMulti: (n) -> T(i18n.sprintf('%d nodes selected', n))
      # Bleh, we need a better solution since this is unreadable eventually. Subclassing the relevant modals?
      delete: (n) -> i18n.ngettext('Delete Node', 'Delete Nodes', n ? 1)
      deleteSingle: T('Deleting %s')
      deleteLong: (n) -> i18n.ngettext('You are about to delete %d node:', 'You are about to delete %d nodes:', n ? 1)
      searchMessage: T("Search Nodes")
      showingNObjects: (n) -> i18n.sprintf(i18n.ngettext('Showing <strong>%d node</strong>', 'Showing <strong>%d nodes</strong>', n ? 1), n)
      showingAllObjects: -> T("Showing <strong>All Nodes</strong>")
      searchingObjects: -> T("Searching nodes...")

    listViewClass: NodeListView
    deleteModalClass: NodeDeleteModal

    tabs: [
      {id: 'details', label: T('Details'), viewClass: NodeDetailsView},
      {id: 'attributes', label: T('Attributes'), viewClass: AttributesView, viewArgs: {editRootAttribute: 'normal', objectType: 'Node' }},
      {id: 'permissions', label: T('Permissions'), viewClass: PermissionsView, condition: (model) -> model.get('access')?.grant},
    ]

    actions: [
      {id: 'delete', label: T('Delete'), multi: true, access: 'delete'},
      {id: 'manageTags', label: T('Manage Tags'), multi: true},
      {id: 'rekey', label: T('Reset Key'), access: 'update'},
      {id: 'editRunList', label: T('Edit Run List'), access: 'update'},
      {id: 'editAttributes', label: T('Edit Attributes'), access: 'update'},
    ]
    routes:
      'organizations/:org/nodes': 'index'
      'organizations/:org/nodes/:id': 'show'
      'organizations/:org/nodes/:id/:tabId': 'show.tab'

    action_manageTags: ->
      new ManageTagsModal({collection: @selection, app: @app, controller: @})

    action_editRunList: ->
      new EditRunListModal({ collection: @selection, app: @app, controller: @ })

    action_rekey: ->
      new RekeyModal({
        app: @app
        controller: @
        model: new Client(
          { name: @selection.at(0).id },
          { collection: @app.currentOrg.clients }
        )
      })

    action_editAttributes: ->
      new AttributesModal({
        app: @app
        controller: @
        model: @selection.at(0)
        title: T('Edit Node Attributes')
      })
