define (require) ->
  config = require('config')
  BaseController = require('controllers/base')
  CreateOrgModal = require('views/organizations/modals/create')
  InviteUserModal = require('views/controls/modals/invite_user')
  LeaveOrganizationModal = require('views/organizations/modals/leave')
  OrganizationListView = require('views/organizations/list')
  OrganizationMembersView = require('views/organizations/members/tab')
  RekeyModal = require('views/controls/modals/rekey')
  StarterKitListView = require('views/organizations/starter_kit_list')
  StarterKitView = require('views/organizations/starter_kit')
  T = require('utils/i18n.gettext')
  _ = require('underscore')
  i18n = require('utils/i18n')
  router = require('router')

  class OrganizationsController extends BaseController
    id: 'organizations'
    collection_type: 'organizations'
    messages:
      label: (n) -> i18n.ngettext('Organization', 'Organizations', n ? 1)
      selectEmpty: -> T('Please select an Organization')
      selectMulti: -> T('%d organizations selected')
      searchMessage: T("Search Organizations")
      showingAllObjects: -> T("Showing <strong>All Organizations</strong>")

    listViewClass: OrganizationListView
    tabs: [
      {id: 'members', label: T('Members'), viewClass: OrganizationMembersView},
    ]

    @createOrganizationAction: ->
      if config.org_creation_enabled
        {id: 'create', label: T('Create'), always: true, dropdown: false}
      else
        {id: 'create', label: T('Create'), disabled_when: (-> true), dropdown: false, disabled_message: 'Creating organizations from Chef Manage has been disabled'}

    actions: [
      @createOrganizationAction(),
      {id: 'rekey', label: T('Reset Validation Key')},
      {id: 'gen_knife_config', label: T('Generate Knife Config')},
      {id: 'invite_user', label: T('Invite User')},
      {id: 'leave_organization', label: T('Leave Organization')},
      {id: 'starterKit', label: T('Starter Kit')},
    ]
    routes:
      'organizations': 'index'
      'organizations/:id': 'show'
      'organizations/:id/details': 'details'
      'organizations/:id/members': 'members'
      'organizations/:id/getting_started': 'gettingStarted'
      'starter-kit': 'starterKit'

    getCollection: (org) ->
      @app.organizations

    checkAccess: (org) ->
      true

    action_invite_user: ->
      new InviteUserModal({model: @selection.at(0), app: @app, controller: @})

    action_create: ->
      new CreateOrgModal({app: @app})

    action_rekey: ->
      org = @selection.at(0)
      new RekeyModal({ model: org.validationClient, app: @app, controller: @ })

    action_gen_knife_config: ->
      window.location.assign(@selection.at(0).url() + "/knife_config")

    action_leave_organization: ->
      new LeaveOrganizationModal({
        model: @selection.at(0)
        app: @app
        controller: @
      })

    action_starterKit: ->
      router.navigate("#{@selection.at(0).url()}/getting_started",
        { trigger: true })

    route_starterKit: @wrapRoute(false) (orgName) ->
      @app.applyController(@, false,
        new StarterKitListView({ organizations: @app.organizations }))

    route_gettingStarted: @wrapRoute(false) (orgName) ->
      org = @app.organizations.getCurrent(orgName)
      unless @checkAccess(org)
        return @app.rootRoute()
      @app.applyController(@, false,
        new StarterKitView({ organizations: @app.organizations, app: @app }))

    route_details: (org) -> @route_show(org, 'details')

    route_members: (org) -> @route_show(org, 'members')

    route_show: (org, tabId) ->
      @currentTabId = tabId
      org = @app.organizations.get(org)
      if org? then [org] else []
