define (require) ->
  BaseController     = require('controllers/base')
  CreateRoleModal    = require('views/roles/modals/create')
  EditRunListModal   = require('views/controls/modals/edit_run_list')
  PermissionsView    = require('views/controls/permissions')
  RoleAttributesView = require('views/roles/attributes')
  RoleDetailsView    = require('views/roles/details')
  RoleListView       = require('views/roles/list')
  T                  = require('utils/i18n.gettext')
  i18n               = require('utils/i18n')

  class RolesController extends BaseController
    id: 'roles'
    endpoint: 'roles'
    collection_type: 'roles'
    messages:
      editRunList: T('Edit Role Run List')
      label: (n) -> i18n.ngettext('Role', 'Roles', n ? 1)
      selectEmpty: -> T('Please select a role')
      selectMulti: (n) -> i18n.sprintf(T('%d roles selected'), n)
      delete: (n) -> i18n.ngettext('Delete Role', 'Delete Roles', n ? 1)
      deleteSingle: T('Deleting %s')
      deleteLong: (n) -> i18n.ngettext('You are about to delete %d role:', 'You are about to delete %d roles:', n ? 1)
      searchMessage: T("Search Roles")
      showingNObjects: (n) -> i18n.sprintf(i18n.ngettext('Showing <strong>%d role</strong>', 'Showing <strong>%d roles</strong>', n ? 1), n)
      showingAllObjects: -> T("Showing <strong>All Roles</strong>")
      searchingObjects: -> T("Searching roles...")

    listViewClass: RoleListView

    tabs: [
        {id: 'details', label: T('Details'), viewClass: RoleDetailsView},
        {id: 'attributes', label: T('Attributes'), viewClass: RoleAttributesView},
        {id: 'permissions', label: T('Permissions'), viewClass: PermissionsView},
      ]

    actions: [
      {id: 'create', label: T('Create'), always: true, dropdown: false},
      {id: 'delete', label: T('Delete'), multi: true},
      {id: 'editRunList', label: T('Edit Run List'), access: 'update', dropdown: true},
    ]

    routes:
      'organizations/:org/roles': 'index'
      'organizations/:org/roles/:id': 'show'
      'organizations/:org/roles/:id/:tabId': 'show'

    action_create: ->
      new CreateRoleModal({ app: @app, controller: @ })

    action_editRunList: ->
      new EditRunListModal({ collection: @selection, app: @app, controller: @ })
