define (require) ->
  BaseController = require('controllers/base')
  RunsCollection = require('collections/runs')
  RunListView = require('views/reporting/runs/list')
  RunsDetailsView = require('views/reporting/runs/details')
  RunsErrorLogView = require('views/reporting/runs/error_log')
  RunsRunListView = require('views/reporting/runs/run_list')
  Time = require('utils/time')
  moment = require('moment')
  i18n = require('utils/i18n')
  T = require('utils/i18n.gettext')

  class RunsController extends BaseController
    id: "runs"
    messages:
      label: (n) -> i18n.ngettext('Run ID', 'Run History', n ? 1)
      selectEmpty: -> T('Please select a run')
      selectMulti: -> T('%d runs selected')
      showingAllObjects: -> T("Showing <strong>All Runs</strong>")
      searchingObjects: -> T("Searching Runs...")
      searchMessage: T("Search Runs")
      showingNObjects: (n) -> i18n.sprintf(i18n.ngettext('Showing <strong>%d run</strong>', 'Showing <strong>%d runs</strong>', n ? 1), n)
    listViewClass: RunListView
    tabs: [
      {id: "details", label: T("Details"), viewClass: RunsDetailsView},
      {id: "run_list", label: T("Run List"), viewClass: RunsRunListView},
      {id: "error_log", label: T("Error Log"), viewClass: RunsErrorLogView},
    ]

    checkAccess: (org) ->
      org ||= @app.organizations.last()
      org.isFeatureEnabled('reporting')

    routes:
      'organizations/:org/runs': 'index'
      'organizations/:org/runs/:id': 'show.tab'
      'organizations/:org/runs/:id/:tabId': 'show.tab'

    route_show: (org, id) ->
      model = @collection.get(id)
      @collection.queryStatus.resting = false
      @collection.set_time_params(model.get('start_time').zone(0).unix(), model.get('end_time').zone(0).unix())
      if model? then [model] else []

    route_index: (org, params) ->
      @collection.cleanServerApi()

      # Default to most recent 24 hours of run history data
      defaultStartTime = moment.utc().subtract('days', 1).unix()
      defaultEndTime = moment.utc().unix()

      if params?.startTime? and params?.endTime?
        startTime = parseInt(params.startTime, 10)
        endTime = parseInt(params.endTime, 10)

        # Free billing tier only has access to most recent 24 hours of run history data
        billingTier = @app.currentOrg.reportingBillingTier()
        numDays = moment().diff(moment.unix(startTime), 'days')

        if billingTier == 'free' and numDays > 1
          startTime = defaultStartTime
          endTime = defaultEndTime
      else
        startTime = defaultStartTime
        endTime = defaultEndTime

      @collection.set_time_params(startTime, endTime)

      if params?.status?
        @collection.set_status_param(params.status)
      else
        delete @collection.server_api.status

      if params?.q?
        @collection.server_api.filter = params.q
      else
        delete @collection.server_api.filter

      if params?.env?
        @collection.server_api.env = params.env
      else
        delete @collection.server_api.env

      @collection.fetch({ reset: true })

      []

    # override
    getCollection: (org, params) ->
      collection = org.runs ? new RunsCollection
      collection
