define (require) ->
  BaseController = require('controllers/base')
  InviteUserModal = require('views/controls/modals/invite_user')
  LeaveOrganizationModal = require('views/organizations/modals/leave')
  ProfileView = require('views/users/profile')
  RekeyModal = require('views/controls/modals/rekey')
  RemoveUserModal = require('views/users/modals/remove_user_modal')
  T = require('utils/i18n.gettext')
  UserChangePasswordModal = require('views/users/modals/change_password')
  UserDetailsView = require('views/users/details')
  UserGroupsTabView = require('views/users/groups/tab')
  UserListView = require('views/users/list')
  UserPassword = require('models/user_password')
  config = require('config')
  i18n = require('utils/i18n')
  state = require('state')

  class UsersController extends BaseController
    id: 'users'
    endpoint: 'users'
    collection_type: 'users'
    messages:
      label: (n) -> i18n.ngettext('User', 'Users', n ? 1)
      selectEmpty: -> T('Please select a user')
      selectMulti: (n) -> i18n.sprintf(T('%d users selected'), n)
      searchMessage: T("Search Users")
      showingAllObjects: -> T("Showing <strong>All Users</strong>")
      delete: (n) -> i18n.ngettext('Delete User', 'Delete Users', n ? 1)
      deleteSingle: T('Deleting %s')
      deleteLong: (n) -> i18n.ngettext('You are about to delete %d user:', 'You are about to delete %d users:', n ? 1)

    listViewClass: UserListView

    tabs: [
      {id: 'details', label: T('Details'), viewClass: UserDetailsView}
    ]

    actions: [
      {id: 'invite', label: T('Invite'), always: true},
      { id: 'change_password', label: T('Change Password'), disabled_when: (-> config.external_auth?.enabled), disabled_message: "Passwords must be changed through #{config.external_auth?.adjective}" if config.external_auth?.enabled },
      {id: 'rekey', label: T('Reset Key')},
      {id: 'remove_user', label: T('Remove from Organization')}
    ]

    routes:
      'organizations/:org/users': 'index'
      'organizations/:org/users/:id(/:tab)': 'show'

    checkAccess: (org) ->
      true

    action_change_password: ->
      new UserChangePasswordModal({
        model: new UserPassword({}, { user: @selection.at(0) })
      })

    action_invite: ->
      new InviteUserModal({model: @app.currentOrg, app: @app, controller: @})

    action_rekey: -> new RekeyModal({ model: @selection.at(0) })

    action_remove_user: ->
      # if the user is trying to "remove" himself, use the leave org modal
      if state.get('user')?.id == @selection.at(0).id
        new LeaveOrganizationModal({
          app: @app
          controller: @
          model: @app.currentOrg
        })
      else
        new RemoveUserModal({
          app: @app
          controller: @
          model: @app.currentOrg
          user: @selection.at(0)
        })
