# Extends a model to have validation of an id field's name (alphanumeric, -, _),
# and existence.
define (require) ->
  _ = require('underscore')
  T = require('utils/i18n.gettext')

  # This method takes the model prototype, sets some options, and returns an
  # object with which we can extend the model.
  extensions = (modelPrototype, options) ->
    idAttribute = modelPrototype.idAttribute
    ObjectType = modelPrototype.constructor
    title = options?.title ? idAttribute

    validationOptions = {}
    validationOptions[idAttribute] = [
      { required: true, msg: T("#{title} is required") }
      {
        pattern: /^[a-zA-Z0-9\-_]+$/
        msg: T("#{title} can only contain letters, numbers, hyphens, and underscores")
      }
      { fn: 'validateObjectExists' }
    ]

    {
      validation: validationOptions
      validateObjectExists: (value, attr, computed) ->
        message = T("#{title} is already in use")
        deferred = new $.Deferred
        if @collection.get(@id)
          deferred.reject(message)
        else
          # Create a new role with the attributes being validated set and fetch to
          # see if it exists
          attrs = {}
          attrs[attr] = value
          new ObjectType(attrs, { collection: @collection }).fetch().done (response) ->
            deferred.reject(message)
          .fail (response) ->
            if (response.status == 404)
              deferred.resolve(true)
            else if (response.status == 400)
              # means the format of the input string was bad
              deferred.reject(T("#{title} is not a valid format"))
            else
              # A non-404 error will not show as already existing
              deferred.reject(message)
        deferred
    }

  (modelPrototype, options) ->
    { validation, validateObjectExists } = extensions(modelPrototype, options)
    modelPrototype.validateObjectExists = validateObjectExists
    modelPrototype.validation ?= {}
    _.extend(modelPrototype.validation, validation)
