define (require) ->
  Backbone        = require('b')
  BaseModel       = require('models/base')
  _               = require('underscore')
  addIDValidation = require('helpers/validation/id')

  class Environment extends BaseModel

    addIDValidation(@::)

    defaults:
      name: ''
      cookbook_versions: {}

    initialize: ->
      super
      @constraints = new Backbone.Collection(null, {model: Backbone.Model})
      @parseConstraints(@get('cookbook_versions'))
      @constraints.on('add remove change', @onConstraintsChange, @)

    parseConstraints: (cookbookVersions) ->
      _(cookbookVersions).forEach((constraint, name) =>
        operator = constraint.substring(0, 2).trim()

        # If we got a number for the operator, that means they meant '= ' but
        # left it off
        operator = '=' if parseInt(operator, 10) >= 0

        version = constraint.replace(operator, '')
        @constraints.add({name: name, operator: operator, version: version})
      )

    onConstraintsChange: ->
      cookbookVersions = @constraints.toJSON().reduce((memo, constraint) ->
        memo[constraint.name] = [constraint.operator,
                                 constraint.version?.trim()].join(' ')
        memo
      , {})
      @set('cookbook_versions', cookbookVersions)
