define (require) ->
  BaseModel       = require('models/base')
  SlickBack       = require('slickback')
  addIDValidation = require('helpers/validation/id')
  Users           = require('collections/users')

  class Group extends BaseModel
    idAttribute: 'groupname'

    defaults:
      groupname: ''
      clients: []
      groups: []
      users: []

    initialize: ->
      super
      @userCollection = new Users(null, {organization: @collection.organization})

    getMembersCollection: () ->
      members = new SlickBack.Collection

      for user, i in @get('users')
        # todo - join_date exposed?
        members.add({ name: user, type: "User"})

      for group, i in @get('groups')
        # todo - join_date exposed?
        members.add({ name: group, type: "Group"})

      for group, i in @get('clients')
        # todo - join_date exposed?
        members.add({ name: group, type: "Client"})

      members

    addIDValidation(@::, { title: 'name' })

    removeMember: (name, type) ->
      updateObj = {}
      members = _.reject(@get(type), (item) -> item == name)

      updateObj[type] = members
      groupname = @get('groupname')

      if ((groupname == 'admins' || groupname == 'billing-admins') && updateObj[type].length == 0)
        throw Error("Sorry, #{groupname} cannot be emptied")
      else
        @save(updateObj, {
          validate: false,
          error: (model, response, options) =>
            # this remove failed, lets add it back to the group since
            # it didn't update on the server
            members.push(name)
            @set(type, members)
            @trigger('remove_error', [model, response, options])
        })

    addMember: (member) ->
      # make sure the users collection is up to date
      #
      # this passes the param fetch_all in the URL and will ignore pagination to get
      # the entire collection
      @userCollection.fetch({data: { fetch_all: true }, processData: true}).then =>

        # members - Items currently in the members list
        members = _.flatten([@get('users'), @get('groups')])
        # validMembers - Items that can be added to the members list
        validMembers = _.flatten([@userCollection.pluck('username') || [], @collection.pluck('name')])
        # We can't add the current group to itself
        validMembers = _.without(validMembers, @get('groupname'))

        # Check to make sure that the addition has not already been added. Isn't
        # in the members list.
        if _.contains(members, member)
          @trigger('currently_member')

        # Check to make sure that the addition can be added. Is in the validMembers
        # list.
        else if not _.contains(validMembers, member)
          @trigger('invalid_member_name')
        else
          memberType = @getMemberType(member)
          members = @get(memberType)
          members.push(member)

          switch memberType
            when "users" then data = {users: members}
            when "groups" then data = {groups: members}

          @save(data, {
            validate: false,
            error: (model, response, options) =>
              # this add failed, lets remove it from the group since
              # it didn't update on the server
              members.pop()
              @set(memberType, members)
              @trigger('update_error', [model, response, options])
            success: (model, response, options) =>
              @trigger('member_valid')
          })

    getMemberType: (member) ->
      if(_.contains(@collection.pluck('name'), member))
        "groups"
      else if(_.contains(@userCollection.pluck('username'), member))
        "users"
      else
        # This should never happen, but if it does we have a problem.
        throw Error("Member does not appear to have a type")

