define (require) ->
  Actions = require('collections/actions')
  Backbone = require('b')
  Client = require('models/client')
  Clients = require('collections/clients')
  Containers = require('collections/containers')
  Cookbooks = require('collections/cookbooks')
  Databags = require('collections/databags')
  Environments = require('collections/environments')
  Groups = require('collections/groups')
  Nodes = require('collections/nodes')
  Roles = require('collections/roles')
  Runs = require('collections/runs')
  T = require('utils/i18n.gettext')
  Users = require('collections/users')
  addIDValidation = require('helpers/validation/id')
  config = require('config')

  class Organization extends Backbone.Model
    idAttribute: 'name'

    defaults:
      name: ''
      full_name: ''
      org_type: 'Business'

    validation:
      full_name:
        required: true
      name: [
        { required: true, msg: T("Short name is required") }
        {
          pattern: /^[a-z0-9\-_]+$/
          msg: T("Short name can only contain lowercase letters, numbers, hyphens, and underscores")
        }
        { fn: 'orgIdValidate'}
      ]

    orgIdValidate: (value, attr, computed) ->
      return unless (value || '').toString().trim()
      deferred = new $.Deferred
      $.ajax
        type: 'GET'
        url: "/organizations/#{value}/exists"
        dataType: 'json'
        success: (data) -> deferred.resolve(true)
        error: (xhr) ->
          if xhr.status == 409 # conflict
            deferred.reject(T("This short name is already in use"))
          else
            deferred.reject(T('A server error occurred. Please refresh and try again'))
      deferred

    initialize: ->
      @app = @get('app')

      @actions = new Actions(null, {organization: @, app: @get('app')})
      @nodes = new Nodes(null, {organization: @})
      @roles = new Roles(null, {organization: @})
      @databags = new Databags(null, {organization: @})
      @cookbooks = new Cookbooks(null, {organization: @})
      @environments = new Environments(null, {organization: @})
      @users = new Users(null, {organization: @})
      @groups = new Groups(null, {organization: @})
      @runs = new Runs(null, {organization: @})
      @clients = new Clients(null, {organization: @})
      # Containers must be last since it references other collections internally
      @containers = new Containers(null, {organization: @})

      @validationClient = new Client(
        { name: "#{@id}-validator" },
        { collection: @clients }
      )

    # returns true if darklaunch feature is enabled,
    # returns false if not present.
    isFeatureEnabled: (feature) ->
      !!@get("darklaunch")[feature]

    # Returns a string, 'free', 'basic' or 'premium', indicating the reporting
    # billing tier
    reportingBillingTier: ->
      if config.private_chef || !@isFeatureEnabled('reporting')
        undefined
      else if @isFeatureEnabled('reporting_premium_tier')
        'premium'
      else if @isFeatureEnabled('reporting_basic_tier')
        'basic'
      else
        'free'
