define (require) ->
  Backbone = require('b')
  T = require('utils/i18n.gettext')

  class SignUpForm extends Backbone.Model
    # Deal with not triggering change notices on initial bind
    defaults:
      username: ''
      email: ''
      display_name: ''
      first_name: ''
      last_name: ''
      job_title: ''
      company: ''
      phone: ''
      terms: null

    validation:
      username: [
        {required: true, maxLength: 50}
        {pattern: /^[a-z0-9\-_]+$/, msg: T('Username can only contain lowercase letters, numbers, hyphens, and underscores')}
        {fn: 'signUpValidate'}
      ]
      email: [
        {pattern: 'email'}
        {fn: 'signUpValidate'}
      ]
      display_name: [
        {required: true, maxLength: 100}
        {pattern: /^[^":*?<>^$\\|\\/.@{}()!_#&`~=+%]+$/, msg: T('^":*?<>$|\/.\\@{}()!_#&`~=+% are not allowed in Full Name')}
        {fn: 'notRequiredForVip'}
      ]
      first_name:
        fn: 'onlyRequiredForVip'
        required: true
      last_name:
        fn: 'onlyRequiredForVip'
        required: true
      job_title:
        fn: 'onlyRequiredForVip'
        required: true
      company:
        fn: 'onlyRequiredForVip'
        required: true
      phone:
        fn: 'onlyRequiredForVip'
        required: true

    initialize: (options) ->
      super
      delete @validation["terms"] if require('config').private_chef

    signUpValidate: (value, attr, computed) ->
      return unless (value || '').toString().trim()
      $.Deferred (deferred) ->
        $.ajax
          type: 'GET'
          url: "/signup/validity/#{attr}"
          data: { value }
          dataType: 'json'
          success: deferred.resolve
          error: (xhr) ->
            if xhr.status == 409 # conflict
              deferred.reject(T("This #{attr} is already in use"))
            else
              deferred.reject(T(
                'A server error occurred. Please refresh and try again'
              ))

    onlyRequiredForVip: (value, attr, computed) ->
      false unless computed.ref == 'vip'

    notRequiredForVip: (value, attr, computed) ->
      return unless (value || '').toString().trim()
      false if computed.ref == 'vip'
