# Test to see if the Blob is (fully) supported.
#
# Blobs are used to download files in the browser. A simple `typeof window.Blob`
# check will not do, since Safari has Blobs, but they don't work correctly in
# the situation in which we use them.
#
# This code is adapted from
# https://github.com/ssorallen/blob-feature-check/blob/master/blob-feature-check.js
#
# Return a promise, since it is asyncronous.
define (require) ->
  Promise = require('promise')

  new Promise (resolve, reject) ->
    try
      svg = new Blob(
        ["<svg xmlns='http://www.w3.org/2000/svg'></svg>"],
        { type: "image/svg+xml;charset=utf-8" }
      )
    catch e
      reject(false)

    # Safari 6 uses "webkitURL".
    url = window.URL || window.webkitURL
    objectUrl = url.createObjectURL(svg)

    if /^blob:/.exec(objectUrl) == null
      # `URL.createObjectURL` created a URL that started with something other
      # than "blob:", which means it has been polyfilled and is not supported by
      # this browser.
      reject(false)
    else
      img = new Image()
      img.onload = -> resolve(true)
      img.onerror = -> reject(false)
      img.src = objectUrl
