define (require) ->
  $ = require('jquery')
  _ = require('underscore')

  class DashQuery
    constructor: (@baseUrl) ->

    query: (params) ->
      $.ajax(
        url: @baseUrl + '/runs_query'
        dataType: 'json'
        data: params
      ).then((data) ->
        data
      )

    getDashCountsData: (start, end) ->
      $.when(
        @getCountsData(start,end)
      ).then((data) ->
        data
      )

    getDashDurationsData: (start, end) ->
      $.when(
        @getDurationsData(start,end)
      ).then((data) ->
        data
      )

    getGranularity: (start, end) ->
      granularity = 'hour'

      # if greater than 24 hour period, return day
      if (end - start > 24 * 60 * 60)
        granularity = 'day'

      # we want to look at the day granularity until
      # we get to > 30 days, then we want to look at
      # the week granularity
      if (end - start > 30 * 24 * 60 * 60)
        granularity = 'week'

      granularity

    getCountsData: (start, end) ->
      granularity = @getGranularity(start, end)
      params = {
        dashboard: true,
        startTime: start,
        endTime: end,
        granularity: granularity
      }

      normalizeRunCounts = (data) ->
        BUCKET_FORMAT = 'YYYY-MM-DD[T]HH:mm:ss[Z]'

        counts = []
        statuses = ['success', 'failure', 'aborted']
        buckets = {}
        thisBucket = moment(data.start_bucket).zone(0)
        lastBucket = moment(data.end_bucket).zone(0)

        emptyCount = (status, start) ->
          runcount: 0
          start_time: start
          status: status

        if _(data.run_counts).isEmpty()
          return counts

        while thisBucket.isBefore(lastBucket) or thisBucket.isSame(lastBucket)
          buckets[thisBucket.zone(0).format(BUCKET_FORMAT)] = []
          thisBucket = thisBucket.zone(0).add(granularity, 1)

        _(data.run_counts).forEach (count) ->
          buckets[count.start_time].push(count)

        _(buckets).forEach (bucket, key) ->
          _(statuses).forEach (status) ->
            count = _(bucket).findWhere(status: status) or emptyCount(status, key)
            count.end_time = moment(key).zone(0).add(granularity, 1)
            counts.push count

        counts

      @query(params).then((data) ->
        normalizeRunCounts(data)
      # On failure, return an empty data set
      , -> [])

    getDurationsData: (start, end) ->
      granularity = @getGranularity(start, end)
      params =
        bucketing: 'duration'
        dashboard: true
        startTime: start
        endTime: end
        granularity: granularity

      @query(params).then((data) ->
        data.run_durations
      # On failure, return an empty data set
      , -> [])

