define (require) ->
  $ = require('jquery')
  Moment = require('moment')
  SpinnerView = require('views/controls/spinner')
  T = require('utils/i18n.gettext')
  d3 = require('d3')
  nv = require('nv')
  router = require('router')

  graph =
    getNewSpinner: ->
      loadingSpinner = new SpinnerView({spinner: {
          lines: 9,
          width: 5,
          length: 20,
          radius: 40,
          trail: 66,
          top: 10,
      }})
      loadingSpinner

    drawLineGraph: (data, time_array, div_ref, status_string, color_css, hourly) ->
      div_ref = div_ref[0]

      dataObject = [{values:[], "color":color_css[0], "key":status_string[0]}, \
                    {values:[], "color":color_css[1], "key":status_string[1]}, \
                    {values:[], "color":color_css[2], "key":status_string[2]}]

      for successDatum, i in data.success
        failureDatum = data.failure[i]
        abortedDatum = data.aborted[i]

        dataObject[0].values.push({"x": time_array[i], "y": successDatum})
        dataObject[1].values.push({"x": time_array[i], "y": failureDatum})
        dataObject[2].values.push({"x": time_array[i], "y": abortedDatum})

      nv.addGraph( ->
        chart = nv.models.lineChart()

        if hourly
          chart.xAxis
          .tickFormat((d) -> moment.utc(d).format("HH:mm"))
        else
          chart.xAxis
          .tickFormat((d) -> moment.utc(d).format("MMM DD"))

        chart.yAxis
        .tickFormat(d3.format(',d'))

        d3.select(div_ref).select("svg").datum(dataObject).call(chart)

        nv.utils.windowResize(chart.update)

        chart
      )

    drawBarChartGraph: (data, color, bucket_list, start_seconds, bucket_time_interval, base_url, clickable, app, div_ref, colorArray, stagger, percent, parseTimeArray, hourly) ->
      div_ref = div_ref[0]

      dataObject = [values:[]]
      for datum, i in data
        if colorArray.length == 0
          dataObject[0].values.push({"label":bucket_list[i], "value":datum, "color":color})
        else
          dataObject[0].values.push({"label":bucket_list[i], "value":datum, "color":colorArray[i]})

      nv.addGraph( ->
        chart = nv.models.discreteBarChart()
        .x((d) -> d.label )
        .y((d) -> d.value )
        .staggerLabels(stagger)
        .showValues(true)
        .tooltips(false)

        if percent
          chart.forceY([0,1])
          chart.yAxis.tickFormat(d3.format(',.0%'))
        else if parseTimeArray
          if hourly
            chart.xAxis
            .tickFormat((d) -> moment.utc(d).format("HH:mm"))
          else
            chart.xAxis
            .tickFormat((d) -> moment.utc(d).format("MMM DD"))
          chart.yAxis
          .tickFormat(d3.format(',d'))
        else
          chart.yAxis
          .tickFormat(d3.format(',d'))

        d3.select(div_ref).select("svg")
        .datum(dataObject)
        .transition().duration(500)
        .call(chart)

        nv.utils.windowResize(chart.update)
        nv.utils.windowResize(->
          if percent
            d3.select(div_ref).select("svg").selectAll(".nv-bar").selectAll("text").text((d) -> (d.value*100).toFixed(2) + "%")
          else #just convert to int
            d3.select(div_ref).select("svg").selectAll(".nv-bar").selectAll("text").text((d) -> d.value)
        )

        d3.select(div_ref).select("svg").selectAll("rect")
          .on("click", (d, i) ->
              if clickable
                start_time = (start_seconds) + (bucket_time_interval*(i-1))
                end_time = start_time + bucket_time_interval
                url = base_url + "/runs?status=" + status + "&endTime=" + parseInt(end_time, 10).toString() + "&startTime=" + parseInt(start_time, 10).toString()
                router.navigate(url, { trigger: true })
        )

        if percent
          d3.select(div_ref).select("svg").selectAll(".nv-bar").selectAll("text").text((d) -> (d.value*100).toFixed(2) + "%")
        else #just convert to int
          d3.select(div_ref).select("svg").selectAll(".nv-bar").selectAll("text").text((d) -> d.value)

        chart
      )
