define (require) ->
  ErrorLine = require('views/controls/alerts/error_line')
  Modal = require('views/controls/modals/base')
  Spinner = require('views/controls/spinner')
  SuccessLine = require('views/controls/alerts/success_line')
  _ = require('underscore')
  state = require('state')
  template = require('j!application/modals/org_invitation')

  class OrgInvitationModal extends Modal
    template: template
    events:
      "click input":"inputClicked"
      "click .btn-danger":"rejectOrgs"
      "click .btn-accept":"acceptOrgs"
      "click .btn-close":"closeEvent"

    className: "#{@::className} modal-org-invite"
    initialize: ->
      @initialized = true
      @orgsUpdated = false
      @usernav = @options.usernav
      @invites = null
      @rejectDisabled = @options.rejectDisabled ? false

      # pass in firstOrgMode:true if you want the modal to return
      # to the NoOrganizationsModal on remove. This is to prevent
      # the user from getting to a black app state on login with no orgs.
      if @options.firstOrgMode?
        @firstOrgMode = true
      else
        @firstOrgMode = false
      super()

      $.ajax({
        url: "/users/#{state.get('user')?.id}/association_requests"
        type: "get"
        dataType: "json"
        success: (data) =>
          @invites = data
          @render()
      })

    afterRender: ->
      # if firstOrgMode, don't let user leave by clicking outside modal
      if @firstOrgMode
        @$el.modal({backdrop: 'static'})
      super

    closeEvent: =>
      # if orgs were updated, reload the the current URL to refresh the org dropdown
      if @orgsUpdated and !@firstOrgMode
        window.location.assign(window.location.href)

      # go back to NoOrganizationsModal instead of blank app
      if @firstOrgMode
        # if the user accepted an invite, navigate to orgs
        if @orgsUpdated
          window.location.href="/starter-kit"
        # else, back to NoOrganizationsModal with you!
        else
          @$el.modal('hide')
          @cleanup()
          @app.goToNoOrganizationsModal()

      @usernav?.trigger('invites_updated')

    serialize: ->
      _.extend(super(), {invites: @invites, rejectDisabled: @rejectDisabled, firstOrgMode: @firstOrgMode})

    beforeRender: ->
      @setView(".spinner", new Spinner)

    inputClicked: ->
      isEnabled = @$("input:checked").length == 0
      @$(".btn-danger").attr("disabled", isEnabled)
      @$(".btn-accept").attr("disabled", isEnabled)

    rejectOrgs: ->
      @handleRequests(@rejectUrl,
                      "reject")

    rejectUrl: (currId, currOrgName) ->
      "/users/#{state.get('user')?.id}/association_requests/#{currId}"

    acceptOrgs: ->
      @handleRequests(@acceptUrl,
                      "accept")

    acceptUrl: (currId, _currOrgName) ->
      "/users/#{state.get('user')?.id}/association_requests/#{currId}"

    handleRequests: (urlFunction, type) ->
      @$("input").each( (index, element) =>
        if @$(element).prop("checked")
          currId = @invites[index].id
          currOrgName = @invites[index].orgname
          $.ajax({
            url: urlFunction(currId, currOrgName)
            type: "put"
            data: { response: type }
            dataType: "json"
            success: (data) =>
              # hide and uncheck check box. could just remove, but that would fk up the
              # index of @invites and its an array so expensive to remove curr element.
              @$(element).parent("label").removeClass("current-visible").css("display", "none")
              @$(element).prop("checked", false)

              if @$(".current-visible").length == 0
                @setView(".extra-info", new SuccessLine({
                  body: "There are no remaining organization invitations."
                })).render()

              @$(".btn-close").html("Close")

              # disable buttons if all checkboxes handled
              @inputClicked()

              # if the user successfully accpeted an org invite, note so we can
              # re-load the page on modal close
              if type == "accept"
                @orgsUpdated = true

            error: (data) =>
              @setView(".extra-info", new ErrorLine({
                body: "There was a problem processing one or more invitations. Please try again later."
              })).render()

              # disable buttons if all checkboxes handled
              @inputClicked()
          })
      )
