define (require) ->
  Backbone = require('b')
  template = require('j!application/sidebar/actions')

  require('bootstrap-tooltip')

  # This is its own view so I can re-render it independently of the sidebar section itself
  class SidebarActions extends Backbone.Layout
    template: template
    className: 'sidebar-actions'
    events:
      'click a': 'clickAction'

    initialize: ->
      @selection = @options.selection
      @actions = @options.actions
      @controller = @options.controller
      @resetBound = false

    beforeRender: ->
      unless @resetBound
        @selection.on('reset', @render, @)
        @resetBound = true

    afterRender: ->
      @$('a.disabled').tooltip({ placement: 'bottom', container: 'body' })

    cleanup: ->
      @selection.off(null, null, @)
      @resetBound = false

    serialize: ->
      enabled = {}
      for action in @actions
        enabled[action.id] = @isActionEnabled(action)
      {actions: @actions, actionsEnabled: enabled}

    clickAction: (evt) ->
      $elm = $(evt.currentTarget)
      unless $elm.hasClass('disabled')
        action = $elm.data('action')
        @trigger("action:#{action}")
        @trigger('action', action)
        @controller.dispatchAction(action) if @controller

    # Given an action, determine if is should be enabled or not
    isActionEnabled: (action) ->
      return true if action.always
      return false if !@selection || @selection.length == 0
      return false if !action.multi && @selection.length != 1
      return !action.disabled_when() if typeof action.disabled_when == 'function'

      if action.access
        for model in @selection.models
          if model.get('access')?[action.access]
            return true
          else
            return false

      true
