define (require) ->
  EmptyObjectView = require('views/application/split_pane/empty_object')
  Layout          = require('views/controls/layout')
  MultiObjectView = require('views/application/split_pane/multi_object')

  class SplitPaneView extends Layout
    template: -> '<div id="list-pane" class="pane"></div><div id="object-pane" class="pane"></div>'

    initialize: ->
      @app = @options.app
      @selection = @options.selection ? new Backbone.Collection
      @options.layoutOptions =
        north:
          paneSelector: '#list-pane'
          size: '50%'
        center:
          paneSelector: '#object-pane'

      super()

    cleanup: ->
      @selection.off(null, null, @)
      super

    beforeRender: ->
      @selection.off('reset', null, @)
      @selection.on('reset', @updateSelection, @)

    afterRender: ->
      super
      @onResize()

    applyController: (controller) ->
      @collection = controller.collection
      @messages = controller.messages ? {}
      @tabs = _.result(controller, 'tabs') ? [] # Might be unused sometimes
      @currentTabId = controller.currentTabId
      @emptyView = new EmptyObjectView({message: @messages.selectEmpty})
      @multiView = new MultiObjectView({ selection: @selection, message: @messages.selectMulti(@selection.length) })

      unless controller.objectViewClass is @objectViewClass
        @objectViewClass = controller.objectViewClass
        if @activeObjectView is @objectView and @activeObjectView? and @objectView?
          @updateSelection()

      unless controller.listViewClass is @listViewClass and @listView?.collection is controller.collection
        @listViewClass = controller.listViewClass
        @listView = new @listViewClass({app: @app, collection: controller.collection, selection: @selection, actions: controller.actions, controller: controller})
        @setView('#list-pane', @listView).render()

    updateSelection: ->
      @activeObjectView = if @selection.length == 0
        @emptyView = new EmptyObjectView({message: @messages.selectEmpty()})
      else if @selection.length == 1
        @objectView = new @objectViewClass({
          model: @selection.at(0),
          app: @app,
          tabs: tab for tab in @tabs when not tab.condition or tab.condition(@selection.at(0), @app),
          currentTabId: @currentTabId,
          label: @messages.label(1)
        })
      else
        @multiView = new MultiObjectView({
          selection: @selection
          message: @messages.selectMulti(@selection.length)
        })
      @setView('#object-pane', @activeObjectView).render()

    resizePanes: ->
      @instance?.resizeAll()
      @onResize()

    onResize: (args...) ->
      @listView.resizeGrid() if @listView?
      @objectView?.resize?()
      super
