define (require) ->
  $ = require('jquery')
  Backbone = require('b')
  MasterQueueModal = require('views/application/modals/master_queue')
  NotificationCenter = require('views/controls/notification_center')
  OrganizationDropdown = require('views/application/usernav/organization_dropdown')
  ReportFeedbackModal = require('views/controls/modals/report_feedback')
  T = require('utils/i18n.gettext')
  UserDropdown = require('views/application/usernav/user_dropdown')
  config = require('config')
  router = require('router')
  state = require('state')

  require('bootstrap-tooltip')

  class Usernav extends Backbone.Layout
    template: require('j!application/usernav')

    events:
      'click .report-bug': 'reportFeedback'
      'click .notifier': 'pendingInvites'
      'click .jobs-in-progress': 'renderMasterQueueModal'

    renderMasterQueueModal: ->
      new MasterQueueModal({app: @app, masterQueue: @app.masterQueue})

    initialize: ->
      @app = @options.app

      @userDropdown = new UserDropdown
      @setView('.user-dropdown', @userDropdown)

      unless @app.organizations.isEmpty()
        @organizationDropdown = new OrganizationDropdown({app: @app})
        @setView('.organization-dropdown', @organizationDropdown)

      @on("invites_updated", @updateInvites, @)

      @totalInvites = 0

      # update master job queue UI
      @app.masterQueue.on("success", @updateJobsInProgress, @)
      @app.masterQueue.on("error", @errorInMasterQueue, @)
      @app.masterQueue.on("queue_extended", @updateJobsInProgress, @)

    beforeRender: ->
      @userDropdown.off(null, null, @)
      @userDropdown.on('action:logout', @logout, @)
      @userDropdown.on('action:account', @showAccount, @)
      @userDropdown.on('action:profile', @showProfile, @)
      @organizationDropdown.off(null, null, @)
      @organizationDropdown.on('action', @switchOrg, @)
      @organizationDropdown.on('action:manage', @manageOrgs, @)

    serialize: ->
      {organizations: @app.organizations}

    errorInMasterQueue: ->
      @updateJobsInProgress()
      @updateNotificationsUI()

    updateJobsInProgress: ->
      @$(".jobs-in-progress").html(@app.masterQueue.length())

    cleanup: ->
      @userDropdown.off(null, null, @)
      @organizationDropdown.off(null, null, @)
      @off(null, null, @)

    afterRender: ->
      @$(".jobs-container").tooltip({placement: 'bottom'})
      @notificationCenter = new NotificationCenter({totalInvites: @totalInvites, errorQueue: @app.masterQueue.generateErrorQueue(), app: @app, usernav: @})
      @setView(".notifications-dropdown", @notificationCenter).render()
      @updateInvites()
      @notificationCenter.hide()
      @updateJobsInProgress()

    updateJobsInProgress: ->
      @$(".jobs-in-progress").html(@app.masterQueue.length())
      if @app.masterQueue.length() > 0
        @$(".jobs-in-progress").attr('disabled', false)
      else
        @$(".jobs-in-progress").attr('disabled', true)

    applyController: (controller) ->
      @controller = controller
      @updateInvites()

    getInviteData: ->
      $.ajax({
        url: "/users/#{state.get('user')?.id}/association_requests",
        type: "get"
      })

    updateInvites: ->
      promise = @getInviteData()
      promise.success (data) =>
        @totalInvites = data.length
        @updateNotificationsUI()

    updateNotificationsUI: ->
      totalNotification = @totalInvites + @app.masterQueue.numberOfErrors()
      if totalNotification > 0
        @$(".notifier").attr('disabled', false)
      else
        @$(".notifier").attr('disabled', true)
        @notificationCenter.hide()
      @notificationCenter.updateNotifications(@totalInvites)
      @$(".notifier").html(totalNotification)

    pendingInvites: (evt) ->
      evt.preventDefault()
      # re-render notifications if closed
      if @$(".notification_center").css("display") == "none"
        @notificationCenter.updateNotifications(@totalInvites)
      @notificationCenter.toggle()

    reportFeedback: (evt) ->
      evt?.preventDefault()
      new ReportFeedbackModal({
        app: @app
        controller: state.get('controller')
      })

    logout: -> $.post('/logout').then -> window.location.assign('/login')

    showHelp: -> window.open(config.chef_documentation_url, '_blank')

    showProfile: ->
      if config.private_chef
        router.navigate(state.get('user')?.url(), { trigger: true })
        state.get('controller').collection.trigger(
          "url_manually_navigated",
          state.get('user')
        )
      else
        window.open(config.chef_identity_url)

    showAccount: ->
      window.open(config.chef_account_management_url)

    switchOrg: (action) ->
      if match = action.match(/^switch:(.+)$/)
        @organizationDropdown.close()
        org = @app.organizations.get(match[1])
        if org and org isnt @app.organizations.last()
          newUrl = @controller.url(org)
          @app.organizations.getCurrent(org.id)
          @app.trigger('change:org', org)
          router.navigate(newUrl, { trigger: true })

    manageOrgs: ->
      @organizationDropdown.close()
      router.navigate(@app.organizations.url(), { trigger: true })
