define (require) ->
  Client = require('models/client')
  ErrorLine = require('views/controls/alerts/error_line')
  Modal = require('views/controls/modals/base')
  Spinner = require('views/controls/spinner')
  SuccessLine = require('views/controls/alerts/success_line')
  router = require('router')
  saveAs = require('FileSaver')
  template = require('j!clients/modals/create')

  # Check if the browser supports the Blob API
  blobSupported = try
    !!new Blob()
  catch e
    false

  class ClientCreateModal extends Modal
    template: template
    className: "#{@::className} modal-client-create"

    events:
      'keyup #client-user-name': 'keyPressed'
      'click .btn-create': 'submit'
      'click .btn-download': 'download'
      'click .btn-close': 'close'
      'click .user-link': 'navigateToUser'

    initialize: ->
      super
      @collection = @options.app.currentOrg.clients
      @model = new Client({}, { collection:  @collection})
      @modelSaved = false

    keyPressed: (event) ->
      # remove the error / success msg cause the user
      # is doing somethine else now, and change close -> cancel
      @$(".message").html("") unless event.which == $.ui.keyCode.ENTER

      if $.trim(@$('#client-user-name').val()).length == 0
        @$(".btn-create").attr('disabled', true)
      else
        @$(".btn-create").attr('disabled', false)

    download: (evt) ->
      evt.preventDefault()
      if @privateKey and blobSupported
        saveAs(new Blob([@privateKey]), "#{@model.id}.pem")

    close: ->
      @remove()

      if @modelSaved
        @model.collection.add(@model, { at: 0 })
        router.navigate(@model.url(), { trigger: true })

    navigateToUser: (event) ->
      event.preventDefault()
      router.navigate(@$(event.target).attr("href"), { trigger: true })

    submit: (event) ->
      event.preventDefault()

      @$(".btn-close").html("Close")
      @setView(".message", new Spinner).render()

      @model.set("name", @$("#client-user-name").val())
      @model.set("validator", @$("#client-validator").is(':checked'))
      @model.sync('create', @model, {
        url: @model.collection.url()
        success: (data, textStatus, jqXHR) =>
          @privateKey = data.private_key
          @modelSaved = true

          @$(".btn-download").css("display", "inline-block")
          @$(".btn-create").remove()
          @$("form").remove()

          @setView(".message", new SuccessLine({body: "Client successfully created. Copy the private key below and keep it somewhere safe. It is NOT stored on the server."})).render()

          @$("pre").css("display", "block").html(@privateKey)
        error: (xhr, textStatus, errorThrown) =>
          errorView = switch xhr.status
            when 409
              # user conflict case
              if xhr.responseJSON.url?
                new ErrorLine({body: "Creating a client by this name would overwrite this <a class='user-link' href='#{xhr.responseJSON.url}'>user</a>. Please either delete that user or choose a different client name to avoid a conflict."})
              else
                new ErrorLine({body: "A client by this name already exists."})
            when 403 then new ErrorLine({body: "You do not have permission to create this client."})
            else new ErrorLine({body: "There was a problem creating the client, please try again later."})

          @setView(".message", errorView).render()
      })
