define (require) ->
  _ = require('underscore')
  Backbone = require('b')
  ConstraintEditItemView = require('views/controls/constraints_editor/edit_item')
  constraintsEditorTpl = require('j!controls/constraints_editor')
  cookbookOptionsTpl = require('j!controls/constraints_editor/cookbook_options')
  ErrorLine = require('views/controls/alerts/error_line')
  $ = require('jquery')

  class ConstraintsEditorView extends Backbone.Layout
    className: 'constraints-editor'

    template: constraintsEditorTpl

    events:
      'click .add-constraint': 'onAddClick'
      'keyup .constraint-input': 'addButtonEnabled'

    initialize: (options) ->
      @views = []
      @cookbooks = options.cookbooks
      @constraints = options.constraints

      @cookbooks.on('reset', @onCookbooksReset, @)
      @constraints.on('add', @onConstraintAdd, @)
      @constraints.on('remove', @onConstraintRemove, @)

    # Add the existing constraints
    #
    # Using the native `bind` here works but fails in the tests for some reason
    # so use underscore's I guess.
    afterRender: ->
      @constraints.map(_.bind(@onConstraintAdd, @))
      @removeCurrentConstraints()

    removeCurrentConstraints: ->
      # remove current constraints from select
      for model in @constraints.models
        currName = model.get("name")
        @$(".constraint-name option[value='#{currName}']").remove()
      @sortCookbookSelect()

    addButtonEnabled: (e) ->
      e?.preventDefault()
      currValue = @$(".constraint-input").val()

      # if a valid version is present, enable add button
      # valid formats are exactly three integers seperated by exactly
      # two dots, i.e. "11.0.232"
      if currValue.length == 0 or not currValue.match(/^(\d+\.\d+\.\d+)$/) or @onlyOneCookbookLeft()
        @$(".add-constraint").attr("disabled", true)
      else
        @$(".add-constraint").attr("disabled", false)

    onlyOneCookbookLeft: ->
      @$(".constraint-name").children().length == 0

    onAddClick: (e) ->
      e.preventDefault()

      # for some reason this event still fires when button is disabled and clicked
      if @$(".add-constraint").attr("disabled") then return

      @constraints.add({
        name: @$('[name="new-constraint-name"]').val(),
        operator: @$('[name="new-constraint-operator"]').val(),
        version: @$('[name="new-constraint-version"]').val(),
      })
      # remove used value
      index = @$(".constraint-name").get(0).selectedIndex
      @$(".constraint-name option:eq(#{index})").remove()
      @sortCookbookSelect()

      # check if add button should be enabled
      @addButtonEnabled(null)

    onConstraintAdd: (constraint) ->
      view = new ConstraintEditItemView({
        model: constraint
      })
      @views.push(view)
      view.render().view.$el.insertBefore(@.$('tbody tr:last-child'))

    onConstraintRemove: (constraint) ->
      view = _(@views).find((view) -> view.model == constraint)
      @views = _(@views).without(view)
      view.remove()

      # add the cookbook back into the select
      cookbookName = view.model.get("name")
      @$(".constraint-name").append($('<option>', { value : cookbookName }).text(cookbookName))
      @sortCookbookSelect()

      # check if add button should be enabled
      @addButtonEnabled(null)

    displayNoCookbooksError: ->
      @setView(".constraint-content", new ErrorLine({body:"You cannot add constraints since there are no cookbooks. Please upload a cookbook if you wish to add constraints."})).render()
      @trigger("no_cookbook_error")

    onCookbooksReset: (cookbooks) ->
      if @cookbooks.length == 0
        @displayNoCookbooksError()
      else
        @$('[name="new-constraint-name"]').html(cookbookOptionsTpl({
          cookbooks: cookbooks.toJSON()
        }))
        @removeCurrentConstraints()

    # http://stackoverflow.com/questions/17021086/sort-alphabetically-more-select-options
    sortCookbookSelect: ->
      $('.constraint-name').each( ->
        options = $(@).children('option')
        options.sort( (optionA, optionB) ->
          return optionA.value.localeCompare(optionB.value)
        )
        $(@).empty().append(options)
      )
