define ['views/controls/semimodal', 'jquery', 'underscore', 'j!controls/dropdown', 'jquery.ui/position'], (Semimodal, $, _, tpl) ->
  class Dropdown extends Semimodal
    className: 'dropdown' # Bleh, stub div makes inline use impossible
    template: tpl
    events:
      'click .dropdown-toggle': 'clickToggle'

    items: []

    initialize: ->
      super
      @label = @options.label
      @items = @options.items ? @items
      @keyboardSelection = null
      _.bindAll(@, 'triggerEvent', 'updateKeyboardSelection', 'clearKeyboardSelection')
      @on('pullout', @attachPulloutEvents, @)
      @on('open', @attachGlobalEvents, @)
      @on('close', @clearKeyboardSelection, @)

    cleanupEvents: ->
      super
      $(document).off('keydown', @updateKeyboardSelection)

    serialize: ->
      _.extend(super, label: @label, labelIcon: @options.labelIcon, items: @items)

    attachPulloutEvents: ->
      @$pullout.on('click', 'a', @triggerEvent)
               .on('mousemove', @clearKeyboardSelection)

    attachGlobalEvents: ->
      $(document).on('keydown', @updateKeyboardSelection)

    triggerEvent: (evt) ->
      evt.stopPropagation()
      evt.preventDefault()
      $elm = $(evt.currentTarget)
      action = $elm.data('action')
      unless $elm.hasClass('disabled')
        @trigger("action:#{action}")
        @trigger('action', action)
        @close()

    updateKeyboardSelection: (evt) ->
      # esc 27
      # up 38
      # down 40
      # enter 13
      # space 32
      # Check just in case
      return unless @$pullout and @isOpen
      switch evt.which
        when 27 # Esc
          @close()
        when 38, 40 # Up, Down
          numOptions = @numOptions()
          isUp = evt.which == 38
          step = if isUp then -1 else 1
          # I love variable mutation, bite me Erlang
          if @keyboardSelection?
            @keyboardSelection += step
          else
            @keyboardSelection = if curHover = @hoverSelection()
               curHover + step
            else
              if isUp then -1 else 0
          @keyboardSelection %= numOptions
          @keyboardSelection += numOptions if @keyboardSelection < 0
          @$pullout.addClass('keyboard-nav').find('li').removeClass('active')
            .find('a').eq(@keyboardSelection).parent('li').addClass('active')
        when 13, 32 # Enter, Space
          cur = @keyboardSelection ? @hoverSelection()
          if cur?
            activeItem = _.find @items, (item) ->
              cur -= 1 unless item == 'divider' or item.id == 'divider'
              cur < 0
            @trigger("action:#{activeItem.id}")
            @trigger('action', activeItem.id)
          @close()

    clearKeyboardSelection: ->
      if @keyboardSelection?
        @keyboardSelection = null
        @$pullout?.removeClass('keyboard-nav').find('li.active').removeClass('active')

    # Return the effective selection index based on hover
    hoverSelection: ->
      curHover = @_curHover()
      if curHover.length
        curHover.prevAll('li').not('.divider').length
      else
        undefined

    # For subclasses
    numOptions: ->
      count = 0
      for item in @items
        count += 1 if item != 'divider' and item.id != 'divider'
      count

    # Testing hook, because you can't fake pseudo selectors
    _curHover: ->
      @$pullout.find('li:hover')
