# A grid display widget based on SlickGrid
define (require) ->
  require('bootstrap-tooltip')
  Backbone = require('b')
  SlickGrid = require('slickgrid/grid')
  RowSelectionModel = require('slickgrid/plugins/rowselectionmodel')
  _ = require('underscore')
  class Grid extends Backbone.Layout
    initialize: ->
      @selection = @options.selection ? new Backbone.Collection
      @ignoreSelectionUpdate = false
      @columns = @options.columns ? []
      @gridOptions = _.defaults(@options.gridOptions, {
        enableCellNavigation: true,
        enableColumnReorder: false,
        forceFitColumns: true,
        rowHeight: 35,
        permissionsTooltipMessage: "You do not have the proper permissions to view this item's content",
        dataItemColumnValueExtractor: (item, columnDef) -> if _.isFunction(columnDef.field) then columnDef.field.call(item) else item.get(columnDef.field),
      })
      # We always handle init manually
      @gridOptions.explicitInitialization = true
      @initialized = false
      @selection.on('navigate', @updateHighlight, @)
      _.bindAll(@, 'updateSelection')
      @collection.searchSortOrder = null;

    afterRender: ->
      @destroyInstance()
      @$el.css({width:'100%', height: '100%'}) # Force fill
      @instance = new SlickGrid(@$el, @collection, @columns, @gridOptions)
      @$el.css('overflow', '') # Clear this top-level overflow since I don't think I need it and it screws with my borders
      @instance.setSelectionModel(new RowSelectionModel())
      @instance.onSelectedRowsChanged.subscribe(@updateSelection)
      @collection.on('add remove reset', @invalidateGrid, @)
      @collection.on('user_added', @scrollToTopAndSelect, @)

      @resize()
      # This fixes race condition of the grid getting rendered twice. Since this might happen and the tooltips
      # are not set in the scope of @el but in the root of the html, we want to make sure these get cleaned up.
      @forbiddenTooltipHide()

      @instance.onMouseEnter.subscribe(@forbiddenTooltipShow)
      @instance.onMouseLeave.subscribe(@forbiddenTooltipHide)
      @instance.onScroll.subscribe(@forbiddenTooltipHide)
      @instance.onSort.subscribe(@sortHandler)

    sortHandler: (e, sort) =>
      @collection.searchSortOrder = (if sort.sortAsc then '' else '-') + sort.sortCol.id
      @collection.totalRecords = -1
      @collection.currentPage = 0
      @collection.reachedEnd = false
      @collection.fetch({reset: true})

    forbiddenTooltipShow: (event) =>
      forbiddenDiv = @$(event.currentTarget).parent(".forbidden-row:eq(0)")
      if forbiddenDiv.length > 0
        forbiddenDiv.tooltip({placement: "top", trigger: "manual", title: @gridOptions.permissionsTooltipMessage, container: 'body'})
        forbiddenDiv.tooltip("show")

    forbiddenTooltipHide: (event) =>
      @$(".forbidden-row").tooltip("destroy")

    cleanup: ->
      @$(".forbidden-row").tooltip("destroy")
      @collection.off(null, null, @)
      @selection.off(null, null, @)
      @instance?.onSelectedRowsChanged.unsubscribe(@updateSelection)
      @instance?.onMouseEnter.unsubscribe(@forbiddenTooltipShow)
      @instance?.onMouseLeave.unsubscribe(@forbiddenTooltipHide)
      @instance?.onScroll.unsubscribe(@forbiddenTooltipHide)
      @destroyInstance()

    resetSelection: ->
      @instance.getSelectionModel().setSelectedRanges([]);
      @instance.resetActiveCell()

    initGrid: ->
      if @instance and not @initialized
        @instance.init()
        @doGridInit()
        @initialized = true
        true
      else
        false

    doGridInit: ->

    resize: ->
      unless @initGrid()
        if @instance?
          @instance.resizeCanvas()

    destroyInstance: ->
      if @instance and @initialized
        @instance.destroy()
        @initialized = false

    updateSelection: ->
      return if @ignoreSelectionUpdate
      newSelection = _.map(@instance.getSelectedRows(), (n) => @collection.at(n))
      @selection.reset(newSelection)
      @selection.trigger('select')
      @trigger('select')

    updateHighlight: ->
      selection = for model, i in @collection.models when @selection.get(model)?
        i
      # Disable the above handler during external updates to avoid an infinte loop
      try
        @ignoreSelectionUpdate = true
        @instance?.setSelectedRows(selection)
      finally
        @ignoreSelectionUpdate = false

    invalidateGrid: ->
      @$(".forbidden-row").tooltip("destroy")
      @instance.invalidate()

    scrollToTopAndSelect: () ->
      @instance.scrollRowToTop(0)
      @instance.setSelectedRows([0])
      @updateSelection()
