define (require) ->
  Backbone = require('backbone')
  Spinner = require('views/controls/spinner')
  highlight = require('utils/highlight')
  markdown = require('utils/markdown')
  template = require('j!controls/highlighted_file')

  class HighlightedFileView extends Backbone.Layout
    className: 'panel'
    template: template

    initialize: ->
      @fileName = @options.fileName
      @baseUrl = @options.baseUrl
      @srcUrl = @options.srcUrl
      @fileType = @options.fileType
      @version = @options.version
      @syntaxHighlightableFile = @options.syntaxHighlightableFile
      @fileDownloaderClass = @options.fileDownloaderClass
      @markdownFile = @options.markdownFile

      @file = new @fileDownloaderClass({
        name: @fileName
        baseUrl: @baseUrl
        fileType: @fileType
        version: @version
      })

      @loading = true

      @file.on('sync', @loaded, @)
      @file.fetch({dataType: 'text'})

    loaded: =>
      @loading = false
      @file.on('change', @render, @)
      @render()

    afterRender: ->
      if @loading
        @$(".loading-spinner").show()
        @$(".file-content").hide()
        @setView(".loading-spinner", new Spinner).render()
      else
        @$(".file-content").show()
        @$(".loading-spinner").hide()
        if @syntaxHighlightableFile and not @markdownFile
          highlight.element(@el)

    cleanup: ->
      @file.off(null, null, @)

    serialize: ->
      content = if @markdownFile then @file.get('content') else _(@file.get('content')).escape()
      fileName: @fileName
      url: @srcUrl
      content: content
      markdownFile: @markdownFile
      markdown: markdown
