define (require) ->
  Backbone      = require('b')
  _             = require('underscore')
  jsonValidator = require('helpers/json_validator')
  template      = require('j!controls/json_editor')
  require('jquery.textarea')

  class JSONEditor extends Backbone.Layout
    className: 'control-group'

    events:
      'change textarea': 'validate'
      'keyup textarea': 'keyup'

    template: template

    initialize: (options) ->
      @attribute = options.attribute or 'data'
      @autofocus = options?.autofocus
      if !options.outOfDate
        @model.on('change', @indicate_updates, @)

      try
        @model.fetch()
      catch e
        # model.fetch pulls in the latest copy of an object to ensure we are
        # modifying the "latest" copy.
        #
        # When creating a new JSON object, attempts to fetch from a non-existent
        # existing source throw and error, preventing new objects from being
        # created.
        #
        # We catch this error and ignore it. This means that it is possible that
        # an unexpected error could occur, and the user could modify an out-of-date
        # JSON object, but the chance of this is small.

      @root = JSON.stringify(@model.get(@attribute), null, '  ') or '{}'
      @rows = options?.rows || 15

      # Debounce the keyup for better UX
      @keyup = _.debounce(_.bind(@keyup, @), 200)

    afterRender: ->
      @validate()
      $(@el).tabby({tabString:'  '})

    serialize: ->
      { @attribute, @autofocus, @error, @root, @rows }

    getValue: ->
      @$('textarea').val()

    keyup: ->
      @validate()

    submit: ->
      value = @validate()
      return false unless value
      @model.set(@attribute, value)

    # We're maniuplating the state of the DOM instead of rendering here because
    # if you re-render a textarea while you're typing in it it resets your
    # cursor position.
    toggleError: (value) ->
      if value
        classMethod = 'removeClass'
        visibility = 'hidden'
        @trigger('valid')
      else
        classMethod = 'addClass'
        visibility = 'visible'
        @trigger('invalid')

      @$el[classMethod]('error')
      @$('.error-message').css('visibility', visibility)

      value

    indicate_updates: ->
      @$('.out-of-date').css('display', 'block')

    # Doing manual validation to avoid weirdness in string/object attributes
    validate: ->
      @toggleError(jsonValidator(@getValue()))
