define (require) ->
  Backbone = require('b')
  router = require('router')
  template = require('j!controls/list_header')

  class ListHeader extends Backbone.Layout
    template: template

    events:
      'click .search-filter .btn': 'triggerSearch'
      'keypress .search-filter input': 'triggerSearchOnEnter'

    initialize: (@options = {}) ->
      @listView = @options.listView
      @messages = @options.messages ? {}

    serialize: ->
      if @collection.isSolrSearchable
        {
          filter: @_getFilter()
          isSolrSearchable: @collection.isSolrSearchable
          searchMessage: @messages.searchMessage
          titleMessage: @getTitleMessage()
          darklaunch: @options.app?.currentOrg?.attributes?.darklaunch || {}
        }
      else
        {
          isSolrSearchable: @collection.isSolrSearchable
          titleMessage: @getTitleMessage()
        }

    getTitleMessage: ->
      if not @collection.isSolrSearchable or @collection.searchFilter == "*:*"
        @messages.showingAllObjects?()
      else
        if @collection.totalRecords >= 0
          @messages.showingNObjects?(@collection.totalRecords)
        else
          @messages.searchingObjects?()

    updateTitleText: ->
      @$('.header-title').html(@getTitleMessage())

    _getFilter: ->
      if (@collection.searchFilter == "*:*")
        ""
      else
        @collection.searchFilter

    search: (searchFilter) ->
      searchFilter = "*:*" if !searchFilter.length
      # set the search filter, reset collection and trigger fetch
      @collection.searchFilter = searchFilter
      @collection.searchSortOrder = null
      @collection.totalRecords = -1 # backbone paginator doesnt reset this, 0 recs is valid response.
      @resetListAndCollectionState()
      @trigger("search")

    resetListAndCollectionState: ->
      @collection.reset()
      @listView.resetState()

      # updating the URL with the search filter will happen later in
      # ListView.updateSelection, in response to searchFilter being set.

      # Trigger the first fetch for filtered rows
      @collection.goTo(0)

    triggerSearch: (evt) ->
      evt.preventDefault()
      searchFilter = @$(".search-filter input").val()
      @search(searchFilter)

    triggerSearchOnEnter: (evt) ->
      if evt.keyCode == 13
        @triggerSearch(evt)
