define (require) ->
  Backbone = require('b')
  ErrorAlert = require('views/controls/alerts/error')
  MembersList = require('views/controls/members/list')
  Spinner = require('views/controls/spinner')
  template = require('j!controls/members/tab')
  $ = require('jquery')

  # Parent class for parent view of scroll grids that have remove buttons.
  # The scroll grid should scale properly and install a spinner.
  # See views/groups/members.coffee as an example of how to use.
  #
  # Feel free to override the view template
  # but you should include it in your custom jade so that the DOM in which
  # the list gets installed is in your view.
  #
  # Your custom jade template should include a .list div somewhere
  class MembersTab extends Backbone.Layout
    @List: MembersList
    # Basic defaults, subclasses may override
    template: template
    collectionClass: Backbone.Collection
    listClass: @List

    initialize: ->
      @collection ||= new @collectionClass

      @collection.on('sync', @fetchSuccessful, @)

      @listView = new @listClass({collection: @collection})
      @listView.setView('.grid', new Spinner).render()
      @setView('.list', @listView)

    updateFilter: (evt) ->
      $input = $(evt.target)
      $input.val('') if evt.which == $.ui.keyCode.ESCAPE
      @applyFilter($input.val())

    applyFilter: (value) ->
      throw "Implement in child class."

    serialize: ->
      {model: @model, total: @collection.length}

    fetchSuccessful: ->
      @listView.renderGrid()
      # not using .show() because we want display inline and it was init'ed to none
      @$(".name").css("display", "inline")
      @onResize()

    afterRender: ->
      if @model.get("forbidden")?
        @setView(new ErrorAlert({title: "Permissions Error", body: "You do not have permissions to view the contents of #{@model.id}."})).render()
      else
        @onResize()

    onResize: ->
      @listView.onResize()
