define (require) ->
  Backbone = require('b')
  T = require('utils/i18n.gettext')
  _ = require('underscore')
  router = require('router')
  appState = require('state')
  template = require('j!controls/modals/base')

  require('bootstrap-modal')

  class Modal extends Backbone.Layout
    className: 'modal'
    closeButtonText: T('Cancel')
    attributes:
      role: 'dialog'
      'aria-labelledby': 'modal-label'
    template: template
    title: T('Dialog')

    # Might want to prevent this in a subclass
    autoActionScope: true
    autoActionObjects: true
    closeOnRun: true

    # Options used for the bootstrap modal
    modalOptions: {}

    initialize: ->
      @app = @options.app
      router.on('all', @remove, @)
      @closeButtonText = @options.closeButtonText if @options.closeButtonText?
      @controller = @options.controller
      @title = @options.title if @options.title?
      @message = @options.message ? 'Something went wrong. Please try again later.'
      @render()

    serialize: ->
      closeButtonText: @closeButtonText
      collection: @collection
      controller: @controller
      message: @message
      model: @model
      state: @action?.get('state')
      title: @title
      T: T

    cleanup: ->
      @$el.modal('removeBackdrop')
      router.off(null, null, @)
      currentModal = appState.get('currentModal')
      appState.unset('currentModal') if currentModal
      @action.off(null, null, @) if @action
      @open = false

    beforeRender: ->
      currentModal = appState.get('currentModal')
      currentModal.remove() if currentModal
      appState.set('currentModal', @)

    afterRender: ->
      unless @open
        @$el.modal(@modalOptions).on('hidden', _.bind(@remove, @))
        @open = true

    # To be called by subclasses
    queueAction: (action, args...) ->
      if action.indexOf('/') == -1 and @autoActionScope
        action = "#{@controller.id}/#{action}"
      if @autoActionObjects
        objects = if @collection
          @collection.map (obj) -> obj.id
        else if @model
          @model.id
        else
          null
        args.unshift(objects)
      @app.organizations.last().actions.create(
        {action_id: action, arguments: JSON.stringify(args)},
        {wait: true, success: _.bind(@actionQueued, @)}
      )

    actionQueued: (model) ->
      @action.off(null, null, @) if @action
      @action = model
      @action.on('change', @actionChanged, @)

    actionChanged: (actionModel, options) ->
      if not @closeOnRun
        @render()

    onSaved: =>
      @model.collection.add(@model, { at: 0 })
      @model.collection.trigger('user_added')
      @remove()
      router.navigate(@model.url(), { trigger: true })
