define (require) ->
  $ = require("jquery")
  ConfirmationModal = require("views/controls/modals/confirmation")
  ErrorLine = require("views/controls/alerts/error_line")
  JobQueue = require("services/job_queue")
  QueueView = require("views/controls/queue_view")
  Spinner = require("views/controls/spinner")
  SuccessLine = require("views/controls/alerts/success_line")
  _ = require("underscore")
  i18n = require("utils/i18n")
  router = require("router")
  template = require("j!controls/modals/delete")

  class DeleteModal extends ConfirmationModal
    template: template

    yesButtonText: i18n.gettext('Delete')

    initialize: ->
      @collection_type = @options.collection_type
      @messages = @options.messages
      @options.title ?= @messages.delete(@collection.length)
      @collectionUrl = @collection.at(0).collection.url()
      @modelWasDeleted = false
      super

    serialize: ->
      _.extend(super, {messages: @messages})

    clickYes: (evt) ->
      evt.preventDefault()
      @$(".btn-danger").remove()
      if @collection.length == 1
        @setView(".modal-body", new Spinner).render()
        model = @collection.at(0)
        $.ajax
          type: 'DELETE'
          url: model.url()
          success: (data) =>
            @modelWasDeleted = true
            model.collection.remove(model)
            @setView(".modal-body", new SuccessLine({body: "#{model.id} was successfully deleted!"})).render()
            @$(".btn-danger").remove()
            @$(".btn-close").html("Close")
          error: (xhr, textStatus, errorThrown) =>
            if xhr.status == 403
              message = "You do not have permission to delete <code>#{model.id}</code>. Contact an administrator for additional privileges."
            else
              message = "Something went wrong and <code>#{model.id}</code> was not deleted. Please try again later."
            @setView(".modal-body", new ErrorLine({body: message})).render()
            @$(".btn-danger").remove()
            @$(".btn-close").html("Close")

      # set up a queueView
      else
        @$(".btn-close").html("Process in Background")
        requestArray = []
        for model in @collection.models
          requestArray.push({
            type: 'DELETE'
            url: model.url()
            data: null
            model: model
            actionVerb: "delete"
            successEvent: (data, status, xhr, model) -> model.collection.remove(model)
          })
        @modelWasDeleted = true
        jobQueue = new JobQueue({requestArray: requestArray})
        queueView = new QueueView({jobQueue: jobQueue, masterQueue: @app.masterQueue})
        @setView(".modal-body", queueView).render()
        jobQueue.processQueue()
        jobQueue.on("all_requests_sent", @requestsFinished, @)

    requestsFinished: ->
      @$(".btn-close").html("Close")

    cleanup: ->
      super
      if @modelWasDeleted
        router.navigate(@collectionUrl, { trigger: true })
