define (require) ->
  $ = require('jquery')
  ErrorLineAlert = require('views/controls/alerts/error_line')
  Modal = require('views/controls/modals/base')
  OrganizationAssociationRequests = require('collections/organization_association_requests')
  Spinner = require('views/controls/spinner')
  SuccessLineAlert = require('views/controls/alerts/success_line')
  template = require('j!controls/modals/invite_user')

  class InviteUserModal extends Modal
    className: "#{@::className} modal-invite-user"
    template: template
    events:
      'click .btn-primary': 'submit'
      'submit form': 'submit'
      'keyup input': 'keyPressed'

    initialize: (@options) ->
      super()
      @associationRequests = new OrganizationAssociationRequests([], {
        organization: @model
      })
      @associationRequest = new Backbone.Model({}, {
        collection: @assocationRequests
      })

      @listenTo(@associationRequest, 'error', @showError)
      @listenTo(@associationRequest, 'request', @showSpinner)
      @listenTo(@associationRequest, 'sync', @showSuccess)

    showError: (model, response, options) ->
      if response.status == 403
        errorMessage = "You do not have permission to invite users to
          <code>#{@model.id}</code>. You must be an administrator in this
          organization."
      else
        errorMessage = response.responseJSON?.error
        if (errorMessage != undefined)
          errorMessage = _.escape(errorMessage)

      @setView(".message", new ErrorLineAlert({ body: errorMessage })).render()

    showSpinner: -> @setView(".message", new Spinner).render()

    showSuccess: (model, data) ->
      # can no longer "Cancel" but can close and do further actions
      @$(".btn:eq(0)").html("Close")

      message = "An invitation has been sent to the user."

      # If the response has a "warning" property, add it to the message.
      #
      # A "warning" property can be present if the invitation was successfully
      # created in the system, but the email notification failed to send.
      message += "<br>#{data.warning}" if data.warning?

      @setView(".message", new SuccessLineAlert({ body: message })).render()

    keyPressed: (event) ->
      # remove the error / success msg cause the user
      # is doing somethine else now, and change close -> cancel
      @$(".message").html("") unless event.which == $.ui.keyCode.ENTER
      @$(".btn:eq(0)").html("Cancel")

      if @$('input').val().trim().length == 0
        @$(".btn-primary").attr('disabled', true)
      else
        @$(".btn-primary").attr('disabled', false)

    submit: (event) ->
      event.preventDefault()
      @associationRequest.set({ user: @$("input").val().trim() })
      @associationRequests.create(@associationRequest)
