define (require) ->
  JSONEditor    = require('views/controls/json_editor')
  Wizard = require('views/controls/modals/wizard')
  Spinner = require('views/controls/spinner')

  # adds in a few extra methods for handling
  # wizards that contain json validation
  class JsonWizard extends Wizard

    bindings:
      '[name=name]': 'name'
      '[name=description]': 'description'

    events:
      'click .btn-primary': 'submit'

    initialize: ->
      @model ?= @options.model
      super()
      @model.on('validated', @toggleButtons)
      @on('sync:success', @onSaved)
      @on('sync:error', @onSaveError)

    beforeRender: ->
      super()

      @defaultAttributesEditor = new JSONEditor({
        attribute: 'default_attributes',
        model: @model,
        outOfDate: true
      })
      @setView('.defaultAttributes', @defaultAttributesEditor)
      @overrideAttributesEditor = new JSONEditor({
        attribute: 'override_attributes',
        model: @model,
        outOfDate: true
      })
      @setView('.overrideAttributes', @overrideAttributesEditor)

      # validation events
      @defaultAttributesEditor.on("valid", @jsonIsValid, @)
      @defaultAttributesEditor.on("invalid", @jsonIsInvalid, @)

      @overrideAttributesEditor.on("valid", @jsonIsValid, @)
      @overrideAttributesEditor.on("invalid", @jsonIsInvalid, @)

    cleanup: ->
      super
      @overrideAttributesEditor.off(null, null, @)
      @defaultAttributesEditor.off(null, null, @)

    afterRender: ->
      @bindValidation()
      super()

    serialize: ->
      _.extend(super(), { failure: @saveFailed, errorMessage: @errorMessage })

    # Replace the content with the spinner, adjust for the heights, and remove
    # the controls
    spin: ->
      formHeight = @$('form').height()
      @setView('form', new Spinner).render()
      @$('form').height(formHeight)
      @$('form .spinner').css('top', (formHeight / 2) + 'px')
      @$('.modal-header .fa-remove').hide()
      @$('button').prop('disabled', true)

    jsonIsValid: ->
      @currentInputValid = true
      # this check is necessary because validation will
      # only work once an ID has been assigned
      if @model.id then @model.validate()

    jsonIsInvalid: ->
      @currentInputValid = false
      if @model.id then @model.validate()

    submit: (event) ->
      event.preventDefault()
      if @defaultAttributesEditor.submit() && @overrideAttributesEditor.submit()
        @model.validate().then =>
          @spin()
          @defaultAttributesEditor.submit()
          @overrideAttributesEditor.submit()
          @model.sync('create', @model, {
            url: @model.collection.url()
            success: (data, textStatus, jqXHR) =>
              @trigger('sync:success', data, textStatus, jqXHR)
            error: (jqXHR, textStatus, errorThrown) =>
              @trigger('sync:error', jqXHR, textStatus, errorThrown)
          })

    # Enable or disable the create button based on the validity
    toggleButtons: (model, result, isValid) =>
      @$('.btn-primary').prop('disabled', !isValid or !@currentInputValid)
      @buttons.nextButton.prop('disabled', @buttons.wizard.isLastStep() or !isValid or !@currentInputValid)
      @buttons.previousButton.prop('disabled', @buttons.wizard.isFirstStep() or !isValid or !@currentInputValid)
