# Class that implements a modal "wizard"
define (require) ->
  $                       = require('jquery')
  Modal                   = require('views/controls/modals/base')
  WizardBreadcrumbs       = require('views/controls/modals/wizard/breadcrumbs')
  WizardNavigationButtons = require('views/controls/modals/wizard/navigation_buttons')
  template                = require('j!controls/modals/wizard')

  class Wizard extends Modal
    template: template

    initialize: (options) ->
      # change this variable to false when current input is invalid
      # to prevent the user from being able to change the current
      # wizard step
      @currentInputValid = true

      @currentStepIndex = 0
      super()
      @toggleSteps()
      @renderNavigation()

    afterRender: ->
      super()
      @toggleSteps()
      @renderNavigation()

    renderNavigation: ->
      breadcrumbs = new WizardBreadcrumbs(wizard: @)
      @buttons = new WizardNavigationButtons(wizard: @)
      @setView('.modal-subheader nav.wizard', breadcrumbs)
      @setView('.modal-footer nav.wizard', @buttons)
      breadcrumbs.render()
      @buttons.render()

    getCurrentStep: ->
      $(@steps[@currentStepIndex])

    goToNextStep: ->
      return false if @isLastStep()
      @goToStep(@currentStepIndex + 1)

    goToPreviousStep: ->
      return false if @isFirstStep()
      @goToStep(@currentStepIndex - 1)

    goToStep: (step) ->
      go = =>
        @currentStepIndex = step
        newStep = @getCurrentStep()
        @trigger('change:step', newStep)
        @toggleSteps()
        @renderNavigation()
        newStep

      if @currentInputValid
        if @model.validate then @model.validate().then(go) else go()

    isFirstStep: -> @currentStepIndex == 0

    isLastStep: -> @currentStepIndex == (@steps.length - 1)

    toggleSteps: ->
      @steps = @$('[data-step]')
      @steps.hide()
      @getCurrentStep().show()
