define (require) ->
  Backbone = require('b')
  Empty = require('views/controls/empty')
  RunListTreeView = require('views/controls/run_list/tree_view')
  SelectDropdown = require('views/controls/select_dropdown')
  errorTpl = require('j!controls/runlist_error')
  state = require('state')
  template = require('j!controls/run_list')

  class RunListView extends Backbone.Layout
    template: template

    events:
      'click nav .expand-all': 'expandAll'
      'click nav .collapse-all': 'collapseAll'
      'click nav .edit': 'edit'
      'click nav .delete': 'delete'

    initialize: ->
      # this is always triggered by models/run_list_parent:parse()
      @collection.on('sync', @render, @)

      # Enable or disable the ability to edit the run list by hiding/showing
      # the edit link
      @editable = if @options.editable? then @options.editable else true

      @app = @options.app
      @type = @options.type
      @environment = null
      # trigger a fetch from server
      if @collection.length == 0
        @fetchXhr = @collection.fetch
          success: =>
            @fetchXhr = undefined
          error: (model, response, options) =>
            @fetchXhr = undefined
            @errorLayout = new Backbone.Layout({template: errorTpl})
            if response.status == 412
              @errorLayout.serialize = { title: 'Unsatisfiable Run List Item', message: response.responseText }
            else
              @errorLayout.serialize = { title: 'Server Error', message: 'Something went wrong. Please try again later.' }
            @setView('.runlist-tree', @errorLayout).render()

      @collection.object.on "change:chef_environment", (model, value, options) =>
        @environment = value

    cleanup: ->
      @fetchXhr.abort() if @fetchXhr

    serialize: ->
      { @collection, @editable, @environment }

    beforeRender: ->
      if @collection.length
        unless @environment?
          @environment = if @collection.get('_default')
            '_default' # More less this is for roles ...
          else
            @collection.at(0).id # ... and this is for nodes
        @tree = new RunListTreeView({root: @collection.get(@environment).get('run_list')})
        if @collection.get(@environment).get('run_list').length < 1
          @setView('.runlist-tree', new Empty() )
        else
          @setView('.runlist-tree', @tree)
        environments = for run_list in @collection.models
          {id: run_list.id, label: run_list.id}
        if environments.length > 1
          @environmentDropdown = @setView('.environment-picker', new SelectDropdown({
            selected: @environment,
            items: environments,
          })).on('action', @changeEnvironment, @)
        else if @environmentDropdown
          @environmentDropdown.remove()
          @environmentDropdown = undefined
      else
        if @environmentDropdown
          @environmentDropdown.remove()
          @environmentDropdown = undefined

    expandAll: (evt) ->
      evt.preventDefault()
      @tree.expandAll() if @tree

    collapseAll: (evt) ->
      evt.preventDefault()
      @tree.collapseAll() if @tree

    edit: (evt) ->
      evt.preventDefault()
      # Temporarily disable role env run list editing
      if @type == 'role' && @environment != '_default'
        alert('Coming soon')
      else
        state.get('controller')?.dispatchAction('editRunList')

    delete: (evt) ->
      evt.preventDefault()
      alert('Coming soon')

    changeEnvironment: (env) ->
      if env != @environment
        @environment = env
        @render()
