# Base view for available roles/recipes
define (require) ->
  _         = require('underscore')
  Backbone  = require('b')
  Spinner   = require('views/controls/spinner')
  draggable = require('views/controls/run_list_editor/helpers/draggable')
  template  = require('j!controls/run_list_editor/available')

  class AvailableView extends Backbone.Layout
    _.extend(@::, draggable)

    template: template

    events:
      'keyup input':'filterEvent'
      'dropped .ui-sortable':'droppedEvent'

    initialize: (options) ->
      @spinnerView = new Spinner
      { @containment, @connectWith, @startingRunList, @filterable } = options

      @setView('.spinner', @spinnerView)

      @collection.fetch(complete: => @fetchSuccessful())
      @collection.on('reset remove', @render, @)

    serialize: ->
      collection = @collection.toArray()
      { collection, @iconClass, @label, @type, @filterable }

    fetchSuccessful: ->
      @render()
      for item in @startingRunList
        if item.type == @type
          @collection.remove(@collection.get(item.name))
      @spinnerView.remove()
      # set up future sync events, don't want to re-render whole view in the future
      @collection.on('sync', @updateListView, @)

    # if the newly dragged over element doesn't
    droppedEvent: (evt) ->
      filterVal = @$("input").val()
      if filterVal?.length > 0
        @$("ul li:not(:contains('#{@$("input").val()}'))").fadeOut("slow")

    filterEvent: (evt) ->
      evt.preventDefault
      @filter()

    filter: ->
      filterVal = @$("input").val()
      if filterVal.length > 0
        @$("ul li:contains('#{filterVal}')").show()
        @$("ul li:not(:contains('#{filterVal}'))").hide()
      else
        @$("ul li").show()

    afterRender: ->
      @draggableSetup()

    # appends the next page of queried items to the view
    updateListView: (event, response) ->
      for newItem in response.results
        @$("ul").append("<li data-type=#{@type}>#{newItem.name}</li>")
