define (require) ->
  $ = require('jquery')
  Backbone = require('b')
  _ = require('underscore')

  require('jquery.ui/position')

  class Semimodal extends Backbone.Layout
    pulloutClassName: '' # Hook for subclasses that want to customize the pullout after it loses DOM position
    events:
      'click .semimodal-toggle': 'clickToggle' # Some subclasses totally replace this, just an example

    initialize: ->
      @align = @options.align ? 'right'
      @isOpen = false
      _.bindAll(@, 'close', 'clickPullout')

      $(window).on('resize', _.bind(@setMaxHeight, @))

    # Remove global event handler
    cleanupEvents: ->
      $(document).off('click.semimodal', @close)
      $(window).off('resize', _.bind(@setMaxHeight, @))

    cleanup: ->
      @cleanupEvents()
      @$pullout?.remove()
      @$pullout = undefined

    serialize: ->
      {pulloutClassName: @pulloutClassName}

    afterRender: ->
      # Don't fail if afterRender is called more than once (for some reason)
      if ($pullout = @$('.semimodal-pullout')).length
        # Drag the pullout div into body to escape overflow:hidden's, and then move
        # it off screen and make it visible so the first position on click will work.
        @$pullout = $pullout.data('semimodal', @).appendTo('body').css('top', '-1000000px').on('click', @clickPullout).show()
        @trigger('pullout')

    toggle: ->
      @trigger('toggle')
      if @isOpen
        @close()
      else
        @open()

    open: ->
      # Close all other dropdowns
      $('.semimodal-pullout').each (i, elm) ->
        if (widget = $(elm).data('semimodal')) and not (widget is @)
          widget.close()
      if @$pullout and not @isOpen
        @$pullout.show()
        @positionAt(@$el)
        # Install global handler
        $(document).on('click.dropdown', @close)
        @trigger('open')
        @isOpen = true
      @

    close: ->
      if @$pullout and @isOpen
        @$pullout.hide()
        @cleanupEvents()
        @trigger('close')
        @isOpen = false
      @

    clickToggle: (evt) ->
      # Stop bubbling so we don't hit the global handler we are about to make
      evt.stopPropagation()
      evt.preventDefault()
      @toggle()

    clickPullout: (evt) ->
      # A backstop for click events that aren't otherwise caught so we don't close the pullout
      evt.stopPropagation()
      evt.preventDefault()

    positionAt: ($el) ->
      @$pullout.position({my: "#{@align} top", at: "#{@align} bottom", of: $el, collision: 'none'})
      # The position method called above works correctly for the horizontal
      # positioning, which is necessary, but can get the vertical way off
      # (notably in IE.) This adjusts it to the bottom of the clicked
      # element no matter what.
      @$pullout.css('top', $el.offset().top + $el.height())
      # Set the max-height to make sure it's not taller than the viewport
      #
      @setMaxHeight()

    # Set the max-height to make sure it's not taller than the viewport
    setMaxHeight: ->
      @$pullout?.css('maxHeight',
        ($(window).height() - @$pullout.offset().top - 20) + 'px'
      )
