define (require) ->
  Backbone   = require('backbone')
  LeafNode   = require('views/controls/tree/leaf_node')
  ParentNode = require('views/controls/tree/parent_node')
  TreeNode   = require('views/controls/tree/node')
  _          = require('underscore')

  class TreeView extends Backbone.Layout
    # For access in other modules
    @Node: TreeNode
    @LeafNode: LeafNode
    @ParentNode: ParentNode
    @renderChildrenRecursively: false

    tagName: 'ol'
    className: 'treeview'
    leafNodeClass: LeafNode
    parentNodeClass: ParentNode

    initialize: ->
      @root = @options.root ? []
      @root = [@root] unless _.isArray(@root)
      @buildTree()

    buildTree: ->
      @nodes = for object in @root
        @node(object)
      @insertViews({'': @nodes})

    expandAll: ->
      @renderChildrenRecursively = true
      for node in @nodes
        if node.className == 'parent'
          node.renderChildren()
      @$('.treeview-children').show()
      @$('.treeview-toggle-icon').removeClass('fa fa-plus').addClass('fa fa-minus')

    collapseAll: ->
      @$('.treeview-children').hide()
      @$('.treeview-toggle-icon').addClass('fa fa-plus').removeClass('fa fa-minus')

    # A subclass might want to override these
    node: (object) ->
      children = @children(object)
      if children?.length
        new @parentNodeClass({model: object, collection: children, tree: @})
      else
        new @leafNodeClass({model: object, tree: @})

    children: (object) ->
      object.children
