define (require) ->
  $ = require('jquery')
  Backbone = require('b')
  CookbookFileItem = require('models/cookbook_file_item')
  DownloadableFileView = require('views/controls/downloadable_file')
  EmptyFileView = require('views/cookbooks/content/empty_file')
  HighlightedFileView = require('views/controls/highlighted_file')
  T = require('utils/i18n.gettext')
  template = require('j!cookbooks/content')

  class CookbookContentView extends Backbone.Layout
    template: template

    sourceFileMatch: /^(Gemfile|LICENSE|README|CONTRIBUTING|Berksfile)$|\.(rb|erb|lock|json|yaml|yml|sh|bash|js|java|pl|txt|py|xml|conf|config|md|git|gitignore|rdoc|log|sample|feature|seed|graph|html|cron|pm|patch|types|users|css|sass|diff|erl|js|java|perl|php|ps|scala|sql|vb|markdown)$/
    syntaxHighlightMatch: /\.(rb|erb|json|yaml|yml|sh|bash|js|java|pl|py|xmlcss|sass|diff|erl|js|java|perl|pl|php|ps|scala|sql|vb|html)$/
    markdownMatch: /\.(md|markdown)$/

    # Should match COOKBOOK_SEGMENTS constant from
    # https://github.com/opscode/chef_objects/blob/master/src/chef_cookbook_version.erl
    segments: ['attributes', 'definitions', 'files', 'libraries', 'providers',
               'recipes', 'resources', 'root_files', 'templates']

    events:
      'click .listheader': 'toggleFileList'
      'click .file-item': 'displayFile'

    initialize: (@options = {}) ->
      if typeof @model?.displayVersionModel == 'function'
        @cookbookVersion = @model.displayVersionModel()
        @listenTo(@cookbookVersion, 'change', @render)

      @setView('.cookbook-file-source', new EmptyFileView({
        message: T('Please select a file')
      }))

    serialize: ->
      cookbookVersion: @cookbookVersion
      segments: @segments

    toggleFileList: (evt) ->
      evt.preventDefault()
      $(evt.currentTarget).parent('.listing').children('.file-list').toggleClass('collapsed expand')
      $(evt.currentTarget).children('.dropdown-icon').toggleClass('fa-caret-right fa-caret-down')

    displayFile: (event) ->
      baseUrl = @model.url()
      currentTarget = $(event.currentTarget)
      name = currentTarget.text()
      fileType = currentTarget.data('filetype')
      version = @cookbookVersion.get('version')
      downloadLink = new CookbookFileItem({
        baseUrl: baseUrl
        fileType: fileType
        name: name
        version: version
      }).url()
      fileViewOptions = {
        baseUrl: baseUrl
        downloadLink: downloadLink
        fileDownloaderClass: CookbookFileItem
        fileName: name
        fileType: fileType
        markdownFile: @markdownMatch.test(name)
        srcUrl: downloadLink
        syntaxHighlightableFile: @syntaxHighlightMatch.test(name)
        version: version
      }

      if @sourceFileMatch.test(name)
        fileViewClass = HighlightedFileView
      else
        fileViewClass = DownloadableFileView

      event.preventDefault()

      @$('li').removeClass('active')
      currentTarget.addClass('active')

      @setView('.cookbook-file-source',
        new fileViewClass(fileViewOptions)).render()
