define (require) ->
  DatabagItem = require('models/databag_item')
  JSONEditor  = require('views/controls/json_editor')
  Modal       = require('views/controls/modals/base')
  T           = require('utils/i18n.gettext')
  template    = require('j!databag_items/modals/create')

  class CreateDatabagItemModal extends Modal
    title: T('Create Data Bag Item')
    template: template

    bindings: { '[name=id]': 'id' }

    events:
      'click .btn-submit': 'submit'

    initialize: (options) ->
      @model = new DatabagItem
      @databag = options.databag
      @model.collection = @databag.items

      @jsonEditor = new JSONEditor(attribute: 'data', model: @model, rows: 12)
      @setView('.data', @jsonEditor)

      super()

      @model.on('validated', @toggleSubmit)

    serialize: ->
      _.extend(super(), { state: @state, databag_name: @databag.get('name') })

    afterRender: ->
      @submitButton = @$('.btn-submit')
      @bindValidation()
      super()

    submit: (event) ->
      event.preventDefault()
      return if @state and @state != 'error'
      if @jsonEditor.submit()
        @model.validate().then =>
          @state = 'started'
          @render()
          @model.sync('create', @model, {
            url: @model.collection.url()
            success: (data, textStatus, jqXHR) => @onSaved()
            error: (jqXHR, textStatus, errorThrown) => @state = 'error'; @render()
          })

    # Enable or disable the create button based on the validity
    toggleSubmit: (model, result, isValid) =>
      @submitButton.prop('disabled', !isValid || @state == 'started')
