define (require) ->
  Backbone = require('b')
  CreatItemModal = require('views/databag_items/modals/create')
  DataBagItem = require('models/databag_item')
  DataBagItemView = require('views/databag_items/details')
  DeleteItemModal = require('views/databag_items/modals/delete')
  EditItemModal = require('views/databag_items/modals/edit')
  ListView = require('views/databag_items/list')
  dataBagDetailsTpl = require('j!databags/items')
  state = require('state')

  class DataBagItemsView extends Backbone.Layout
    template: dataBagDetailsTpl

    events:
      'click .items a': 'onItemClick'
      'click .create-item': 'onCreateItemClick'

    initialize: (options) ->
      @app = options.app
      @controller = state.get('controller')

    serialize: ->
      {bag: @model.toJSON()}

    onResize: ->
      tabHeight = parseInt(@$el.parents('.tab-content').css('min-height'))
      @$('.databag-items').height(tabHeight - @$('.items-header').outerHeight())
      @listView?.resizeGrid()

    afterRender: ->
      @listView = new ListView
        app: @app
        collection: @model.items
        controller: @controller
        selection: new Backbone.Collection
        actions: [
          {id: 'edit', label: 'Edit', type: 'databag'},
          {id: 'delete', label: 'Delete', type: 'databag'}
        ]

      @listView.selection.on('select', @onItemSelect, @)
      @listView.on('action', @onItemAction, @)

      @setView('#list-pane', @listView).render()

      # Check if an initial item was selected (like on page load) and select it
      @setInitialSelection(@listView.selection)

    onItemSelect: ->
      item = @listView.selection.first()
      @showDataBagItem(item)

    onItemAction: (action) ->
      item = @listView.selection.first()
      @["action_#{action}"].call(@, item)

    action_edit: (item) ->
      new EditItemModal({
        model: item
        app: @app
      })

    action_delete: (item) ->
      new DeleteItemModal({
        model: item
        app: @app
      })

    onCreateItemClick: (e) ->
      e.preventDefault()
      new CreatItemModal(app: @app, databag: @model)

    # If the controller has an initialItemId property set, we want to get that
    # item, put it at the top of the list, and select it.
    setInitialSelection: (selection) ->
      id = @controller.initialItemId
      items = @model.items

      if id
        # Create an empty item from the id
        item = new DataBagItem({ id }, { collection: items })
        # Unset the initialItemId so we don't re-use it when navigating back
        delete @controller.initialItemId
        # Fetch the rest of the properties of the item
        item.fetch().then (fullItem) ->
          # Remove the item from the existing collection if it's there, so we
          # can put it at the top
          items.remove(item, { silent: true })
          # Put it at the top
          items.add(fullItem, { at: 0 })
          # Set the selection to be the item
          selection.reset([item])
          # Trigger events to make it be selected
          selection.trigger('navigate select')

    showDataBagItem: (item) ->
      @$el.find('.items li').removeClass('active').filter("[data-item='#{item.id}']").addClass('active')

      dataBagItem = new DataBagItemView({
        app: @app
        model: item
      })

      @setView('.item', dataBagItem).render()
