define (require) ->
  AddButton = require('views/controls/add_button')
  Backbone = require('b')
  ErrorLine = require('views/controls/alerts/error_line')
  GroupMembersList = require('views/groups/members/list')
  MembersTab = require('views/controls/members/tab')
  router = require('router')
  state = require('state')
  template = require('j!groups/members')

  class GroupMembersTab extends MembersTab
    template: template
    listClass: GroupMembersList

    events:
      'keyup input.filter': 'updateFilter'
      'click .remove-member': 'removeMember'

    initialize: (@options = {}) ->
      @collection = @model.getMembersCollection?()
      @currentOrg = @options.app?.currentOrg
      @currentUser = state.get('user')
      @listenTo(@model, 'sync', @updateCollection, @)
      @listenTo(@model, 'sync', @fetchSuccessful, @)
      @selection = state.get('controller')?.selection

      super

      @listView.grid.model = @model

      @addButton = new AddButton
      @addButton.on('add', @addMember, @)
      @addButton.on('toggle', @clearError, @)
      @setView('.add', @addButton)

      # handle member add events
      @model.on('member_valid', @validMemberAdded, @)
      @model.on('invalid_member_name', @invalidMemberName, @)
      @model.on('currently_member', @memberAlreadyAdded, @)
      @model.on('update_error', @updateError, @)
      @model.on('remove_error', @removeError, @)

    beforeRender: ->
      @model.fetch()

    cleanup: ->
      @model.off(null, null, @)

    clearError: ->
      @$(".error-container").html('')

    addMember: (member) ->
      @clearError()
      @addButton.enableSpinner()

      # will trigger one of the three methods below
      @model.addMember(member)

    validMemberAdded: ->
      @addButton.close()
      @addButton.disableSpinner()

    invalidMemberName: ->
      @addButton.error("Sorry, but that is not a valid member name")
      @addButton.disableSpinner()

    memberAlreadyAdded: ->
      @addButton.error("Sorry, that member is already in this group")
      @addButton.disableSpinner()

    # args = [model, response, options]
    updateError: (args) ->
      response = args[1]
      if response.status == 403
        @addButton.error("Sorry, you do not have permission to update this group")
      else
        @addButton.error("Sorry, something went wrong")
      @addButton.disableSpinner()

    # args = [model, response, options]
    removeError: (args) ->
      response = args[1]
      if response.status == 403
        @setView(".error-container", new ErrorLine({body: "Sorry, you do not have permission to remove the user"})).render()
      else
        @setView(".error-container", new ErrorLine({body: "Sorry, something went wrong while removing the user"})).render()

    removeMember: (evt) ->
      evt.preventDefault()

      @clearError()

      # Make sure there is nothing in the filter
      @$('input.filter').val('')

      $btn = $(evt.target)
      name = $btn.data('name')

      if @currentUser.get('display_name') == name
        rerender = ->
          # Trigger an event on the collection to re-render the list view
          @model.collection.trigger('render')
          # Reset the selection since the current one may no longer be valid
          @selection.reset()
          # route to the group root
          router.navigate("/organizations/#{@currentOrg.get('name')}/groups")
        @model.once('sync', rerender, @)

      @model.removeMember(name,"#{$btn.data('type').toLowerCase()}s")

    applyFilter: (value) ->
      filter = new RegExp("^#{value}", "i")
      allMembers = @model.getMembersCollection()
      filteredMembers = allMembers.filter (item) ->
        item.get('type').match(filter) or item.get('name').match(filter)

      @collection.reset(filteredMembers)
      @listView.grid.updateRemoveButtons()

    updateCollection: ->
      members = @model.getMembersCollection()
      @collection.reset(members.models)
      @updateTotal()
      @listView.grid.updateRemoveButtons()

    updateTotal: ->
      @$(".totals").html("(#{@collection.length} total)")
