define (require) ->
  $ = require('jquery')
  Backbone = require('b')
  router = require('router')
  template = require('j!nodes/environment_editor')

  class EnvironmentEditor extends Backbone.Layout
    template: template

    events:
      'change select': 'selectChange'
      'click .cancel': 'cancel'
      'click .save': 'save'

    initialize: (options) ->
      @app = options.app
      @savedEnv = @model.get('chef_environment')
      @selectedEnv = @savedEnv
      @environments = @app?.currentOrg?.environments

      # Re-render when environments are added
      @environments.on('add', @render, @)

      # Fetch environments if they are empty. Any org should always at least
      # have a _default env, so if they're empty it means they have not yet
      # loaded.
      @environments.fetch() if @environments.length == 0

    serialize: ->
      environments: @environments.pluck('name').sort()
      justSaved: @justSaved
      selectedEnv: @selectedEnv
      savedEnv: @savedEnv
      state: @state

    beforeRender: ->
      if @environments.length == 0 or @saving and not @error
        @state = 'loading'
      else if @error
        @state = 'error'
      else
        @state = 'ready'

    cancel: ->
      @selectedEnv = @savedEnv
      @render()

    save: (event) ->
      @saving = true
      val = @$('select').val()
      @model.save({ chef_environment: val }, {
        validate: false
        success: =>
          @savedEnv = @selectedEnv = val
          @justSaved = true; @saving = false; @error = false; @render()
        error: =>
          @selectedEnv = @savedEnv
          @saving = false; @error = true; @render()
      })
      @render()

    selectChange: (event) ->
      @justSaved = false
      @error = false
      select = event.currentTarget
      isManageEnvironments = $(select.options[select.selectedIndex]).
        data('action') == 'manage'

      if isManageEnvironments
        router.navigate(@app.currentOrg.environments.url(), { trigger: true })
      else
        @selectedEnv = select.value
        @render()
