define (require) ->
  config = require('config')
  $                   = require('jquery')
  ErrorAlert          = require('views/controls/alerts/error')
  ExtremeWarningModal = require('views/controls/modals/extreme_warning_modal')
  SpinnerView         = require('views/controls/spinner')

  class LeaveOrganizationModal extends ExtremeWarningModal
    className: "#{@::className} modal-leave-org"
    title: "Leave Organization"
    initialize: ->
      @title ?= "Leave #{@model.id}"
      @subTitle ?= @model.id
      @bodyText ?= @body()
      @commandText ?= "Leave Organization"
      super()

    body: ->
      "Are you certain you wish to leave the organization <code>#{@model.id}</code>? This action cannot be undone by you, as you would need to be re-invited to the organization to get access."

    events:
      'click .btn-warning': 'leaveOrg'

    # This is used by the RemoveUserModal subclass
    ajaxData: ->
      {}

    userText: ->
      "You"

    ajaxUrl: ->
      "#{@model.url()}/leave"

    successEvent: ->
      window.location = "/organizations"

    leaveOrg: ->
      @setView(".modal-body", new SpinnerView).render()

      # send delete request, if it returns valid,
      # we will inform the user, if it fails,
      # we will inform user why.
      $.ajax
        url: @ajaxUrl()
        type: "post"
        data: @ajaxData()
        success: @successEvent
        error: (data) =>
          @$(".btn-warning").prop("disabled", true)

          viewOpts = {}

          # if this user leaving would cause admins or billing-admins to be
          # empty, display proper error
          group = data.responseJSON?.failure

          if group == "admins" || group == "billing-admins"
            viewOpts.body = """
              #{@userText()} cannot leave this organization because it would
              cause the group <code>#{group}</code> to be empty. Please contact
              <a target='_blank' href=#{config.support_site_url}>Chef
              support</a> if you wish to delete this organization altogether or
              add a new user to the <code>#{group}</code> group and try again.
            """
          else if data.responseJSON?.error
            viewOpts.body = data.responseJSON?.error

          @setView(".modal-body", new ErrorAlert(viewOpts || {})).render()
