define (require) ->
  config = require('config')
  BaseModal = require('views/controls/modals/base')
  CreateFirstOrgModal = require('views/organizations/modals/create_first')
  ErrorLine = require('views/controls/alerts/error_line')
  OrgInvitationModal = require('views/application/modals/org_invitation')
  Spinner = require('views/controls/spinner')
  state = require('state')
  template = require('j!organizations/modals/no_organizations')

  class NoOrganizationsModal extends BaseModal
    template: template
    className: "#{@::className} modal-no-organization"
    modalOptions: { backdrop: 'static' }
    events:
      "click .btn-create": "createNewOrg"
      "click .btn-invite": "manageInvites"
      "click .fa-refresh": "refreshOrgInvites"

    beforeRender: ->
      super
      @setView(".spinner", new Spinner)

    afterRender: ->
      @$(".modal-footer").remove()
      super
      @getOrgInvites()

    refreshOrgInvites: ->
      @$(".fa-refresh").addClass("fa fa-spin")
      @getOrgInvites()

    getOrgInvites: ->
      $.ajax({
        url: "/users/#{state.get('user')?.id}/association_requests",
        type: "get"
        success: (data) =>
          @$(".spinner").html("")
          @$(".fa-refresh").removeClass("fa fa-spin")
          if data.length > 0
            @$(".btn-invite").attr("disabled", false).html("Accept Invite (#{data.length} pending)")
          @$(".buttons").css("display", "block")
        error: (xhr) =>
          @$(".fa-refresh").removeClass("fa fa-spin")
          @setView(".spinner", new ErrorLine({
            body: "There was an error retrieving invitations. Please try again or create an organization."
          })).render()
          @$(".buttons").css("display", "block")
      })

    createNewOrg: ->
      @remove()
      new CreateFirstOrgModal({app: @app})

    manageInvites: ->
      if !@$(".btn-invite").attr("disabled")
        @remove()
        new OrgInvitationModal({app: @app, usernav: null, title: "Organization Invitations", rejectDisabled: true, firstOrgMode: true})

    serialize: ->
      org_creation_enabled: config.org_creation_enabled
      state: null
