define (require) ->
  Backbone = require('b')
  CountsChartView = require('views/reporting/dashboard/counts_chart')
  DashQuery = require('utils/dash_query')
  DashboardControlsView = require('views/reporting/dashboard/dashboard_controls')
  DurationsChartView = require('views/reporting/dashboard/durations_chart')
  ErrorAlertView = require('views/controls/alerts/error')
  InfoAlertView = require('views/controls/alerts/info')
  SummaryChartView = require('views/reporting/dashboard/summary_chart')
  _ = require('underscore')
  router = require('router')
  state = require('state')
  template = require('j!reporting/dashboard')

  class DashboardView extends Backbone.Layout
    template: template

    initialize: ->
      @app = @options.app
      @collection = state.get('controller').collection
      @baseUrl = @options.baseUrl
      @billingTier = @app.currentOrg.reportingBillingTier()

      @dashQuery = new DashQuery(@baseUrl)
      @dashControls = new DashboardControlsView(
        baseUrl: @baseUrl + '/dashboard'
        collection: @collection
        app: @app
        from: moment.unix(@collection.startTime)
        until: moment.unix(@collection.endTime)
        # TODO: billing tiers
        enabled: true
      )
      @dashControls.on('date_changed', @refresh, @)

      colors = ['#2ca02c', '#1f77b4', '#aec7e8']

      @summaryChart = new SummaryChartView(
        colors: colors
      )
      @durationsChart = new DurationsChartView(
        colors: colors
      )
      @countsChart = new CountsChartView(
        colors: colors
      )

      @countsChart.on('pointSelected', @onPointSelected)

    afterRender: ->
      @setView("#dashboard_controls", @dashControls).render()
      @setView('.summary-chart-container', @summaryChart).render()
      @setView('.counts-chart-container', @countsChart).render()
      @setView('.durations-chart-container', @durationsChart).render()

      @dashControls.setDates(moment.unix(@collection.startTime), moment.unix(@collection.endTime))
      @refresh(@collection.startTime, @collection.endTime)

    refresh: (start, end) ->
      router.navigate(@baseUrl + '/dashboard?endTime=' + end + '&startTime=' + start)
      @dashControls.renderSpinner()
      @removeView('.dashboard-message')

      # Only have one handler for the getDash* methods, because on success or failure
      # of either function we want to render the page. This is because on failure of one function
      # (meaning a 500 is received from the server) we still want to render the other graphs on
      # the page. In the case of a 500 error, empty data will be returned.
      #
      # Is there a way to tell the difference between no data and a 500 so the user can get a better message?
      #
      $.when(
        @dashQuery.getDashCountsData(start,end).then(
          @setCountsChartData
        )
        @dashQuery.getDashDurationsData(start,end).then(
          @setDurationsChartData
        )
      ).then( @$('.dashboard-charts').show() ).always(@dashControls.removeSpinner)

    # The fat arrow (=>) matters, so that the function is bound to the correct
    # this context when used in callbacks
    setCountsChartData: (data) =>
      @summaryChart.setData(data)
      @countsChart.setData(data)

    # The fat arrow (=>) matters, so that the function is bound to the correct
    # this context when used in callbacks
    setDurationsChartData: (data) =>
      @durationsChart.setData(data)

    onPointSelected: (point) =>
      fromSeconds = moment(point.start_time).zone(0).unix()
      untilSeconds = moment(point.end_time).zone(0).unix()
      status = point.status
      url = @baseUrl + '/runs?endTime=' + untilSeconds + '&startTime=' + fromSeconds + '&status=' + status
      router.navigate(url, { trigger: true })

