define (require) ->
  countsChartTpl = require('j!reporting/dashboard/counts_chart')
  Backbone = require('b')
  nv = require('nvd3')
  _ = require('underscore')

  tooltip = (key, x, y, e, graph) ->
    [
      '<h3>' + key + '</h3>',
      '<dl>',
        '<dt>Range</dt>',
        '<dd>',
          moment(e.point.start_time).zone(0).format('MMM D HH:mm'),
          ' - ',
          moment(e.point.end_time).zone(0).format('MMM D HH:mm'),
        '</dd>',
        '<dt>Count</dt>',
        '<dd>' + y + '</dd>',
      '</dl>'
    ].join('\n')

  class CountsChartView extends Backbone.Layout
    template: countsChartTpl

    events:
      'click .legend span': 'onLegendClick'

    initialize: ->
      @data = []

    afterRender: ->
      nv.addGraph =>
        @data = []
        @chart = nv.models.lineChart().options(
          x: (d, i) ->
            moment(d.start_time).zone(d.start_time).unix()
          y: (d, i) ->
            d.runcount
          showLegend: false
          showXAxis: true
          showYAxis: true
          transitionDuration: 250
          color: @options.colors
          tooltipContent: tooltip
        )
        @chart.xAxis.options(
          axisLabel: 'Time'
          tickFormat: (d, i) -> moment.unix(d).zone(0).format('MMM D HH:mm')
          scale: d3.scale.ordinal()
          showMaxMin: false
        )
        @chart.yAxis.options(
          axisLabel: 'Counts'
          axisLabelDistance: 35
          tickFormat: d3.format(',')
          showMaxMin: false
        )
        d3.select(@$('.counts-chart svg').get(0)).datum(@data).call(@chart)
        nv.utils.windowResize(@chart.update)
        @chart.lines.dispatch.on('elementClick.point', @onPointClick)
        @chart

    formatData: (rawData) ->
      data = []
      if (rawData)
        statuses = ['success', 'failure', 'aborted']
        data = d3.nest()
          .key((d) -> d.status)
          .sortKeys((a, b) -> statuses.indexOf(a) - statuses.indexOf(b))
          .entries(rawData)
      data

    setData: (data) ->
      @clearOldData(data)
      currentData = @data
      newData = @formatData(data)

      # Clear the currentData array
      currentData.length = 0

      # Replace the currentData array contents with the contents of newData
      # The preserves the pointer to currentData, which is used by nvd3
      Array.prototype.push.apply(currentData, newData)

      if (@chart)
        @chart.yDomain(@getYDomain(data))
        @chart.update()

    clearOldData: (data) ->
      # Work around this issue in nvd3 https://github.com/novus/nvd3/issues/349
      # The issue is fixed in the dev branch of nvd3 at the time of this writing
      # so a future upgrade of the library will make this code unnecessary
      # Need to clear the chart if the data is empty, as otherwise any old data
      # will be displayed with the no data message over it
      if data.length == 0
        d3.select(@$('.counts-chart svg').get(0)).html("")

    getYDomain: (data) ->
      [0, d3.max(data, (d) -> d.runcount * 1.1)]

    onLegendClick: (e) ->
      $items = @$('.legend > span')
      $current = $(e.currentTarget)
      
      $current.toggleClass('disabled')

      disabled = @$('.legend .disabled').map((i, el) ->
        $.trim($(el).text())
      ).get()

      if (disabled.length == $items.length)
        $items.removeClass('disabled')
        _(@data).forEach((d) -> 
          d.disabled = false
        )
      else
        _(@data).forEach((d) -> 
          d.disabled = _(disabled).contains(d.key)
        )

      enabledData = _(@data).reject((d) -> d.disabled)
      rawEnabledData = []
      _(enabledData).forEach((d) ->
        rawEnabledData = rawEnabledData.concat(d.values)
      )

      @chart.yDomain(@getYDomain(rawEnabledData))
      @chart.update()

    onPointClick: (e) =>
      @trigger('pointSelected', e.point)
