define (require) ->
  Backbone = require('b')
  daterangepicker = require('daterangepicker')
  template = require('j!reporting/dashboard/dashboard_controls')

  class DashboardControlsView extends Backbone.Layout
    template: template

    initialize: (options) ->
      @baseUrl = options.baseUrl
      @from = options.from
      @until = options.until
      @enabled = options.enabled

    afterRender: () ->
      if @enabled
        @renderPicker()
      else
        @$(".date-range").addClass("disabled")

    renderPicker: () ->
      options =
        startDate: @from
        endDate: @until
        maxDate: moment()
        minDate: moment().subtract(1, 'year')
        ranges:
          'Last day': [moment().subtract(1, 'day'), moment()]
          'Last week': [moment().subtract(1, 'week'), moment()]
          'Last month': [moment().subtract(1, 'month'), moment()]
          'Last 3 months': [moment().subtract(3, 'month'), moment()]
        applyClass: 'btn-primary'

      @$('.date-range').daterangepicker(options, @onRangeChange)

    onRangeChange: (start, end) =>
      @setDates(start, end)
      @trigger('date_changed', @fromSeconds, @untilSeconds)

    renderSpinner: =>
      @$(".small-spinner").html('<i class="fa fa-spinner fa-spin fa-large"></i>')

    removeSpinner: =>
      @$(".small-spinner").html("")

    setDates: (start, end) ->
      @from = start.startOf("day")
      @until = end.endOf("day")

      # If the until time is in the future, use now instead
      @until = moment() if @until.isAfter()

      @range = "#{@from.format('MM/DD/YYYY')} - #{@until.format('MM/DD/YYYY')}"
      @fromSeconds = @from.zone(0).unix()
      @untilSeconds =  @until.zone(0).unix()
      @$('.date-range').text(@range)
