define (require) ->
  summaryChartTpl = require('j!reporting/dashboard/summary_chart')
  Backbone = require('b')
  nv = require('nvd3')
  _ = require('underscore')

  class SummaryChartView extends Backbone.Layout
    template: summaryChartTpl

    initialize: ->
      @data = []
 
    afterRender: ->
      nv.addGraph =>
        @data = []
        @chart = nv.models.discreteBarChart().options(
          x: (d, i) ->
            d.status
          y: (d, i) ->
            d.percentage
          showLegend: false
          showControls: false
          showValues: true
          tooltips: false
          valueFormat: d3.format('%')
          yDomain: [0, 1]
          transitionDuration: 250
          color: @options.colors
          margin:
            top: 10
            right: 0
            bottom: 20
            left: 0
        )
        @chart.xAxis.axisLabel('Status')
        @chart.yAxis.axisLabel('Percentage').axisLabelDistance(35).tickFormat(d3.format('%'))
        d3.select(@$('.summary-chart svg').get(0)).datum(@data).call(@chart)
        nv.utils.windowResize(@chart.update)
        @chart

    formatData: (rawData) ->
      data = []

      if (rawData)
        statuses = ['success', 'failure', 'aborted']
        
        total = _(rawData).reduce((memo, bucket) -> 
          memo + bucket.runcount
        , 0)

        values = _(rawData).chain()
          .groupBy('status')
          .map((group, status) ->
            groupTotal = _(group).reduce((memo, bucket) -> 
              memo + bucket.runcount
            , 0)
            percentage = (groupTotal / total) * 100
            {status: status, total: groupTotal, percentage: percentage}
          )
          .sortBy((group) ->
            statuses.indexOf(group.status)
          )
          .value()

        # Modified from http://stackoverflow.com/a/13485888/319074 to keep original sort order
        roundToTotal = (nums, total) ->
          offset = total - _(nums).reduce((actual, x) ->
            actual + Math.round(x)
          , 0)

          numsWithIndex = _(nums).map((x, i) ->
            [x, i]
          )

          _.chain(numsWithIndex).sortBy((x) ->
            Math.round(x[0]) - x[0]
          ).map((x, i) ->
            [Math.round(x[0]) + (offset > i) - (i >= (nums.length + offset)), x[1]]
          ).sortBy((x, i) ->
            x[1]
          ).map((x, i) ->
            x[0]
          ).value()

        roundedPercentages = roundToTotal(_(values).pluck('percentage'), 100)

        _(roundedPercentages).forEach((percentage, index) ->
          values[index].percentage = percentage / 100
        )

        data = [
          {
            'key': 'summary'
            'values': values
          }
        ]

      data

    setData: (data) ->
      @clearOldData(data)
      currentData = @data
      newData = @formatData(data)

      # Clear the currentData array
      currentData.length = 0

      # Replace the currentData array contents with the contents of newData
      # The preserves the pointer to currentData, which is used by nvd3
      Array.prototype.push.apply(currentData, newData)

      if (@chart)
        @chart.update()

    clearOldData: (data) ->
      # Work around this issue in nvd3 https://github.com/novus/nvd3/issues/349
      # The issue is fixed in the dev branch of nvd3 at the time of this writing
      # so a future upgrade of the library will make this code unnecessary
      # Need to clear the chart if the data is empty, as otherwise any old data
      # will be displayed with the no data message over it
      if data.length == 0
        d3.select(@$('.summary-chart svg').get(0)).html("")

