define (require) ->
  ScrollGrid = require('views/controls/scroll_grid')
  RunDetailsModal = require('views/controls/modals/run_details')
  InformationModal = require('views/controls/modals/information')
  DiffModal = require('views/reporting/runs/diff')
  Backbone = require('b')
  $ = require('jquery')
  T = require('utils/i18n.gettext')

  class RunStepsGridView extends Backbone.Layout
    className: 'run-steps-grid'
    events:
      'click .fa-eye-open' : 'handleRowClickEvent'
      'click .fa-file-alt' : 'handleDiffClickEvent'
      'mouseover .slick-row' : 'handleMouseOver'
      'mouseout .slick-row'  : 'handleMouseOut'
      'click .slick-cell'     : 'overrideParent'
    divFormatter = (row, cell, value, columnDef, dataContext) ->
      """<div class=#{columnDef.id}>#{value}</div>"""
    columns: [
      {
        id: "step",
        name: T("Step"),
        field: "index",
        width: 80,
        formatter: divFormatter,
      },
      {
        id: "type",
        name: T("Type"),
        field: "type",
        width: 80,
      },
      {
        id: "name",
        name: T("Name"),
        field: "name",
        width: 80,
      },
      {
        id: "action",
        name: T("Action"),
        field: "result",
        width: 80,
      },
      {
        id: "duration",
        name: T("Duration"),
        field: "duration",
        width: 80,
      },
      {
        id: "diff",
        name: T("Diff"),
        field: "uri",
        width: 80,
        formatter: (row, cell, value, columnDef, dataContext) ->
          # since we currently have no way of knowing if a resouce has a diff before we query for it, we only
          # want to be able to query for resouce types that have a diff
          if _.contains(["file", "template", "cookbook_file", "remote_file"], dataContext.get("type"))
            split = value.split("/")
            """<div class='diff-button' data-node_id='#{split[7]}' data-run_id='#{split[9]}' data-run_step_index='#{split[10]}'></div>"""
      },
      {
        id: "parameters",
        name: T("Parameters"),
        field: null,
        width: 80,
        formatter: (row, cell, value, columnDef, dataContext) -> """<div class='click-here-div'></div>"""
      },
    ]

    initialize: ->
      @collection = @options.collection
      @parent_div_str = @options.parent_div_str
      @app = @options.app
      @base_url = @app.organizations.last().url()
      @scroll_grid = new ScrollGrid({
        collection: @collection,
        columns: @columns,
        gridOptions: {rowHeight: 25},
      })
      @resize()

    afterRender: ->
      @resize()

    resize: ->
      @$el.height( $(@parent_div_str).height() )
      @$el.width( $(@parent_div_str).width() )
      @setView(@scroll_grid).render()

    handleRowClickEvent: (e) ->
      row = @$(e.target).parents(".slick-row")[0]
      run_step_details = $(row).append("<div class='run-step-details'><\div>")[0]

      # uses 1 based indexing for better UX, convert to 0 based for collection
      currModel = @collection.at(parseInt($(@$(row).find(".step")[0]).html()) - 1)

      modal = new RunDetailsModal({app: @app, controller: null, run_step: currModel})
      modal.on("modalCloseEvent", @handleModalCloseEvent)

    handleDiffClickEvent: (e) ->
      # query server for file diff
      params = {
        diff: true,
        run_id: @$(e.target).data("run_id"),
        node_id: @$(e.target).data("node_id"),
        resource_index: @$(e.target).data("run_step_index"),
      }
      $.ajax @base_url + "/runs_query",
        dataType: "json"
        data: params
        success: (data, status, xhr) =>
          if data.diff_data.length < 1
            new InformationModal({app: @app, collection: null, title: T("File Diff"), message: T("There was no content change for this resource.")})
          else
            new DiffModal({app: @app, collection: null, title: T("File Diff"), message: null, diff: data.diff_data}).render()

    handleModalCloseEvent: =>
      @scroll_grid.resetSelection()

    handleMouseOver: (e) ->
      row = @$(e.target).parents(".slick-row")[0]
      @$(row).find(".click-here-div").eq(0).addClass('fa fa-eye-open')
      @$(row).find(".diff-button").eq(0).addClass('fa fa-file-alt')

    handleMouseOut: (e) ->
      row = @$(e.target).parents(".slick-row")[0]
      @$(row).find(".click-here-div").eq(0).removeClass('fa fa-eye-open')
      @$(row).find(".diff-button").eq(0).removeClass('fa fa-file-alt')

    overrideParent: (e) ->
      e.preventDefault()
      @$(".selected").removeClass("selected")


