define (require) ->
  $ = require('jquery')
  Backbone = require('b')
  RunStep = require('models/run_step')
  RunSteps = require('collections/run_steps')
  RunStepsGridView = require('views/reporting/run_steps/grid')
  Spinner = require('views/controls/spinner')
  T = require( 'utils/i18n.gettext')
  _ = require('underscore')
  router = require('router')
  template = require('j!reporting/runs/details')

  class RunsDetailsView extends Backbone.Layout
    template: template
    initialized: false
    initialize: ->
      @app = @options.app
      @timeout = 500
      @baseUrl = "#{@app.organizations.last().url()}/#{@model.get('node_name')}"
      @nodeUrl = "#{@app.organizations.last().url()}/nodes/#{@model.get('node_name')}"
      @org = @options.app.organizations.last()
      @collection = new RunSteps([], {organization: @org})
      _.bindAll(@, 'resize')
      @resizeBound = false

    events:
      "click .node_link": "clickNodeLink"

    clickNodeLink: (evt) ->
      router.navigate($(evt.target).attr("value"), { trigger: true })

    onResize: ->
      @resizeEvent()

    resize: =>
      if @collection.length > 0
        clearTimeout(@timeout)
        @timeout = setTimeout(=>
          @resizeEvent()
        , 200)

    resizeEvent: =>
      margin_bot = 38
      $("#details_table").height($(document).height() - $("#details_table").offset().top - $(".step_header").height() - margin_bot)
      width_offset = 40
      $("#details_table").width($(document).width() - $("#sidebar").width() - width_offset)
      if @initialized
        @step_grid.resize()
        @setView("#details_table", @step_grid).render()

    serialize: ->
      {
        node: @model.get("node_name"),
        status: @model.get("status"),
        resource_summary: @model.get("resource_summary"),
        node_url: @nodeUrl,
      }
    beforeRender: ->
      unless @resizeBound
        $(window).bind('resize', @resize)
        @resizeBound = true

    cleanup: ->
      if @resizeBound
        $(window).unbind('resize', @resize)
        @resizeEvent = false

    afterRender: ->
      @resizeEvent()
      @setView(".details_table_content", new Spinner).render()

      params = {
        node_id: @model.get("node_name"),
        details: true,
        startTime:  null,
        endTime: null,
        start_row: 0,
        num_rows: 20,
        run_id: @model.id,
      }

      @collection.reset([])
      @collection.set_params(params)

      @collection.fetch().then(@successfulServerResponse, @errorServerResponse)

    successfulServerResponse: =>
      if @collection.length > 0
        @step_grid = new RunStepsGridView({collection: @collection, parent_div_str: "#details_table", app: @app, run_id: @run_id})
        @setView("#details_table", @step_grid).render()
        @initialized = true
      else
        @$("#details_root").html(T("There were no resources executed for this run."))

    errorServerResponse: (xhr, textStatus, errorThrown) =>
      if xhr.status == 404
        message = 'It looks like this node no longer exists'
      else
        message = errorThrown

      @$("#details_root").html('<div class="alert alert-error empty"><i class="fa fa-warning-sign"></i>&nbsp;Error: ' + message + '.</div>')
