define (require) ->
  $ = require('jquery')
  ErrorAlertView = require('views/controls/alerts/error')
  ListView = require('views/controls/list')
  RunsListHeader = require('views/reporting/runs/list_header')
  MetaHeader = require('views/reporting/runs/meta_header')
  T = require('utils/i18n.gettext')

  class RunsListView extends ListView
    actionColumn: false
    headerClass: MetaHeader

    initialize: ->
      @app = @options.app
      @baseUrl = @app.currentOrg.url() + "/runs"
      super

    showError: (model, xhr) ->
      if xhr.status && xhr.status == 404
        error = new ErrorAlertView
        error.body = xhr.responseJSON.error
        @setView('.grid', error).render()
      else
        super

    afterRender: ->
      super
      # TODO: fix after billing tiers implemented
      if @app.currentOrg.reportingBillingTier() == 'free'
        @$(".reporting-coming-soon").css("display", "block")

    columns: [
      {
        id: "start_time",
        name: T("Start Time"),
        field: "start_time",
        formatter: (row, cell, value, columnDef, dataContext) ->
          value.utc().format("YYYY-MM-DD HH:mm:ss")
      },
      {
        id: "finish_time",
        name: T("Finish Time"),
        field: "end_time",
        formatter: (row, cell, value, columnDef, dataContext) ->
          if dataContext.attributes.status == "started"
            return ""
          else
            return value.utc().format("YYYY-MM-DD HH:mm:ss")
      },
      {
        id: "node",
        name: T("Node"),
        field: "node_name"
      },
      {
        id: "duration",
        name: T("Duration"),
        field: "duration"
      },
      {
        id: "total_resources",
        name: T("Resources Executed / Total"),
        field: "resource_summary",
        formatter: (row, cell, value, columnDef, dataContext) ->
          if dataContext.attributes.status == "started"
            return ""
          else
            return value
      },
      {
        id: "status",
        name: T("Status"),
        field: "status",
        formatter: (row, cell, value, columnDef, dataContext) ->
          if value == "success"
            """<span><i class='fa fa-ok-sign pull-left'></i><div class='pull-left status-descriptor'>#{value}</div></span>"""
          else if value == "failure"
            """<span><i class='fa fa-minus-sign pull-left'></i><div class='pull-left status-descriptor'>#{value}</div></span>"""
          else if value == "aborted"
            """<span><i class='fa fa-exclamation-sign pull-left'></i><div class='pull-left status-descriptor'>#{value}</div></span>"""
          else if value == "started"
            """<span><i class='fa fa-retweet pull-left'></i><div class='pull-left status-descriptor'>#{value}</div></span>"""
          else
            """<span>#{value}</span>"""
      },
    ]
