define (require) ->
  $ = require('jquery')
  Backbone = require('b')
  DashboardControlsView = require('views/reporting/dashboard/dashboard_controls')
  RunsListHeader = require('views/reporting/runs/list_header')
  moment = require('moment')
  router = require('router')
  template = require('j!reporting/runs/meta_header')

  # this class contains the date header with the traditional list header search box embedded
  class MetaHeader extends Backbone.Layout
    template: template

    initialize: ->
      @listView = @options.listView
      @collection = @options.collection

      @dashControls = new DashboardControlsView(
        baseUrl: @listView.baseUrl
        collection: @collection
        from: moment.unix(@collection.server_api.startTime)
        until: moment.unix(@collection.server_api.endTime)
        # TODO: billing tiers
        enabled: @listView.app.currentOrg.reportingBillingTier() != 'free'
      )
      @dashControls.on('date_changed', @handleRangeChange, @)

      @listHeaderView = new RunsListHeader({collection: @collection, listView: @listView})

      @setView("#search-container",   @listHeaderView)
      @setView("#dashboard_controls", @dashControls)

    updateEnvDropdown: =>
      @$("#environments-select").append($('<option></option>').attr("value", "null").text("All Environments"))
      for model in @collection.organization.environments.models
        @$("#environments-select").append($('<option></option>').attr("value", model.id).text(model.id))
      @$(".fa-spinner").remove()
      @$("#environments-select").removeAttr('disabled')

    cleanup: ->
      @listHeaderView.off(null, null, @)
      router.off("route:runs:index", null, @)
      router.off("route:runs:show", null, @)
      @collection.organization.environments.off("sync", null, @)

    serialize: ->
      {search_enabled: @collection.organization.isFeatureEnabled('reporting_solr_search')}

    events:
      "change #environments-select" : "handleEnvChange"
      "change #status-select"       : "handleStatusChange"
      "click  #dashboard-link"      : "handleDashboardClick"

    afterRender: ->
      @setProperUiFilterState()

      # pull down the environments
      @collection.organization.environments.on("sync", @updateEnvDropdown, @)
      if @collection.organization.environments.length == 0
        @collection.organization.environments.fetch()
      else
        @updateEnvDropdown()

      if @collection.server_api.endTime and @collection.server_api.startTime
        @dashControls.setDates(moment.unix(@collection.server_api.startTime), moment.unix(@collection.server_api.endTime))

    setProperUiFilterState: ->
      if @collection.server_api.env? and @collection.server_api.env != "null"
        @$("#environments-select").val(@collection.server_api.env)
      else
        @$("#environments-select").val("null")

      if @collection.server_api.status? and @collection.server_api.status.length > 0
        $("#status-select").data("init_val", @collection.server_api.status)
        $("#status-select").children("option").each( ->
          init_val = $("#status-select").data("init_val")
          if $(@).val() == init_val
            $(@).prop("selected", true)
            $("#status-select").val(init_val)
        )
      else
        $("#status-select").val("all")

    handleEnvChange: ->
      if @$("#environments-select").val() != "null"
        @collection.server_api.env = @$("#environments-select").val()
      else # if no env selected, delete param
        delete @collection.server_api.env
      @listHeaderView.resetListAndCollectionState()

    handleStatusChange: ->
      if $("#status-select").val() != "all"
        @collection.server_api.status = $("#status-select").val()
      else
        delete @collection.server_api.status
      @listHeaderView.resetListAndCollectionState()

    handleRangeChange: (start, end) ->
      @collection.set_time_params(start, end)
      @listHeaderView.resetListAndCollectionState()

    # List views expect these methods to exist on their header classes
    resetListAndCollectionState: ->
    updateTitleText: ->


