define ['b', 'models/search', 'j!search/show'], (Backbone, Search, tpl) ->
  class SearchBuilderView extends Backbone.Layout
    template: tpl

    initialize: ->
      @_viewsByCid = {}
      @model ?= new Search
      @model.on('change', @render, @)
      @model.get('clauses').on('add', @render, @)
      @model.get('clauses').on('remove', @removeSubView, @)
      @model.get('clauses').on('add remove', @updatedSearchBuilder, @)

    serialize: ->
      {search: @model}

    afterRender: ->
      @model.get('clauses').each (clause, index) =>
        elm = @$(".clause[data-index=#{index}] .input")
        if @_viewsByCid[clause.cid]
          @_viewsByCid[clause.cid].setElement(elm).render()
        else
          @_viewsByCid[clause.cid] = new clause.View({model: clause, el: elm})

    events:
      'change .add-filter select': 'addClause'
      'click .remove': 'removeClause'
      'change .search-mode': 'changeMode'
      'change .comparator select': 'changeComparator'

    # Event handlers
    addClause: (evt) ->
      elm = $(evt.target)
      if elm.val()
        @model.addClause(elm.val())
        @trigger('add:clause')
        elm.val('')

    removeClause: (evt) ->
      elm = $(evt.target).parents('.clause')
      try
        @trigger('remove:clause')
        @model.removeClause(elm.data('index'))
      catch err
        throw err unless err instanceof @model.SearchError

    changeMode: (evt) ->
      elm = $(evt.target)
      @model.set({matchAny: elm.val() == 'any'})
      true

    changeComparator: (evt) ->
      elm = $(evt.target)
      index = elm.parents('.clause').data('index')
      clause = @model.get('clauses').at(index)
      comparator = @model.Comparators[elm.val()]
      clause.set({comparator: comparator}) if clause and comparator

    removeSubView: (clause) ->
      delete @_viewsByCid[clause.cid]
      @render()

    updatedSearchBuilder: ->
      @trigger('updated:searchbuilder')
