define (require) ->
  config = require('config')
  Backbone            = require('b')
  ChangePasswordModal = require('views/users/modals/change_password')
  ErrorLine           = require('views/controls/alerts/error_line')
  Spinner             = require('views/controls/spinner')
  SuccessLine         = require('views/controls/alerts/success_line')
  UserPassword        = require('models/user_password')

  require('bootstrap-tooltip')

  class UserProfileView extends Backbone.Layout
    template: require('j!users/profile')

    bindings:
      '#email': 'email'
      '#first_name': 'first_name'
      '#middle_name': 'middle_name'
      '#last_name': 'last_name'

    events:
      'click nav .change-password': 'changePassword'
      'submit form': 'save'

    initialize: ->
      @change_password_disabled = config.external_auth.enabled
      @model.on('validated', @toggleSubmit)

    afterRender: ->
      @$('nav a').tooltip({ placement: 'bottom' })
      @bindValidation()

    save: (event) ->
      event.preventDefault()

      @model.validate().then =>
        @setView(".messages", new Spinner).render()
        @model.sync('update', @model, {
          success: (xhr) =>
            @setView(".messages", new SuccessLine({
              body: xhr.message or "Your profile was successfully updated." })).render()
          error: (xhr) =>
            @setView(".messages", new ErrorLine({
              body: "Error: " +
                xhr.responseJSON?.error or
                'There was a problem. Please try again later.' })).render()})

    serialize: ->
      account_management_link_enabled: !config.private_chef && @model.id == config.user.username
      change_password_disabled: @change_password_disabled
      change_password_disabled_message: "Passwords must be changed through #{config.external_auth.adjective}"
      chef_account_management_url: config.chef_account_management_url
      model: @model

    changePassword: ->
      unless @change_password_disabled
        new ChangePasswordModal({ model: new UserPassword({}, { user: @model }) })

    # Enable or disable the create button based on the validity
    toggleSubmit: (model, result, isValid) =>
      @$('.btn-primary').prop('disabled', !isValid)
