class AccountLinksController < ApplicationController
  layout "outside"
  skip_before_action :check_for_password_reset
  skip_before_action :check_for_session_expiration
  skip_before_action :require_login

  # GET /link-account
  def show
    # You shouldn't be here if this isn't in your session
    redirect_to login_path and return if session[:link_account_authenticated_user].nil?

    @username_to_link = session[:link_account_authenticated_user]['username']

    begin
      @existing_user = server_root.get_rest("users/#{@username_to_link}")
    rescue Net::HTTPServerException => e
      raise if e.response.code.to_i != 404
    end
  end

  # POST /link-account
  #
  # For when a user does not have an existing Chef account and wants to link a
  # new one.
  def create
    user_to_link = session[:link_account_authenticated_user]
    begin
      # If LDAP bindings do not provide displayname we will fall back to
      # the cn binding (common name) if available (even if it's unknown)
      if user_to_link['display_name'] == 'unknown' && user_to_link.has_key?('common_name')
        user_to_link['display_name'] = user_to_link['common_name']
      end

      server_root.post_rest('users', user_to_link)
      session[:username] = user_to_link['username']
      redirect_to root_path and return
    rescue Net::HTTPServerException => e
      if e.response.code.to_i == 409
        flash[:alert] = 'A conflict was detected. Either this user has already been linked to you, or a user exists with the same email.  Try logging in again, and if that doesn\'t work, contact your administrator.'
        redirect_to login_path and return
      else
        raise
      end
    ensure
      session[:link_account_authenticated_user] = nil
    end
  end

  # PUT /link-account
  #
  # For when a user has an existing Chef account for which they have a username
  # and password and want to link it to the externally auth'd one
  def update
    if params[:username].blank? || params[:password].blank?
      flash[:alert] = 'Please enter a username and password.'
      redirect_to account_links_path and return
    end

    begin
      # Find the user in the local db
      chef_user = server_root.post_rest('authenticate_user', {
        username: params[:username],
        password: params[:password],
        local: true
      })['user']
      # Set the users ext auth id property
      chef_user[:external_authentication_uid] =
        session[:link_account_authenticated_user]['external_authentication_uid']
      # Update the user
      server_root.put_rest("users/#{params[:username]}", chef_user)
      # Clear the session var
      session[:link_account_authenticated_user] = nil
      # Set the user to be logged in
      session[:username] = params[:username]
      # Proceed
      redirect_to root_path and return
    rescue Net::HTTPServerException => e
      if e.response.code.to_i == 401
        flash[:alert] = 'Incorrect username or password.'
        redirect_to account_links_path and return
      else
        raise
      end
    end
  end
end
