class AclsController < ApplicationController

  ACL_CONTAINERS_REGEX = Regexp.new("^(#{ACL_CONTAINERS.join('|')})(/[^/]+)?$")
  ACL_ORDER = {'group' => 0, 'user' => 1, 'client' => 2}
  SUPERUSER = 'pivotal'

  def show
    # Fetch the raw ACL
    raise "Invalid ACL #{params[:id]}" unless ACL_CONTAINERS_REGEX.match(params[:id])
    raw_acl = server.get_rest("#{params[:id]}/_acl")
    # Fetch the list of users so we can classify users vs. clients
    raw_users = server.get_rest('users')
    users = raw_users.inject({}) do |memo, raw_user|
      memo[raw_user['user']['username']] = true
      memo
    end
    # Flip the data into the format used in the UI
    inverted_data = {'actors' => {}, 'groups' => {}}
    raw_acl.each do |ace, data|
      data.each do |data_type, names|
        inverted_type = inverted_data[data_type]
        names.each do |name|
          inverted_type[name] ||= {'id' => "#{data_type}-#{name}", 'name' => name, 'type' => data_type == 'groups' ? 'group' : users[name] ? 'user' : 'client'}
          inverted_type[name][ace] = true
        end
      end
    end
    # Users shouldn't see and can't edit ACLs for the superuser
    inverted_data['actors'].delete(SUPERUSER)
    # Build the final ACL data
    @acl = inverted_data['groups'].values.concat(inverted_data['actors'].values)
    @acl.sort! do |a, b|
      n = ACL_ORDER[a['type']] <=> ACL_ORDER[b['type']]
      n == 0 ? a['id'] <=> b['id'] : n
    end
    render :json => @acl
  end

  # Doesn't really create anything, just used from JS to get the object type
  def create
    name = params[:name]
    if server.get_rest('groups')[name]
      type = 'group'
    elsif server.get_rest('users').any? {|user| user['user']['username']==name }
      type = 'user'
    elsif server.get_rest('clients')[name]
      type = 'client'
    end
    if type
      render :json => {:id => "#{type}-#{name}", :name => name, :type => type}
    else
      render :json => {:error => true}, :status => 404
    end
  end

  def update
    # Fetch the raw ACL
    raise "Invalid ACL #{params[:id]}" unless ACL_CONTAINERS_REGEX.match(params[:id])
    acl = server.get_rest("#{params[:id]}/_acl")
    ace = acl[params[:ace]]
    raise "Invalid ACE #{params[:ace]}" unless ace
    ace_entry = ace[params[:type] == 'group' ? 'groups' : 'actors']
    if params[:value] == 'true'
      ace_entry << params[:name] unless ace_entry.include?(params[:name])
    else
      ace_entry.delete(params[:name])
    end
    server.put("#{params[:id]}/_acl/#{params[:ace]}", {params[:ace] => ace})
    render json: {ok: true}
  end
end
