require "object_retriever"

class ChefObjectController < ApplicationController

  USAG = /\A\h{32}\z/

  # GET /type
  # Return a list of objects in a format Backbone.Paginator can use
  def index
    @objects = server.get_rest(index_url)
    @total = @objects.length
    if not params[:fetch_all]
      per_page = [params.fetch(:count, 20).to_i, 1].max
      from = [params[:page].to_i, 0].max * per_page
      to = from + per_page - 1
    else
      from = 0
      to = @objects.length
    end
    @objects = get_objects_details(from, to)
    respond_to do |format|
      format.html { render_app }
      format.json { render :json => { :total => @objects.length, :results => @objects } }
    end
  end

  # GET /type/:id
  # Return a single object, can be used with Backbone loading
  def show
    @obj = object_to_hash(object_for_id(params[:id]))
    respond_to do |format|
      format.html { render_app }
      format.json { render :json => @obj }
    end
  end

  def destroy
    begin
      server.delete_rest(show_url(params[:id]))
    rescue Net::HTTPServerException => e
      raise e unless e.response.code.to_i == 403
      render :json => {"success" => false, :message => "You do not have permission to delete this object."}, :status => 403
      return
    end
    render :json => {"success" => true}, :status => 204
  end

  protected

  # Helper to convert a Chef object into a Hash for later rendering to JSON
  def object_to_hash(obj)
    obj.to_hash.update(:access => object_matched_aces(obj))
  end

  def object_matched_aces(obj)
    name = obj.respond_to?(:name) ? obj.name : obj['name']
    fetch_matched_aces(show_url(name))
  end

  # Helper to return an object given its ID, allows short circuiting the server in some cases
  def object_for_id(id)
    ObjectRetriever.get_object(id: id, url: show_url(id), server: server)
  end

  # Helper to get the Chef API URL for the list of all objects
  def index_url
    raise "Subclasses must provide an implementation for #{index_url}"
  end

  # Helper to get the Chef API URL for a single object given its ID
  def show_url(id)
    "#{index_url}/#{id}"
  end

  def get_objects_details(from, to)
    (@objects.keys[from..to] || []).map {|id| object_to_hash(object_for_id(id)) }
  end

end
