require 'exception_helper'

class ChefSolrSearchableObjectController < ChefObjectController
  rescue_from ::PermissionsException, :with => lambda { |exception| render_error 403, exception }

  # GET /type
  # Return a list of objects in a format Backbone.Paginator can use
  def index
    per_page = [params.fetch(:count, 20).to_i, 1].max
    from = params[:page].to_i == 0 ? 0 : (params[:page].to_i * per_page)
    sort = params[:sort]
    search_str = params.has_key?("filter") ? params[:filter] : '*:*'
    filter = get_search_filter(search_str)

    solr_query = "search/#{object_type}?q=#{escape(filter)}&start=#{from}&rows=#{per_page}"

    if sort.present?
      solr_query += "&sort=#{sort}"
    end

    @objects = server.get_rest(solr_query)
    @total = @objects["total"]

    @objects = @objects["rows"].map do |obj|
      begin
        # If the chef server has strict_search_result_acls enabled,
        # then we don't need to check permissions on each object; the
        # server guarantees that we're allowed to see any entities
        # returned by search.
        unless Settings.webapp.strict_search_result_acls
          # this will raise an exception if session_user doesn't have show perms
          has_object_permission(obj)
        end
        object_to_hash(obj)
      rescue Net::HTTPServerException => e
        if e.response.code.to_i == 403
          {"_id" => obj.name, "forbidden" => true}
        else
          raise e
        end
      end
    end

    respond_to do |format|
      format.html { render_app }
      format.json { render :json => { :total => @total, :results => @objects } }
    end

  end

  def show
    @obj = object_to_hash(object_for_id(params[:id]))

    if @obj.has_key?("forbidden") && @obj["forbidden"]
      raise PermissionsException
    else
      respond_to do |format|
        format.html { render_app }
        format.json { render :json => @obj }
      end
    end
  end

  protected

  def has_object_permission(obj)
    if obj.is_a?(Hash)
      server.get_rest(show_url(obj["name"]))
    else
      server.get_rest(show_url(obj.name))
    end
  end

  # derived classes to override this
  def searchable_fields
    raise "Subclasses must provide an implementation for #{searchable_fields}"
  end

  def escape(s)
    s && URI::UrlParser.escape(s.to_s)
  end

  def get_search_filter(search_str)
    if !search_str.include? ':'
      filter = '*:*'
      if searchable_fields.length > 0
        filter = searchable_fields.map{|f| "#{f}:*#{search_str}*" }.join(' OR ')
      end
      filter
    else
      search_str
    end
  end

end
