require "certificate_parser"

class ClientsController < ChefSolrSearchableObjectController

  def create
    users = server_root.get_rest("organizations/#{params[:organization_id]}/users")

    users.each do |user|
      user = user["user"]["username"]
      if user == params[:name]
        return render json: {"url" => "/organizations/#{params[:organization_id]}/users/#{user}"}, status: 409
      end
    end

    response = server.post('clients', params[:client])
    render json: response, status: 201
  end

  def update
    begin
      render json: server.put_rest(
        "clients/#{params[:id]}", params[:client])
    rescue Net::HTTPServerException => e
      render json: parsed_json_for_error(e), status: e.response.code
    end
  end

  protected

  def object_for_id(id)
    client_info = super
    # if the hash contains certificate, we need to
    # parse the cert to properly get its public_key.
    if client_info.is_a?(Chef::ApiClient)
      client_info = client_info.to_hash
    end
    if !client_info["certificate"].nil?
      client_info["public_key"] = CertificateParser.new(client_info["certificate"]).parse_public_key_from_cert()
      # we have the public key, no longer need cert
      client_info.delete("certificate")

    # if there was not certificate or public_key field, throw 500
    # should never happen, but we want to capture it if it does
    elsif client_info["certificate"].nil? && client_info["public_key"].nil?
      raise "Client data contained neither a certificate nor a public_key."
    end
    client_info
  end

  def object_type
    'client'
  end

  def searchable_fields
    ['name']
  end

  def index_url
    'clients'
  end
end
