class CookbooksController < ChefObjectController

  def index
    @cookbooks = self.class.format_list(
      server.get_rest('cookbooks?num_versions=all')
    )

    respond_to do |format|
      format.html do
        @preload_object = @cookbooks
        render_app
      end

      format.json do
        render json: @cookbooks
      end
    end
  end

  # GET /type/:id/:version_id
  def show_version
    respond_to do |format|
      format.html do
        @preload_object = object_for_id(params[:id])
        render_app
      end
      format.json do
        @obj = server.get_rest(show_version_url(params[:id], params[:version_id]))
        render :json => object_to_hash(@obj)
      end
    end
  end

  # GET /type/:id/fetchfile?fetchurl="..."
  def fetchfile
    file_type = params[:filetype]
    file_name = params[:filename]
    cookbook_version = params[:cookbookversion]

    # get the cookbookversion and identify the URL for requested file
    @obj = server.get_rest(show_version_url(params[:id], cookbook_version))

    file_obj = @obj[file_type].find { |file| file['name'] == file_name }
    file_url = file_obj['url']
    # TODO - later check for content length using HEAD and disallow large files showing a error to use knife command.

    # Set the Accept header to text/plain on the request, so we expect to get
    # back the file contents instead of JSON.
    #
    # The second argument is "raw", which should be false.
    content = server.get_rest(file_url, 'Accept' => 'text/plain')

    response.headers["Content-Disposition"] = "attachment; filename=\"#{file_name}\""
    send_data(content, :filename => file_name)
  end

  protected

  # pick the object details from default version.
  def object_for_id(id)
    begin
      object = server.get_rest(show_url(id))

      # Get the latest version details and send across.
      cb_ver = server.get_rest(show_version_url(id, object[id]["versions"][0]["version"]))

      @cookbook = {}
      @cookbook[:name] = id
      @cookbook[:current_version] = object[id]["versions"][0]["version"]
      @cookbook[:versions] = object[id]["versions"].map { |obj| obj["version"] }
      @cookbook[:description] = cb_ver["metadata"]["description"]
      @cookbook
    rescue Net::HTTPServerException => e
      if e.response.code.to_i == 403
        @cookbook = {"_id" => id, "forbidden" => true}
      else
        raise e
      end
    end
  end

  # Helper to get the Chef API URL for a single object given its ID
  def show_version_url(id, version)
    "cookbooks/#{id}/#{version}"
  end

  def show_url(id)
    "cookbooks/#{id}"
  end

  private

  def self.format_list(cookbooks)
    cookbooks.sort.map do |name, item|
      {
        name: name,
        current_version: item['versions'][0]['version'],
        versions: versions_for(item),
      }
    end
  end

  def self.versions_for(item)
    item['versions'].map { |v| v['version'] }
  end
end
