class DataBagsController < ChefObjectController
  before_action :permitted_params

  def create
    databag = Chef::DataBag.from_hash(@permitted_params)
    server.post('data', databag)
    render json: databag.to_hash, status: :created
  end

  protected

  def permitted_params
    @permitted_params = params.permit(:name, :items, :organization_id, data_bag: {}).to_h
  end

  def index_url
    "data"
  end

  # The search will raise a 404 if the databag does not exist
  def object_for_id(id)
    begin
      server.get_rest("data/#{id}")
      {:name => id}
    rescue Net::HTTPServerException => e
      if e.response.code.to_i == 403
        {"_id" => id, "forbidden" => true}
      else
        raise e
      end
    end
  end

  def preload_collection
    'databags'
  end
end
