class EnvironmentsController < ChefSolrSearchableObjectController
  before_action :permitted_params

  def recipes
    @recipes = server.get_rest("environments/#{params[:id]}/recipes")
    render :json => @recipes
  end

  def cookbooks
    @cookbooks = server.get_rest("environments/#{params[:id]}/cookbooks")
    render :json => @cookbooks
  end

  def create
    begin
      environment = Chef::Environment.from_hash(@permitted_params.to_h)
      server.post('environments', environment)
      render json: environment.to_hash, status: :created
    rescue Chef::Exceptions::ValidationFailed => e
      render json: { error: e.message }, status: :forbidden
    end
  end

  def update
    begin
      environment = Chef::Environment.from_hash(@permitted_params.to_h)
      server.put("environments/#{params[:id]}", environment)
      render json: environment.to_hash
    rescue Chef::Exceptions::ValidationFailed => e
      render json: { error: e.message }, status: :forbidden
    end
  end

  protected

  def permitted_params
    @permitted_params = params.permit(:page, :count, :filter, :sort, :organization_id, :_, :name, :description, cookbook_versions: {}, default_attributes: {}, override_attributes: {})
  end

  def object_type
    'environment'
  end

  def searchable_fields
    ['name', 'description']
  end

  def index_url
    "environments"
  end
end
