require 'usag_helper'

class GroupsController < ChefObjectController
  include ApplicationHelper

  # Creates an empty group with a name
  def create
    group = { 'groupname' => params[:groupname],
              'actors'    => { 'clients' => [],
                               'groups'  => [],
                               'users'   => [] } }
    server.post('groups', group)
    render json: object_to_hash(object_for_id(group['groupname'])), status: :created
  end

  # GET /groups
  def index
    @groups = groups_without_usags

    respond_to do |format|
      format.html do
        @preload_object = @groups
        render_app
      end

      format.json do
        render json: @groups
      end
    end
  end

  # PUT /group/:id
  def update
    group = { 'groupname' => params[:groupname],
              'actors'    => { 'clients' => Array(params[:clients]),
                               'groups'  => Array(params[:groups]),
                               'users'   => Array(params[:users]) } }

    # When updating the admins group, make sure to add the
    # platform user back in to the group so that we don't lock
    # the webui out of everything
    if params[:id] == 'admins'
      group['actors']['users'] = (group['actors']['users'] << platform_user).uniq
    end

    # When modifying the users group, make sure to re-add USAGs
    if params[:id] == 'users'
      group['actors']['groups'].concat(usags.map { |g| g['groupname'] })
    end

    group['groupname'] = params[:name] if params[:name].present?
    server.put_rest("groups/#{params[:id]}", group)

    render json: object_to_hash(object_for_id(group['groupname']))
  rescue Net::HTTPServerException => e
    if e.response.code.to_i == 403
      render(
        json: {
          success: false,
          verb: 'PUT',
          message: 'You do not have permission to change this group.',
        },
        status: :forbidden
      )
    else
      render json: parsed_json_for_error(e), status: e.response.code
    end
  end

  private

  def all_groups_with_usags
    @all_groups_with_usags ||= server.get_rest('groups').keys.sort.map do |key|
      { 'groupname' => key }
    end
  end

  def groups_without_usags
    @groups_without_usags ||= all_groups_with_usags.reject do |group|
      group['groupname'].match(USAG)
    end
  end

  def usags
    @usags ||= all_groups_with_usags - groups_without_usags
  end

  protected

  def index_url
    'groups'
  end

  def object_to_hash(*args)
    obj = super

    unless obj.key?('forbidden')
      usag_helper = USAGHelper.new(server, USAG, 'groups')
      obj = usag_helper.populate_single_object!(obj)
      obj['actors'].delete(platform_user)
      obj['users'].delete(platform_user)
    end

    obj
  end
end
