class NodesController < ChefSolrSearchableObjectController

  include RunlistHelper

  def update
    # Because of https://github.com/rails/rails/issues/8832 (Rails being silly),
    # an empty ([]) run list will be parsed as nil. Ensure that it's an empty
    # Array as intended.
    permitted_params[:run_list] ||= []

    node = Chef::Node.from_hash(permitted_params.to_h)
    server.put("nodes/#{permitted_params[:id]}", node)
    render json: node.to_hash, status: :created
  end

  def destroy
    # delete node
    begin
      server.delete_rest(show_url(params[:id]))
    rescue Net::HTTPServerException => e
      raise e unless e.response.code.to_i == 403
      render :json => {"success" => false, :message => "You do not have permission to delete this object."}, :status => 403
      return
    end

    # if client exists, delete client
    begin
      server.delete_rest("clients/#{params[:id]}")
    rescue  Net::HTTPServerException => e
      # return 403 to UI
      if e.response.code.to_i == 403
        render :json => {"success" => false, :message => "You do not have permission to delete this object."}, :status => 403
        return
      # If the client didn't exist, just move on
      elsif ! e.response.code == '404'
        raise e
      end
    end

    render :json => {"success" => true}, :status => 204
  end

 protected

  def permitted_params
    params.permit!
  end

  def object_type
    'node'
  end

  def searchable_fields
    ['name', 'platform', 'fqdn', 'ipaddress', 'chef_environment']
  end

  def index_url
    "nodes"
  end
end
