class OidcController < ApplicationController
  include ApplicationHelper
  include OidcHelper

  layout "outside"

  skip_before_action :check_for_password_reset
  skip_before_action :check_for_session_expiration
  skip_before_action :require_login

  def new
    return go_to_root unless saml_enabled?
  end

  def callback
    return go_to_root unless saml_enabled?

    reset_session

    if params['error']
      @detail = params['error_description']
      return
    end

    unless params['code']
      @detail = 'No code provided'
      return
    end

    unless token = client.exchange_code_for_token(params['code'])
      @detail = 'Could not exchange authorization code for access token'
      return
    end

    external_uid = token.sub
    Rails.logger.info("OIDC login with external_uid: #{external_uid}")
    user = find_user_on_chef_server(external_uid)

    unless user
      session[:link_account_authenticated_user] = client.user_from_token(token)
      redirect_to account_links_path
    else
      session[:username] = user['username']
      session[:session_start_time] = Time.zone.now
      state_path = URI::UrlParser.unescape params.fetch(:state, '/')
      redirect_to redirect_destination(state_path)
    end
  end

  private

  def body_class
    'oidc'
  end

  def go_to_root
    redirect_to root_path
  end
end
