require 'reporting_query_helper'

class ReportingQueryController < ChefObjectController
  include ReportingQueryHelper
  respond_to :json

  def query
    # if there are start and end params, round them up and down to the proper time unit
    if params[:startTime].present? && params[:endTime].present?
      params[:startTime] = params[:startTime].to_i
      params[:endTime] = params[:endTime].to_i
    end

    if params[:diff].present?
      @query = prepare_diff_query(params)
    elsif params[:dashboard].present?
      @query = prepare_runs_query(params)
    else
      @query = prepare_run_query(params)
    end

    if @query.nil?
      render :json => {:error => true, :message => "client requested invalid query"}, :status => 400
    else
      if params.has_key?(:diff)
        response = reporting_query(@query)
        @diff = {}
        @diff[:diff_data] = response["resource_detail"]["content_delta"]
        @diff[:success] = true
        respond_with @diff
      else
        if params[:dashboard].present?
          respond_with reporting_query(@query)
        else
          # call ChefObjectController index for run step list.
          # we must paginate on the rails backend because reporting
          # does not support returning paginated results for run steps
          @objects = reporting_query(@query)
          @total = @objects.length
          per_page = [params.fetch(:count, 20).to_i, 1].max
          from = [params[:page].to_i, 0].max * per_page
          to = from + per_page - 1
          @objects = get_objects_details(from, to)
          respond_to do |format|
            format.html { render_app }
            format.json { render :json => { :total => @total, :results => @objects } }
          end
        end
      end
    end
  end

  private

  def query_url
    "query"
  end

  def reporting_query(query)
    protocol_version = Settings.reporting.protocol_version
    server.get_rest(query, {'X-Ops-Reporting-Protocol-Version' => protocol_version})
  end

  # set up run details in the format Backbone.Paginator expects for run step list
  def get_objects_details(from, to)
    if @objects["run_detail"]["status"] == "started"
      @objects = []
    else
      @objects = @objects["run_resources"]
    end
    @total = @objects.length
    i = 0
    for idx in from..to
      if i >= @objects.length
        break
      end
      @objects[i]["index"] = idx + 1
      i += 1
    end
    @objects = @objects[0..i]
  end
end
