class RolesController < ChefSolrSearchableObjectController
  include RunlistHelper

  before_action :permitted_params

  def create
    role = Chef::Role.from_hash(@permitted_params.to_h)
    server.post('roles', role)
    render json: role.to_hash, status: :created
  end

  protected

  def object_type
    'role'
  end

  def searchable_fields
    ['name', 'description', 'environment']
  end

  def index_url
    "roles"
  end

  def run_lists_for_object(role)
    if role.is_a?(Hash)
      role["env_run_lists"].merge("_default" => role["run_list"])
    else
      role.env_run_lists
    end
  end

  private

  def permitted_params
    @permitted_params = params.permit(:id, :name, :description, :organization_id, role: {}, cookbook_versions: {}, default_attributes: {}, override_attributes: {}, run_list: [:name, :type])
  end
end
