require 'reporting_query_helper'

class RunsController < ChefObjectController
  include ReportingQueryHelper

  def index
    begin
      @runs = get_runs(params)
      if ! @runs && request.format.json?
        render :json => {:error => true, :message => "client requested invalid query"}, :status => 400
      else
        respond_to do |format|
          format.html { render_app }
          format.json { render :json => { :total => @runs["run_history"].length, :results => @runs["run_history"] } }
        end
      end
    rescue Net::HTTPFatalError
      render :json => {:error => "Reporting is currently unavailable"}, :status => :not_found
    end
  end

  def show
    @runs = get_runs(params)
    if ! @runs && request.format.json?
      render :json => {:error => true, :message => "client requested invalid query"}, :status => 400
    else
      respond_to do |format|
        format.html {
          run = server.get_rest("reports/org/runs/#{params[:id]}")['run_detail']
          parse_run_list(run)
          @preload_object = run
          render_app
        }
        format.json { render :json => { :total => @runs["run_history"].length, :results => @runs["run_history"] } }
      end
    end
  end

  def preload_collection
    'runs'
  end

  def parse_run_list(run)
    # parse ALL THE RUN LIST ITEMS
    this_run_list = Array.new
    string_arr = run["run_list"].present? ? JSON.parse(run["run_list"]) : []
    if string_arr.length > 0
      for item in string_arr
        parsed_run_list = item.scan(/(\w+)\[(.*)\]/)
        type = parsed_run_list[0][0]
        names = parsed_run_list[0][1]
        for name in names.split(",")
          this_run_list.push([type, name])
        end
      end
      run["run_list"] = this_run_list
    else
      run["run_list"] = []
    end

    # parse the backtrace if it exists
    if run["data"]
      if run["data"]["exception"] and run["data"]["exception"]["backtrace"]
        run["data"]["exception"]["backtrace"] = JSON.parse(run["data"]["exception"]["backtrace"])
      end
    end
  end

  private
  def index_url
    "runs"
  end

  def get_runs(params)
    query = prepare_runs_query(params)
    if query.nil?
      return runs = nil
    else
      runs = server.get_rest(query)
      runs[:success] = true
    end

    if runs
      for run in runs["run_history"]
        parse_run_list(run)
      end
    end

    runs
  end
end
