module AccountsHelper
  def demandbase_field_names
    %w(
      demandbase_sid
      db_industry
      sub_industry
      revenue_range
      employee_range
      db_street_address
      db_city
      db_state
      db_postal_code
      db_country_code
      data_source
      db_company_name
      db_phone
      db_country_name
      db_sic_code
      watch_list_account_type
      watch_list_account_status
      watch_list_campaign_code
      watch_list_account_owner
      db_annual_revenue
      fortune_1000
      forbes_2000
      db_website
      db_stock_ticker
    )
  end
  module_function :demandbase_field_names

  def demandbase_fields
    demandbase_field_names.map { |f| hidden_field_tag f }.join("\n").html_safe
  end

  def resolve_user_from_username_or_email(username_or_email)
    if username_or_email.include?('@')
      candidate_users = server_root.get_rest(
        "users?#{{ email: username_or_email }.to_query}"
      )
      find_user_among_candidates(username_or_email, candidate_users)
    else
      username_or_email # it's a username.
    end
  end

  def find_user_among_candidates(queried_email, users)
    case users.keys.length
    when 0
      raise UserEmailNotFoundException.new("If '#{queried_email}' username/email address exists, a password reset email has been sent.")
    when 1 # only one match, ignore case
      users.first[0]
    else   # multiple candidates, look for user with case sensitive email match
      collected_mails = []
      users.each do |username, uri|
        record = server_root.get_rest(uri)
        collected_mails.push(record['email'])
        return username if record['email'] == queried_email
      end
      logger.warn("Ambiguous email '#{queried_email}', known emails: #{collected_mails.join(', ')}")
      raise UserEmailNotFoundException.new("'#{queried_email}' email address not unique, "\
                                           "please use your username (or contact your administrator).")
    end
  end

  def create_user_signature
    expires = 1.day.from_now.to_i
    Signature.new(
      @user['username'],
      expires,
      Chef::Manage::Application.config.secret_token,
      filter_user_params
    )
  end

  def filter_user_params
    @user.select { |key, value| %w(username email display_name company terms).include?(key) }
  end
end
