require 'chef/web/core/url_helpers'
require 'dark_launch'

module ApplicationHelper
  include Chef::Web::Core::URLHelpers

  delegate :chef_documentation_url, :chef_title, :external_password_reset_url,
           :support_email_address, :support_site_url, :support_tickets_url,
           :privacy_policy_url, :cookie_policy_url,
           to: 'Settings'

  def chef_documentation_url
    Settings.chef_documentation_url || chef_docs_url
  end

  def chef_mailing_list_subscribe_url
    'https://discourse.chef.io/'
  end

  def disable_sign_up?
    !!Settings.disable_sign_up
  end

  def require_recaptcha?
    !!Settings.recaptcha && !!Settings.recaptcha.enabled
  end

  def recaptcha_site_key
    Credentials.get("recaptcha", "site_key")
  end

  def recaptcha_secret_key
    Credentials.get("recaptcha", "secret_key")
  end

  def external_auth_adjective
    Settings.ldap.adjective
  end

  def external_auth_enabled?
    !!Settings.ldap.enabled
  end

  def saml_enabled?
    !!Settings.saml && !!Settings.saml.enabled
  end

  def free_trial_available?
    !private_chef?
  end

  def platform_public_url
    Settings.platform.public_url || Settings.platform.url
  end

  def private_chef?
    @private_chef ||= DarkLaunch.is_feature_enabled?('private-chef', nil)
  end

  def show_documentation_link_on_sign_up?
    private_chef?
  end

  def show_open_source_link_on_sign_up?
    !private_chef?
  end

  def show_marketing_fields?
    params[:ref] == "other"
  end

  def terms_acceptance_required_on_sign_up?
    marketplace_sign_up_enabled? || !private_chef?
  end

  # True if the external_password_reset_url is set
  def use_external_password_reset?
    external_password_reset_url.present?
  end

  # disabling the feedback form everywhere in 2021
  # since we're not maintaining manage longterm there's need to cleanup
  # all the codebase to remove the feature entirely
  def feedback_form_enabled?
    false
  end

  def marketplace_sign_up_enabled?
    Settings.marketplace.sign_up.enabled
  end

  def marketplace_sentinel_expired?
    if File.exist?(Settings.marketplace.sentinel.file)
      time_threshold = Time.now - Settings.marketplace.sign_up.timeout.to_i
      access_time = File.atime(Settings.marketplace.sentinel.file)
      access_time < time_threshold
    else
      true
    end
  end

  def redirect_destination(url)
    # Only get the path section of the to parameter to avoid open redirects
    destination = URI.parse(url).path

    # Paths that are empty or not absolute (starting with /; we can't use
    # URI#absolute? because the input may have been just a path, not a full URI)
    # should go home, they're drunk.
    if destination.empty? || destination.first != '/'
      root_path
    else
      destination
    end
  end

  def platform_user
    Settings.platform.user
  end

  def air_gapped?
    Settings.air_gapped == true
  end

  def gtm_enabled?
    !!Settings.gtm && !!Settings.gtm.enabled && !!Settings.gtm.id
  end

  def onetrust_enabled?
    !!Settings.onetrust && !!Settings.onetrust.enabled && !!Settings.onetrust.id
  end
end
