require 'chef/web/core/url_helpers'
require 'oidc_client'
require 'veil'

module OidcHelper
  include Chef::Web::Core::URLHelpers
  include SecretsHelpers

  def client
    @client ||= OidcClient.new(client_id, client_secret, redirect_uri, issuer_url, verify_tls)
  end

  # this is defined by `:as => 'oidc_callback'` in config/routes.rb
  def redirect_uri
    oidc_callback_url
  end

  def provider_discovery_error?
    !client.ready?
  end

  def authorization_uri
    return unless client.ready?
    state = params.fetch(:to, '/')
    client.authorization_uri(state: state)
  end

  def issuer_url
    Settings.saml.issuer_url
  end

  def client_id
    Credentials.get("saml", "client_id")
  end

  def client_secret
    Credentials.get("saml", "client_secret")
  end

  def verify_tls
    Settings.saml.verify_tls
  end

  def saml_adjective
    Settings.saml.adjective || 'SAML'
  end

  def find_user_on_chef_server(external_uid)
    users = server_root.get_rest(
      "users?#{{ external_authentication_uid: external_uid }.to_query}"
    )
    if (users.length == 1)
      username = users.first[0]
      user = server_root.get_rest("users/#{username}")
      return user unless user['username'] == platform_user
    end
  rescue Net::HTTPServerException, Net::HTTPFatalError => e
    unless e.response.code == "404"
      Rails.logger.error("OIDC HTTP error calling Chef Infra Server: #{e.response.code}")
      raise
    end
    nil
  end
end
