module RunlistHelper

  # GET .../run_lists
  def run_lists
    @object = object_for_id(params[:id])
    @run_lists = run_lists_for_object(@object).map do |env, run_list|
      run_list = Chef::RunList.new(*run_list)
      @run_list_expansion = BetterRunListExpansionFromAPI.new(env, run_list.run_list_items, server)
      @run_list_expansion.expand
      @seen_items = {:recipe => {}, :role => {}}
      find_cookbook_versions(env)
      @position = -1
      {:id => env, :run_list => convert_run_list_trace('top level')}
    end
    render :json => @run_lists
  rescue UnsatisfiableRunListItemError => e
    render :status => 412, :body => e.message
  end

  private

  # Return pairs of environment and run list for this object
  def run_lists_for_object(obj)
    # Seems sane as a default, given that there are only two uses of this for now?
    if obj.has_key?("chef_environment")
     [[obj["chef_environment"], obj["run_list"]]]
    else
      obj["env_run_lists"]
    end
  end

  # Keep asking for a cookbook version set until you get one that works
  # TODO: Need more testing about how this behaves in imposible constraint sets <NPK 2012-10-24>
  def find_cookbook_versions(env)
    @cookbook_versions = {}
    recipes = @run_list_expansion.recipes.with_version_constraints_strings
    @non_existent_cookbooks = {}
    @cookbooks_with_no_versions = {}
    until recipes.empty? # Usually this will bail before the list is empty, just a backstop
      begin
        @cookbook_versions = server.post_rest("environments/#{env}/cookbook_versions", :run_list => recipes)
        break
      rescue Net::HTTPServerException => ex
        case ex.response.code.to_i
        # A 404 happens when the environment does not exist. In this case its
        # recipes aren't valid, so clear the recipes array
        when 404
          recipes.clear
        when 412
          message = ActiveSupport::JSON.decode(ex.response.body)
          # sometimes it is already decoded
          error_data = message['error'].first

          if error_data.class != Hash && error_data.class != IO
            error = ActiveSupport::JSON.decode(error_data)
          else
            error = error_data
          end
          non_existent_cookbooks = error.fetch('non_existent_cookbooks', [])
          cookbooks_with_no_versions = error.fetch('cookbooks_with_no_versions', [])

          unless error.fetch('unsatisfiable_run_list_item', []).empty?
            raise UnsatisfiableRunListItemError.new(error.fetch('message'))
          end

          non_existent_cookbooks.each do |name|
            @non_existent_cookbooks[name] = true
          end
          cookbooks_with_no_versions.each do |dependency|
            # dependency is a dependency constraint string like
            # (foobar_pkg >= 1.0) The regex here is just to pick out
            # the package name out of the full dependency string.
            #
            # Ideally we'd like the server to be giving us a more
            # structured JSON object that has the package name and
            # version constraint string separated out so we don't have
            # to parse them here, but the ultimate source of the
            # dependency string is the dep_selector gem, so we'd have
            # to thread through that change (or maybe like an
            # additional "cookbook_names_with_no_versions" field)
            # through both that ruby code and the chef-server itself.
            # This is an adequate workaround.
            name_match = /\((.*?) /.match(dependency)
            if name_match
              name = name_match[1]
              @cookbooks_with_no_versions[name] = true
            else
              raise "Don't know how to parse package from dependency string '#{dependency}'"
            end
          end

          recipes.delete_if do |name|
            name = name.split('@').first.split('::').first
            @non_existent_cookbooks.include?(name) || @cookbooks_with_no_versions.include?(name)
          end
        else
          raise
        end
      end
    end
  end

  class UnsatisfiableRunListItemError < StandardError
    def initialize(msg)
      super
    end
  end

  # Super hacky solution to get the tree form of the run list expansion
  class BetterRunListExpansionFromAPI < Chef::RunList::RunListExpansionFromAPI
    attr_reader :better_run_list_trace
    attr_reader :all_missing_roles
    attr_reader :role_errors

    def initialize(*args)
      super
      # @better_run_list_trace = Hash.new {|h, key| h[key] = [] }
      # @all_missing_roles = {}
      # @role_errors = {}
    end

    # I need the missing roles so I can fill in the :missing key in the tree
    def role_not_found(name, included_by)
      super
      # @missing_roles_with_including_role << [name, included_by]
      # @all_missing_roles[name] = true
      # nil
    end

    # Don't stringify things, I need details
    def expand_run_list_items(items, included_by="top level")
      # @better_run_list_trace[included_by.to_s] << items[0] unless items.empty?
      super
    end

    def fetch_role(name, included_by)
      super
    rescue Exception => e
      # Something that wasn't caught lower down
      @role_errors[name] = e.to_s
      nil
    end
  end

  # Recursive helper to decode the non-nested hash form back into a tree
  def convert_run_list_trace(base)
    @run_list_expansion.better_run_list_trace[base].map do |item|
      skipped = @seen_items[item.type][item.name]
      @seen_items[item.type][item.name] = true
      case item.type
      when :recipe
        name = item.name.split('::').first
        cookbook_version = @cookbook_versions[name]
        {:type => :recipe, :name => item.name, :version => cookbook_version ? cookbook_version["version"] : nil, :position => (!cookbook_version || skipped) ? nil : @position += 1,
         :missing => !!@non_existent_cookbooks[name], :no_version => !!@cookbooks_with_no_versions[name], :skipped => !!skipped}
      when :role
        error = @run_list_expansion.role_errors[item.name]
        missing = @run_list_expansion.all_missing_roles[item.name]
        {:type => :role, :name => item.name, :children => (missing || error || skipped) ? [] : convert_run_list_trace(item.to_s),
         :missing => missing, :error => error, :skipped => skipped}
      end
    end
  end
end
