module TitleHelper
  # Returns a page title by retrieving the localized app name and a
  # page-specific title based on the controller and action. For example, if the
  # action is `SessionsController#new`, `page_title` will attempt to fetch the
  # following keys, in order from first to last:
  #
  #  1. title.sessions.new
  #  2. title.sessions.default
  #  3. title.default
  #
  # If a translation is found, a title like "<title> - <app_name>" will be
  # returned, e.g. "Sign In - Chef Manage". If none is found, only the app name
  # will be returned, e.g. "Chef Manage."
  #
  def page_title
    sprintf( "%s - %s",
      I18n.t(page_title_translation_key, **page_title_i18n_opts),
      I18n.t(:app_name) )
  rescue I18n::MissingTranslationData
    I18n.t(:app_name)
  end

  private
  def page_title_translation_key
    :"title.#{controller_name}.#{action_name}"
  end

  # Allow view assigns (i.e. instance variables assigned in the controller) to
  # be used in I18n interpolation, e.g. assigning `@name` in the controller
  # would enable the translation "Welcome {{name}}!"
  def page_title_context
    controller.view_assigns.symbolize_keys
  end

  def page_title_i18n_opts
    page_title_context.merge :raise => true,
      :default => [ :"title.#{controller_name}.default", :"title.default" ]
  end
end
