class UserVerifyMailer < ActionMailer::Base
  #
  # Sends an email to a user to verify that their email address is legit before
  # creating their user record.
  #
  # @param user_params [Hash] the user values
  # @param ref [String] the ref param from signup
  # @param expires [Int] the time before the link is invalid
  # @param signature [String] the email verification validation signature
  #
  def user_verify(user_params, ref, expires, signature, action="create")

    @user = user_params.is_a?(Hash) ? user_params : permitted_user_params(user_params)
    @ref = ref
    @expires = expires
    @signature = signature
    @action = action
    @signature.user_params = @user
    mail :from => Settings.email_from_address,
         :subject => "Validate your #{Settings.chef_title} email address",
         :to => @user['email']
  end

  def permitted_user_params(user_params)
    user_params.permit(:display_name, :email, :terms, :username, :first_name, :last_name, :company).to_h
  end
end
