# Worker tasks for Nodes

require_dependency 'util_worker.rb'

module NodesWorker

  class ManageTagsWorker < BaseWorker
    task_label 'Manage tags'
    def perform(nodes, tags, operation)
      progress(0, nodes.size)
      tags = tags.split(",")
      nodes.each_with_index do |node_id, i|
        node = server.get_rest("nodes/#{node_id}")
        node['normal']['tags'] = [] unless node['normal']['tags'].is_a?(Array)
        current_node_tags = node['normal']['tags']
        case operation
        when 'Add'
          node['normal']['tags'] = (node['normal']['tags'] + tags).uniq
        when 'Delete'
          node['normal']['tags'] -= tags
        when 'Set'
          node['normal']['tags'] = tags
        end
        if current_node_tags != node['normal']['tags']
          server.put_rest("nodes/#{node_id}", node)
          expire(:nodes, node_id)
        end
        progress(i+1)
      end
    end
  end

  class EditRunListWorker < UtilWorker::EditRunListWorker
    def expire_run_list(object_id)
      expire(:nodes, object_id)
    end

    def chef_url(object_id)
      "nodes/#{object_id}"
    end
  end

  class NormalWorker < ::AttributesWorker
    task_label 'Update node attributes'
    def chef_url(node_id)
      "nodes/#{node_id}"
    end

    def update_attributes(node, attributes)
      node["normal"] = attributes
    end

    def expire_attributes(node_id)
      expire(:nodes, node_id)
    end
  end

end
