# Worker tasks for Organizations

module OrgWorker
  class CreateOrgWorker < BaseWorker
    task_label 'CreateOrg'
    def perform(collection, org)
      # how can we ovevrride the perform method to ignore collection parameter?
      org_id = org['id']
      body_hash = {
                     :name => "#{org_id}",
                     :full_name => "#{org_id}",
                     :org_type => 'Business',
                   }
      begin
        response = server_root.post_rest("organizations", body_hash)
        request_body_hash = { :user => @chef_rest_user }
        association_request_response = server_root.post_rest("organizations/#{org_id}/association_requests", request_body_hash)
        association_request_id = association_request_response['uri'].split('/').last
        response_body_hash = { :response => 'accept' }
        response = server_root.put_rest("users/#{@chef_rest_user}/association_requests/#{association_request_id}", response_body_hash)

        # Add the new user to admins, and billing-admins group
        add_new_user_to_group("admins", org_id)
        add_new_user_to_group("billing-admins", org_id)

        user = server_root.get_rest("users/#{@chef_rest_user}")
        user_hash = {
        :billing_contact => {
          :first_name => user['first_name'],
          :last_name => user['last_name'],
          :email => user['email']
          },
        :billing_plan => "platform-free"
        }
        response  = server_root.put_rest("organizations/#{org_id}", user_hash)
        response['redirect_to'] = "/organizations/#{org_id}/getting_started"
        response['response'] = {status: ["200 OK"]}
        update_user(collection, org, 'create_org', response)
      rescue Net::HTTPServerException => e
        update_user(collection, org, 'create_org', e)
      end
    end

    # FIXME - reuse/share code with rails controllers?
    ACL_CONTAINERS = %w{clients groups containers data nodes roles cookbooks sandboxes environments}
    # Helper to get all the ACEs that a user matches for a given object
    # Pending backend support, so just a stub for now
    def fetch_matched_aces(object)
      {'read' => true, 'create' => true, 'grant' => true, 'update' => true, 'delete' => true}
    end

    def add_new_user_to_group(groupname, org_id)
      group = server_root.get_rest("organizations/#{org_id}/groups/#{groupname}")
      update_body_hash = {
        :groupname => "#{groupname}",
        :actors => {
          "users" => group["actors"].concat(["#{@chef_rest_user}"]),
          "groups" => group["groups"]
        }
      }
      server_root.put_rest("organizations/#{org_id}/groups/#{groupname}", update_body_hash)
    end

  end
end
