# Worker Utility Classes

module UtilWorker

  class EditRunListWorker < BaseWorker
    task_abstract true
    task_label 'Edit run lists'

    def perform(collection, run_list)
      progress(0, collection.size)
      collection.each_with_index do |element_id, i|
        element = server.get_rest(chef_url(element_id))
        element[:run_list] = run_list
        server.put_rest(chef_url(element_id), element)
        progress(i+1)
        expire_run_list(element_id)
      end
    end

    def chef_url(object_id)
      # ex. "nodes/#{object_id}"
      raise 'Subclass must implement chef_url'
    end

    def expire_run_list(object_id)
      # ex. expire(:nodes, object_id)
      raise 'Subclass must implement expire_run_list'
    end
  end

  class ReportFeedbackWorker < BaseWorker
    task_label 'Feedback'
    def perform(collection, title, description, browser_info, app_state)
      progress(0, 1)
      FeedbackMailer.send_email(title, description, browser_info, app_state).deliver_now
      progress(1)
    end
  end

  class RekeyWorker < BaseWorker
    task_label 'Rekey'
    def perform(collection, endpoint, collection_string)
      progress(0, collection.size)
      collection.each_with_index do |element_id, i|
        client = {'name' => element_id, 'private_key' => "true"}
        response = server.put_rest("clients/#{element_id}", client)
        if response.is_a?(Chef::ApiClient)
          response = response.to_hash
        end
        progress(i+1)
        update_user(collection_string, element_id, 'rekey', response['private_key'])
      end
    end
  end
end

class AttributesWorker < BaseWorker
  task_abstract true
  task_label 'Update attributes'
  def perform(object_id, attributes)
    progress(0, 1)
    obj_url = chef_url(object_id)
    obj = server.get_rest(obj_url)
    update_attributes(obj, attributes)
    server.put_rest(obj_url, obj)
    progress(1)
    expire_attributes(object_id)
  end

  def chef_url(object_id)
    # ex. "nodes/#{object_id}"
    raise 'Subclass must implement chef_url'
  end

  def update_attributes(object, attributes)
    # ex. object.attributes = attributes
    raise 'Subclass must implement update_attributes'
  end

  def expire_attributes(object_id)
    # ex. expire(:nodes, object_id)
    raise 'Subclass must implement expire_attributes'
  end
end
