(defproject reagent-manage "0.1.0-SNAPSHOT"
  :description "FIXME: write description"
  :url "http://example.com/FIXME"
  :license {:name "Eclipse Public License"
            :url "http://www.eclipse.org/legal/epl-v10.html"}

  ;; TODO(jcreed): when cljsjs/peg lands in cljsjs/packages,
  ;; remove the pegjs bits in :source-paths and :git-dependencies
  ;; in favor of just ["cljsjs/peg" 0.8.0] in :dependencies.
  :source-paths ["src/clj" "src/cljs" ".lein-git-deps/pegjs-cljs"]

  :git-dependencies [["https://github.com/jcreedcmu/pegjs-cljs.git"]]

  :dependencies [[org.clojure/clojure "1.6.0"]
                 [ring-server "0.4.0"]
                 [cljsjs/react "0.12.2-5"]
                 [reagent "0.5.0"]
                 [reagent-forms "0.5.0"]
                 [reagent-utils "0.1.4"]
                 [cljs-http "0.1.24"]
                 [cljs-ajax "0.3.9"]
                 [org.clojure/clojurescript "0.0-3169" :scope "provided"]
                 [org.clojure/core.match "0.3.0-alpha4"]
                 [org.clojure/core.async "0.1.346.0-17112a-alpha"]
                 [ring "1.3.2"]
                 [ring/ring-defaults "0.1.4"]
                 [prone "0.8.1"]
                 [compojure "1.3.3"]
                 [selmer "0.8.2"]
                 [environ "1.0.0"]
                 [secretary "1.2.3"]]

  :plugins [[lein-cljsbuild "1.0.4"]
            [lein-marginalia "0.8.0"] ;; run `lein marg` to produce nice html docs
            [lein-git-deps "0.0.2-SNAPSHOT"]
            [lein-environ "1.0.0"]
            [lein-ring "0.9.1"]
            [lein-asset-minifier "0.2.2"]]

  :ring {:handler reagent-manage.handler/app
         :uberwar-name "reagent-manage.war"}

  :min-lein-version "2.5.0"

  :uberjar-name "reagent-manage.jar"

  :main reagent-manage.server

  :clean-targets ^{:protect false}
  [[:cljsbuild :builds :test :compiler :output-dir]
   [:cljsbuild :builds :test :compiler :output-to]
   [:profiles :dev :cljsbuild :builds :app :compiler :output-dir]
   [:profiles :dev :cljsbuild :builds :app :compiler :output-to]
   [:profiles :dev :cljsbuild :builds :browser-test :compiler :output-dir]
   [:profiles :dev :cljsbuild :builds :browser-test :compiler :output-to]]

  :cljsbuild {:builds {:app {:source-paths ["src/cljs"]
                             :compiler {
                                        ;; This is a uri path, not a filesystem path.
                                        ;; It is only absolute with respect to the webserver.
                                        :asset-path   "/js/out"

                                        :optimizations :none
                                        :pretty-print  true}}
                       }}

  :profiles {:dev {:repl-options {:init-ns reagent-manage.repl
                                  :nrepl-middleware [cemerick.piggieback/wrap-cljs-repl]}
                   :dependencies [[ring-mock "0.1.5"]
                                  [ring/ring-devel "1.3.2"]
                                  [leiningen "2.5.1"]
                                  [figwheel "0.2.5"]
                                  [weasel "0.6.0"]
                                  [com.cemerick/piggieback "0.2.0"]
                                  [org.clojure/tools.nrepl "0.2.10"]
                                  [pjstadig/humane-test-output "0.7.0"]]

                   :source-paths ["env/dev/clj"]
                   :plugins [[lein-figwheel "0.2.5"]]

                   :injections [(require 'pjstadig.humane-test-output)
                                (pjstadig.humane-test-output/activate!)]

                   :figwheel {:http-server-root "public"
                              :server-port 3449
                              :ring-handler reagent-manage.handler/app}

                   :env {:dev? true}

                   :cljsbuild {
                               :test-commands {"phantom" ["phantomjs" "phantom/unit-test.js" "phantom/unit-test.html"]}
                               :builds {:app {:source-paths ["env/dev/cljs"]
                                              :compiler {:main       "reagent-manage.dev"
                                                         :output-dir   "../public/js/out"
                                                         :output-to  "../app/assets/javascripts/reagent-manage-dev.js"
                                                         :source-map true}}
                                        :browser-test {:source-paths ["src/cljs" "test"]
                                                       :compiler {:main "test-runner.browser"
                                                                  :optimizations :none
                                                                  :source-map true
                                                                  :asset-path "/js/out"
                                                                  :output-dir "resources/public/js/out"
                                                                  :output-to  "resources/public/js/reagent-test-runner.js"}}
                                        :test {
                                               ;; If you really want lein cljsbuild auto test
                                               ;; to work, you can uncomment this line, but it causes
                                               ;; the whole test suite to run redundantly every time
                                               ;; each file is compiled during a normal
                                               ;; lein cljsbuild test

                                               ;; :notify-command ["phantomjs" "phantom/unit-test.js" "phantom/unit-test.html"]
                                               :source-paths ["src/cljs" "test"]
                                               :compiler {:optimizations  :whitespace
                                                          :main "test-runner.core"
                                                          :pretty-print   true
                                                          :output-dir      "target/out"
                                                          :output-to      "target/testable.js"}}}}}

             :prod {:env {:dev? false}
                    :cljsbuild {:builds {:app {:source-paths ["env/prod/cljs"]
                                               :compiler {:main "reagent-manage.prod"
                                                          :output-to "../app/assets/javascripts/reagent-manage.js"
                                                          :optimizations :advanced
                                                          :externs ["externs.js"]}}}}}

             ;; XXX probably can delete this
             :uberjar {:hooks [leiningen.cljsbuild minify-assets.plugin/hooks]
                       :env {:production true}
                       :aot :all
                       :omit-source true
                       :cljsbuild {:jar true
                                   :builds {:app
                                            {:source-paths ["env/prod/cljs"]
                                             :compiler
                                             {:optimizations :advanced
                                              :pretty-print false}}}}}}

  :aliases { "repl" ["trampoline" "run" "-m" "clojure.main" "repl.clj"]
             "figwheel" ["figwheel" "app"]
             "figwheel-test" ["figwheel" "browser-test"]
             "deploy" ["with-profile" "prod" "cljsbuild" "once"]})
