(ns reagent-manage.entity
  (:require
   [reagent.session :as session]
   [reagent-manage.utils :as u]
   [reagent-manage.popup :as popup]
   [reagent-manage.query-builder :as qb]
   [reagent-manage.api :as api]
   [reagent-manage.search-interface :as search]
   [reagent-manage.tabs :refer [tabbed-view]]
   [reagent-manage.panels :as panels]
   [reagent.core :as reagent :refer [atom cursor]]
   [reagent-manage.ajax :refer [GET POST PUT DELETE]]
   [cljs.core.match :refer-macros [match]]
   [reagent-manage.json-browser :as tree])
  (:require-macros
   [reagent-manage.macros :refer [<? go-api]]))

(defn set-current-entity!
  "Set the current entity being viewed."
  [satm entity]
  (reset! (cursor satm [:current-entity]) entity))

(defn node-tag-link [tag]
  [:a {:on-click
       (fn [e]
         (session/put! :current-entity [:saved-search :new])
         (let [doc-atm (cursor session/state [:search-data :new])]
              (reset! doc-atm
                      (search/internalize-doc {:index "node" :query (str "tags:" tag)}))
              (search/execute-search doc-atm)))} tag])

(defn node-details
  "Component to render high-level node summary"
  [data attr]
  [:span
   [:h4 (:name data)]
   [:table
    [:tr [:td "Last Check In"] [:td (:ohai_time attr)]]
    [:tr [:td "Uptime"] [:td (:uptime_seconds attr)]]
    [:tr [:td "Environment"] [:td (:chef_environment data)]]
    [:tr [:td "Tags"]
     [:td (interpose ", "
                     (for [tag (:tags attr)]
                       ^{:key tag} [node-tag-link tag]))]]]
   (if (> (count (:run_list data)) 0)
     [:span
      [:h5 "Run List"]
      [:ul
       (for [run (:run_list data)] ^{:key (:name run)}
            [:li (case (:type run)
                   "recipe" [:i.fi-book]
                   "role" [:i.fi-target]
                   [:i.fi-asterisk]) " " (:name run)])]]
     [:span])]
  )

(defn node-data-view
  "Component to render node viewer"
  [data-atm satm]
  (let [data @data-atm
        current-tab-atm (cursor satm [:current-tab])]
    (swap! current-tab-atm #(or % "details"))
    (if data
      (let [attr (apply merge (map data [:automatic :normal :default :override]))]
        [:span
         [tabbed-view [{:name "Details" :id "details"
                        :content [node-details data attr]}
                       {:name "Attributes" :id "attributes"
                        :content [tree/clj-data-browser attr]}
                       {:name "Permissions" :id "permissions"
                        :content [tree/clj-data-browser (:access data)]}]
          current-tab-atm]])
      [:span])))

(defn view
  "Component to render an entity viewer"
  [url satm]
  (let [data-atm (cursor satm [:data])]
    (go-api
     (reset! data-atm (<! (api/request-absolute GET url))))
    (fn [url]
      [node-data-view data-atm satm])))
