(ns reagent-manage.fragment)

;; Fragment Generator Component
;; ----------------------------

(defn set-hash-if-different!
  "A little utility function to update the hash itself, and be extra
  paranoid about inserting duplicate history entries just in case
  react redundantly tells us to update when we needn't."
  ([v] ;; v includes "#" character already
   (if (not= (.-state js/history) v)
     ;; The first argument to history.pushState is the data that we're
     ;; allowed to later observe via history.state. The second
     ;; argument is a "title" which seems to be unused so far by
     ;; browsers in practice. The third argument is what actually ends
     ;; up in the url bar.
     (js/history.pushState v "" v))))

(defn generator* "Wrap a stub component with lifecycle methods."
  [fragment] [:span])

(def generator
  "An attempt to manage the url fragment identifier in a way that
  feels idiomatically compatible with the react mindset. The contract
  for the rest of the code is that it should simply set application
  state to the correct thing, and this generator component
  will keep the url fragment updated accordingly."
  (with-meta generator*
    {:component-will-update
     (fn [this [_ fragment]]
       (set-hash-if-different! fragment))}))
