(ns reagent-manage.macros
  (:require [cljs.test]))

(defmacro <?
  "Try to read from a channel, but if the value got from it is an
  Error, raise an exception in the caller's context."
  [expr]
  `(reagent-manage.utils/throw-err (cljs.core.async/<! ~expr)))

(defmacro go-api
  "Just like a core.async go block, except wrapped with the default
  search console error-handler to display api errors in a
  human-friendly way."
  [& exprs]
  `(cljs.core.async.macros/go (try
         ~@exprs
         (catch js/Object e
           (reagent-manage.api/error-handler e)))))

(defmacro go-re
  "Just like a core.async go block, except wrapped with an exception handler that
  captures exceptions and returns them as data."
  [& exprs]
  `(cljs.core.async.macros/go (try
         ~@exprs
         (catch js/Object e (print e) (reagent-manage.utils/make-err e)))))

(defmacro try-expr
  "Used by the 'is-go' macro to catch unexpected exceptions.
  You don't call this."
  [msg form]
  `(try
     ~(cljs.test/assert-expr &env msg form)
     ;; I would rather do (catch :default e ...) here (as cljs.test
     ;; does) but core.async commit
     ;; https://github.com/clojure/core.async/commit/bd71ba73b43ea09e242d37a54eaaa0fed761504c
     ;; which enables this in go bodies has not made
     ;; it into a release yet.
     (catch js/Object t#
       (cljs.test/do-report
         {:type :error, :message ~msg,
          :expected '~form, :actual t#}))))

(defmacro is-go
  "Assertion macro like cljs.test/is except can be used inside go in
  older versions of core.async."
  ([form] `(reagent-manage.macros/is-go ~form nil))
  ([form msg]
   `(reagent-manage.macros/try-expr ~msg ~form)))
