(ns reagent-manage.modal
  (:require [reagent.core :as reagent :refer [cursor]]
            [reagent.session :as session]))

(defn modal-bg
  "Component to render shaded background for modal dialog. If called
  like `[modal-bg close-handler :key-dismiss true]`, any key event
  dismisses the modal. Likewise if `:bg-dismiss true` is supplied,
  clicking on the background dismisses the modal. "
  [close-handler & {:keys [key-dismiss bg-dismiss]}]
  (reagent/create-class
   {:component-did-mount
    (fn [this]
      (.setTimeout js/window
                   #(do
                      (aset this "prevFocused" (.-activeElement js/document))
                      (.focus (.getDOMNode this))) 0))

    :component-will-unmount
    (fn [this] (.focus (aget this "prevFocused")))

    :reagent-render
    (fn []
      [:div.reveal-modal-bg
       (merge
        {:tabIndex 0} ;; needed to allow div to get keyboard focus
        (if key-dismiss {:on-key-down close-handler} {})
        (if bg-dismiss {:on-click close-handler} {}))])}))

(defn modal
  "Component to render zurb foundation modal dialog."
  [close-fn content & options]
  (let
      [close-handler (fn [e]
                       (close-fn)
                       (.preventDefault e))]
    [:span
     (vec (concat [modal-bg close-handler] options))
     [:div.reveal-modal
      content
      [:a.close-reveal-modal {:on-click close-handler} "×"]]]))
