(ns reagent-manage.panels
  (:require
   [reagent-manage.utils :as u]
   [reagent.core :as reagent :refer [atom cursor]]))

(defn get-local-storage
  "Get JSON-serialized clojurescript value named `key` out of local storage."
  [key]
  (let [val (aget js/localStorage key)]
    (if (nil? val) nil (js->clj (.parse js/JSON val)))))

(defn set-local-storage
  "Put JSON-serialized clojurescript value `val` named `key` into local storage."
  [key val]
  (aset js/localStorage key (.stringify js/JSON (clj->js val))))

(def perma-hidden-panels-key
  "perma_hidden_panels")

(defn perma-hidden
  "Panels the user has permanently dismissed, as stored in localStorage"
  []
  (set (map keyword (get-local-storage perma-hidden-panels-key))))

(defn hide
  "Hide optional panel `panel`."
  [satm panel]
  (swap! (cursor satm [:visible-panels]) disj panel))

(defn perma-hide
  "Hide optional panel named by keyword `panel` permanently, recording
  this fact in localStorage."
  [satm panel]
  (hide satm panel)
  (set-local-storage
   perma-hidden-panels-key
   (conj (perma-hidden) (name panel))))

(defn toggle
  "Toggle visibilty of `panel`."
  [satm panel]
  (swap! (cursor satm [:visible-panels]) u/set-toggle panel))

(defn visible? [satm panel]
  (contains? (:visible-panels @satm) panel))
