(ns reagent-manage.popup
  (:require
   [reagent.core :as reagent :refer [atom]]))

(def popup-atm
  "Which popup is currently being shown, nil if none."
  (atom nil))

(defn clear
  "Hide all popups."
  []
  (reset! popup-atm nil))

(defn toggle
  "Show (or hide, if already shown) the popup named `popup-name`.
  The name doesn't have to be a string, just any clojurescript data
  structure that can be tested for equality. "
  [popup-name]
  (swap! popup-atm
         (fn [old] (if (= old popup-name) nil popup-name))))

(defn popup-on
  "Component to render popup interface element. `base` is what the
  popup should be centered on, `popup-name` is the popup's name,
  `popup` is the content to be shown.

  If popup-name is nil, then the popup will categorically not be
  shown, even when `@popup-atm` is nil.

  I decided to do it this way because it tends to be easier to simply
  pass in a maybe-nil value into the name than wrap components with
  an (if) at the callsite that's essentially a redundant copy of
  the (if) happening here."
  [base popup-name popup]
  (if (and popup-name (= @popup-atm popup-name))
   [:span.tooltip base
    [:span.tip {:class "visible"} popup]]
   base))
