(ns reagent-manage.tabs
  (:require
   [reagent.session :as session]
   [reagent-manage.utils :as u]
   [reagent-manage.popup :as popup]
   [reagent-manage.query-builder :as qb]
   [reagent-manage.api :as api]
   [reagent-manage.panels :as panels]
   [reagent.core :as reagent :refer [atom cursor]]
   [reagent-manage.ajax :refer [GET POST PUT DELETE]]
   [cljs.core.match :refer-macros [match]]
   ))

(defn tabbed-view
  "Component to render tabbed view where each element of `tabs` has
  the form `{:id :name :content}` and `active-tab-atm` is an atom
  whose value is the value is an `:id` field of one the tabs. `:name`
  is used for human-readable text, and `:content` is what to show when
  the tab is selected."
  [tabs active-tab-atm]
  (let [active-tab @active-tab-atm]
    [:div
     [:ul.tabs
      (for [tab tabs]
        ^{:key (:id tab)}
        [:li.tab-title {:class (if (= active-tab (:id tab)) "active" nil)}
         [:a {:on-click #(reset! active-tab-atm (:id tab))} (:name tab)]])]
     [:br]
     (:content (first (filter #(= (:id %) active-tab) tabs)))]))
