(ns test-runner.browser
  (:use-macros
   [test-runner.core :only [run-tests-in-env]])
  (:require
   [figwheel.client :as figwheel :include-macros true]
   [cljs.test :as test :refer [report]]
   [test-runner.core :as tests]))

(enable-console-print!)

(defn set-favico [color]
  (let [canvas (.createElement js/document "canvas")
        ctx (.getContext canvas "2d")]
    (set! (.-width canvas) 16)
    (set! (.-height canvas) 16)
    (set! (.-fillStyle ctx) color)
    (.fillRect ctx 0 0 16 16)
    (.attr (js/$ "link[type=\"image/x-icon\"]") "href" (.toDataURL canvas))))

(derive ::test/browser ::test/default)

(defmethod report [::test/browser :summary] [m]
  (.text (js/$ "#results") (str "\nRan " (:test m) " tests containing "
                 (+ (:pass m) (:fail m) (:error m)) " assertions, "
                 (:fail m) " failures, " (:error m) " errors."))
  (let [test-failures (+ (:fail m) (:error m))]
    (aset js/window "test-failures" test-failures)
    (set-favico (if (> test-failures 0) "#f30" "#0a0"))))

(defn run-tests-in-browser []
  (run-tests-in-env ::test/browser))

(defn init []
  (run-tests-in-browser)

  (figwheel/start
   {:websocket-url "ws://localhost:3449/figwheel-ws"
    :jsload-callback run-tests-in-browser
    :build-id "browser-test"}))
