require 'chef'
require 'mixlib/authentication'
require 'log_level'

# This initializer sets Rails' log level and creates a new Logger for Chef to
# log to. Chef will log to the same device (e.g. file) as Rails but separate
# loggers let us set their log levels independently.
#
# Log levels are configured in `settings.yml`.

log_level = LogLevel.new(Settings.logging.log_level)
chef_log_level = LogLevel.new(Settings.logging.chef_log_level ||
                              Settings.logging.log_level)

# Set Rails log level
#
# In an environment initializer (e.g. `environments/development.rb`) we can
# just set `config.log_level` and it'll set `Rails.logger.level` for us, but
# not here. So we have to set both.
Rails.application.config.log_level = log_level.to_sym

# `config.log_level` is e.g. `:info`, so get the corresponding integer value
# e.g. `Logger::Severity::INFO`.
Rails.logger.level = log_level.to_i

if Rails.env.test?
  # For unknown reasons, the BaseWorker specs fail if this is not set to
  # UNKNOWN in the test environment.
  Sidekiq.logger.level = LogLevel.new(:unknown).to_i
else
  # Use the same level as Rails for Sidekiq
  Sidekiq.logger.level = Rails.logger.level
end

# Create a new Logger instance for Chef so we can set its log level
# independently of Rails, but use the same logdev
chef_logger = Logger.new(Rails.application.config.logdev)

# Set the Chef log level
chef_logger.level = chef_log_level.to_i

Chef::Log.logger = Mixlib::Authentication::Log.logger = chef_logger
