begin
  require 'cover_me'
  require 'paint'

  module CoverMe
    class OCConsoleFormatter < HtmlFormatter
      def format_index(index)
        super
        if index.percent_tested >= 90
          puts Paint["Coverage: #{index.percent_tested}%", :green]
        elsif index.percent_tested >= 60
          file_to_improve = index.reports.select{|r| r.executed_percent < 60}.sample.filename
          puts Paint["Coverage: #{index.percent_tested}%\nHow about improving coverage on #{file_to_improve}?", :yellow]
        else
          puts Paint["Coverage: #{index.percent_tested}%\nRun `rake cover_me:open` to view the coverage report", :red]
        end
      end
    end
  end

  CoverMe.config do |c|
    # Only use the Emma formatter for CI builds
    CoverMe.config.formatter = if ENV['CI']
      CoverMe::EmmaFormatter
    elsif $stdout.tty?
      CoverMe::OCConsoleFormatter
    else
      CoverMe::HtmlFormatter
    end
    # Don't open the browser every time, thats annoying. We have a 'cover_me:open'
    # rake task to open the report, going to try and make it just print summary data though.
    c.at_exit = Proc.new { }
  end
rescue LoadError # cover_me is not installed in production
end