require 'uri'
require 'raven_settings'

Chef::Manage::Application.configure do
  rails_config = config
  SecureHeaders::Configuration.default do |config|
    config.x_frame_options = 'DENY'
    config.x_xss_protection = '1; mode=block'
    config.hsts = 'max-age=31536000; includeSubdomains'
    default = %w('self')

    if rails_config.action_controller.asset_host.present?
      asset_uri = URI(rails_config.action_controller.asset_host)
      default << "#{asset_uri.scheme}://#{asset_uri.host}:#{asset_uri.port}"
    end

    sentry_src = if RavenSettings.public_dsn.present?
      sentry_uri = URI(RavenSettings.public_dsn)
      ["#{sentry_uri.scheme}://#{sentry_uri.host}:#{sentry_uri.port}"]
    else
      []
    end

    google_analytics_src = ['https://ssl.google-analytics.com',
                            'https://stats.g.doubleclick.net',
                            'https://www.googletagmanager.com',
                            'https://www.google-analytics.com',
                            'https://cdn.cookielaw.org' ]


    demandbase_src = [
      'http://scripts.demandbase.com',
      'https://scripts.demandbase.com',
      'http://api.demandbase.com',
      'https://api.demandbase.com',
      'http://b.company-target.com',
      'https://b.company-target.com',
      'https://www.googletagmanager.com',
      'https://www.google-analytics.com',
      'https://cdn.cookielaw.org',
      'https://*.onetrust.com',
      'https://connect.facebook.net',
      'https://snap.licdn.com',
      'https://*.linkedin.com',
      'https://bat.bing.com/',
      'https://stats.g.doubleclick.net/',
      'https://static.ads-twitter.com/uwt.js',
      'https://www.google.com',
      'https://www.google.co.in',
      'https://*.eloqua.com',
      'https://analytics.twitter.com',
      'https://www.facebook.com',
      'https://t.co',
      'https://analytics.google.com',
      'https://obseu.ytwohlcq.telerik.com',
      'https://script.hotjar.com',
      'https://content.hotjar.io',
      'wss://ws.hotjar.com'
    ]

    recaptcha_src = [ 'https://www.google.com',
                      'https://www.gstatic.com' ]

    csp = {
      :default_src => default,
      :font_src => default + ['themes.googleusercontent.com'],
      :style_src => default + ['fonts.googleapis.com'] + %w('unsafe-inline'),
      :script_src => default + google_analytics_src + demandbase_src + recaptcha_src + %w('unsafe-eval') + ['blob:'],
      :img_src => default + sentry_src + google_analytics_src +
        demandbase_src + ['data:', 'blob:'],
      :object_src => %w('none'),
      :media_src => %w('none'),
      :frame_src => default + recaptcha_src + demandbase_src,
      :connect_src => default + demandbase_src,
    }

    if Rails.env.production?
      # csp[:enforce] = true
    else
      # csp[:enforce] = false
      # csp[:report_uri] = ['csp_reports']
    end

    config.csp = csp
  end
end
