class StoreIdentityMiddleware
  def call(worker_class, msg, queue, redis_pool)
    worker_class.store_identity(msg)
    yield
  end
end

class SetIdentityMiddleware
  def call(worker, msg, queue)
    worker.set_identity(msg)
    yield
  end
end

# This can be removed for Sidekiq > 2.5.2
class SetJidMiddleware
  def call(worker, msg, queue)
    worker.jid = msg['jid']
    yield
  end
end

class LifeCycleClientMiddleware
  def call(worker_class, msg, queue, redis_pool)
    worker_class.life_cycle_queued(msg)
    yield
  end
end

class LifeCycleServerMiddleware
  def call(worker, msg, queue)
    worker.life_cycle_started(msg)
    begin
      yield
    rescue Exception => ex
      worker.life_cycle_error(msg, ex)
      raise
    else
      worker.life_cycle_finished(msg)
    end
  end
end

def configure_middleware(config)
  config.client_middleware do |chain|
    chain.add StoreIdentityMiddleware
    chain.add LifeCycleClientMiddleware
  end

  config.server_middleware do |chain|
    chain.add SetIdentityMiddleware
    chain.add SetJidMiddleware
    chain.add LifeCycleServerMiddleware
  end
end

Chef::Manage::Application.configure do
  config.autoload_paths += %W(#{config.root}/workers)

  Sidekiq.configure_client do |sidekiq_config|
    configure_middleware(sidekiq_config)
    # In production this has to be done in an after_fork hook
    sidekiq_config.redis = { :url => Settings.redis.url } unless defined?(Unicorn)
  end

  Sidekiq.configure_server do |sidekiq_config|
    configure_middleware(sidekiq_config)
    # In production this has to be done in an after_fork hook
    sidekiq_config.redis = { :url => Settings.redis.url } unless defined?(Unicorn)
  end
end
