require 'action_view/helpers/sanitize_helper'
require 'rexml/rexml'
require 'kramdown'

class Changelog
  include ActionView::Helpers::SanitizeHelper

  attr_reader :text

  def initialize(text)
    @text = text
  end

  def title
    "Chef Management Console Change Log"
  end

  def description
    "New release information for the Chef Management Console"
  end

  def latest_date
    if releases.first && releases.first[:date]
      date = releases.first[:date]
    else
      date = DateTime.now
    end
    date.strftime("%a, %d %b %Y %H:%M:%S %z")
  end

  def releases
    @releases = []
    doc.root.children.each do |elm|
      if elm.type == :header && elm.options[:level] == 2 && md = elm.options[:raw_text].match(/^(.+) \(([\d-]+)\)$/)
        @releases << {version: md[1], date: DateTime.parse(md[2]), doc: Kramdown::Element.new(:root, doc.root.value, doc.root.attr, doc.root.options)}
      elsif !@releases.empty?
        @releases.last[:doc].children << elm
      end
    end
    @releases.delete_if do |rel|
      if text?(rel[:doc])
        rel[:html] = strip_links(
          Kramdown::Converter::Html.convert(rel[:doc]).first
        )
        rel.delete(:doc) # Don't leak this out to the world
        false
      else
        true
      end
    end
    @releases.compact
  end

  def to_html
    strip_links(doc.to_html)
  end

  private

  def doc
    @doc ||= Kramdown::Document.new(text,input: 'GFM')
  end

  def text?(elm)
    elm.type == :text  || elm.children.any?{ |child| text?(child) }
  end
end
