require 'opscode/dark_launch'

class DarkLaunch < Opscode::DarkLaunch
  private

  def self.config_file_path
    [
      File.join(Rails.root, 'config', 'dark_launch_features.json'),
      '/etc/chef-manage/dark_launch_features.json',
      '/etc/opscode-manage/dark_launch_features.json',
      '/etc/opscode-webui2/dark_launch_features.json',
      '/etc/opscode/dark_launch_features.json',
    ].each do |path|
      if File.exists?(path) && File.readable?(path)
        Rails.logger.debug("Loading dark launch features from #{path}")
        return path
      end
    end
    Rails.logger.debug('Not loading dark launch features, no files found')
    nil # No match
  end

  # generates a hash of all enabled darklaunch keys
  def self.all_features(org)
    is_feature_enabled?('totally_not_there', org) # Force data reload
    darklaunch = Hash.new
    @features_by_org.each do |k, v|
      feature_config = @features_by_org[k]
      is_enabled = case feature_config
                   when TrueClass, FalseClass
                     # feature is globally enabled/disabled
                     feature_config
                   when Hash
                     (org && feature_config[org])
                   else
                     false
                   end
      if !!is_enabled
        darklaunch[k] = true
      end
    end
    darklaunch
  end
end
