# frozen_string_literal: true

# Eloqua methods to be used in the sessions controller
require 'net/http'
require 'json'

require_relative 'eloqua_transform'

module Eloqua
	def submit_user_to_eloqua
		add_lead(
			lead_from_form(params)
		)
	end

	private

	def add_lead(data)
		send_to_eloqua(data)
	rescue => e
		# We should never crash the application for marketing purposes
		Rails.logger.error e
	end

	def send_to_eloqua(data)
		url = "#{ENV['ELQ_URL']}#{ENV['ELQ_FORM_ID']}"
		uri = URI(url)

		req = Net::HTTP::Post.new uri
				req['Content-Type'] = 'application/json'
				req.body  = data.to_json
				req.basic_auth ENV['ELQ_USER'],ENV['ELQ_PWD']

		response = Net::HTTP.start(uri.hostname, uri.port,
			:use_ssl => uri.scheme == 'https') do |http|

				http.request(req)
			end

		raise "Submission to Eloqua failed. MESSAGE #{response.message}" unless response.code == 201.to_s
	end

end
