require 'fileutils'

# Cloud Marketplace Sign Up
module MarketplaceSignUp
  def register_marketplace_user
    Registrar.new(params[:user]).register
  end

  def redirect_to_sign_in_if_marketplace_sentinel_is_expired
    if marketplace_sign_up_enabled? && marketplace_sentinel_expired?
      flash[:alert] = 'Sign Up is disabled because it has been longer ' \
                      "than #{Settings.marketplace.sign_up.timeout} seconds since " \
                      "Manage has been configured. Please run 'chef-manage-ctl " \
                      "reconfigure' to restart the timer."
      redirect_to(login_path) && return
    end
  end

  def validate_marketplace_user
    raise AccountsController::ValidationError.new "Missing 'password'" unless @validated_user[:password]
  end

  # Registrar
  class Registrar
    attr_reader :user

    def initialize(user_params)
      @user = user_params
    end

    def register
      register_node if user[:support]
    end

    private

    def register_node
      cmd_segments = [
        "chef-marketplace-ctl",
        "register-node",
        "-f",
        user[:first_name],
        "-l",
        user[:last_name],
        "-e",
        user[:email],
        "-o",
        user[:company]
      ]
      pid = Process.spawn(*cmd_segments)
      Process.detach(pid)
    end
  end
end
