module ReportingQueryHelper
  def escape(s)
    s.to_s.gsub(/([^ a-zA-Z0-9_.-]+)/n) {
      '%'+$1.unpack('H2'*$1.size).join('%').upcase
    }.tr(' ', '+')
  end

  def fuzzify_query(query)
    if query !~ /:/
      "tags:*#{query}* OR roles:*#{query}* OR fqdn:*#{query}* OR name:*#{query}*"
    else
      query
    end
  end

  def prepare_runs_query(params)
    query = ""

    if params[:dashboard].present?
      if params.has_key?(:bucketing) && params[:bucketing] == "duration"
        query += "reports/runs/durations?"
      else
        query += "reports/runs/counts?"
      end
    else
      if params[:node_id]
        query += "reports/nodes/#{params[:node_id]}/runs?"
      else
        query += "reports/org/runs?"
      end
    end

    if params.has_key?(:endTime) && params.has_key?(:startTime)
      query += "from=#{params[:startTime]}&until=#{params[:endTime]}&"
    end

    if params[:dashboard].present?
      if params.has_key?(:endTime) && params.has_key?(:startTime)
        query += "granularity=#{params[:granularity]}&"
      else
        Chef::Log.error("Dashboard query is executing without endTime and startTime parameters.")
        return nil
      end
    elsif params.has_key?(:start_row) && params.has_key?(:num_rows)
      query += "start=#{params[:start_row]}&rows=#{params[:num_rows]}&"

      if params.has_key?(:env)
        environment = "chef_environment:#{params[:env]}"
        escaped_environment = escape(URI::UrlParser.escape(environment, Regexp.new("[^#{URI::PATTERN::UNRESERVED}]")))
        query += "q=#{escaped_environment}&"
      end

      if params.has_key?(:status)
        query += "status=#{params[:status]}&"
      end
    else
      # Invalid run query
      return nil
    end

    # Remove the trailing &
    query = query.chomp("&")
    Chef::Log.debug(query)
    query
  end

  def prepare_diff_query(params)
    query = "reports/nodes/#{params[:node_id]}/runs/#{params[:run_id]}/#{params[:resource_index]}"
    query
  end

  def prepare_run_query(params)
    query = "reports/nodes/#{params[:node_id]}/runs/#{params[:run_id]}?start=#{params[:start_row]}&rows=#{params[:num_rows]}"

    query
  end
end
