# Takes input from the sessions controller validity action and returns a hash
# that can be rendered based on the input.
#
# The specs for this live in the sessions controller, since it tests everything
# there

require 'net/http'
require 'uri'

class SignUpValidator
  attr_reader :server

  def initialize(server)
    @server = server
  end

  def validate(params)
    case params[:type]
    when 'email'
      validate_email(params)
    when 'organization'
      validate_item("organizations/#{params[:value]}")
    when 'username'
      validate_item("users/#{params[:value]}")
    else
      { body: nil, status: :not_found }
    end
  rescue URI::InvalidURIError
    { json: { available: false }, status: :bad_request }
  end

  def validate_email(params)
    # NOTE: This will only work in private if you're using
    #       11.0.0-tech.preview.4 or newer, since the version of
    #       opscode-account in older versions does not support querying users
    #       by email.
    no_found_users = server.get_rest(
      "users?#{{ email: params[:value] }.to_query}"
    ).length == 0

    { json: { available: no_found_users },
      status: no_found_users ? :ok : :conflict }
  end

  def validate_item(uri)
    if server.get_rest(uri)
      { json: { available: false }, status: :conflict }
    end
  rescue Net::HTTPServerException => e
    if e.response.code.to_i == 404
      { json: { available: true }, status: :ok }
    end
  end
end
