require 'openssl'

# Manages signatures used for password resets
class Signature
  attr_reader :expiration, :secret_token, :username
  attr_accessor :user_params

  def initialize(username, expiration, secret_token, user_params={})
    @username = username
    @expiration = expiration
    @secret_token = secret_token
    @user_params = user_params
  end

  # String representation of signature
  def to_s
    OpenSSL::Digest::SHA256.hexdigest(canonical_string)
  end

  # Lets you compare signature == "some string"
  def ==(other)
    to_s == other.to_s
  end

  # Checks if another signature matches and is not expired
  def valid_for?(other)
    self == other && !expired?
  end

  private

  def canonical_string
    canonical = ["--#{username}", "--#{expiration}", "--#{secret_token}"]
    canonical.push("--#{user_params}") if user_params.present?
    canonical.join
  end

  def expired?
    Time.zone.now > Time.zone.at(expiration.to_i)
  end
end
