# class for handling USAGs
class USAGHelper
  def initialize(server, usag_regex, groups_url)
    @server = server
    @usag_regex = usag_regex
    @groups_url = groups_url
  end

  def populate_single_object!(obj)
    fill_in_usags!(obj) { |name| @server.get_rest("#{@groups_url}/#{name}") }
  end

  private

  def fill_in_usags!(obj, &block)
    unless obj.key?('forbidden')
      obj['groups'].delete_if do |name|
        if @usag_regex.match(name)
          usag_group = block.call(name)
          raise "USAG #{name} not found" unless usag_group
          raise "USAG #{name} has no actors: #{usag_group}" unless usag_group['actors']
          usag_member = usag_group['actors'].first
          if usag_member
            obj['actors'] << usag_member
          else
            Rails.logger.warn "USAG #{name} does not have a member: #{usag_group}"
          end
          true
        else
          false
        end
      end
    end
    obj
  end
end
