# Takes input from organization controller leave action and
# handles functionality around whether it is safe for a user to
# leave an org or not

class UserDissociation
  attr_reader :server, :org, :username

  # server:   chef API client object
  # username: string of the user to leave org
  # org:      string of the org the user is to leave
  def initialize(server, username, org)
    @server = server
    @org = org
    @username = username
  end

  def user
    @user ||= server.get_rest("users/#{username}")
  end

  def users
    @users ||= server.get_rest("organizations/#{org}/users")
  end

  def admins
    @admins ||= server.get_rest("organizations/#{org}/groups/admins")
  end

  def billing_admins
    @billing_admins ||= server.get_rest("organizations/#{org}/groups/billing-admins")
  end

  # true if @user leaving org would cause billing-admins group to be empty
  def last_user_in_billing_admins_group?
    would_leave_group_empty?(billing_admins["users"])
  end

  # true if @user leaving org would cause admins group to be empty
  def last_user_in_admins_group?
    would_leave_group_empty?(admins["users"])
  end

  # finds if a list of users from a group would be empty if @user
  # was deleted
  def would_leave_group_empty?(group_users)
    if group_users.present?
      group_users.delete_if do |group_user|
        found = false
        users.each do |user|
          username = user["user"]["username"]
          if username == group_user
            found = true
            break
          end
        end
        not found
      end
    end
    group_users.delete(user["username"])

    group_users.empty?
  end
end
