#
# Copyright:: Copyright (c) 2013-2016 Chef Software, Inc.
# License:: Apache License, Version 2.0
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

require 'benchmark'
require 'json'
require 'net/http'
require 'openssl'
require 'optparse'
require 'uri'

add_command 'test', 'Ensure Management Console is operational', 2 do
  # Parse incoming options
  options = {}
  OptionParser.new do |opts|
    opts.on("-J", "--junit-xml PATH") do |junit_xml|
      options[:junit_xml] = junit_xml
    end

    opts.on("--all") do |all|
      options[:run_all_tests] = all
    end

    opts.on("--smoke") do |smoke|
      options[:smoke_tests_only] = smoke
    end
  end.parse!

  manage_attributes = JSON.load(
    open('/var/opt/chef-manage/etc/chef-manage-running.json')
  )['chef-manage']
  private_chef_attributes = JSON.load(
    open('/etc/opscode/chef-server-running.json')
  )['private_chef']
  uri = URI.parse(
    "#{private_chef_attributes['nginx']['url']}:#{manage_attributes['public_port']}/login"
  )

  # Run a basic test
  request_success = false
  response = nil
  command_elapse = Benchmark.measure do
    http = Net::HTTP.new(uri.host, uri.port)
    http.use_ssl = true
    http.verify_mode = OpenSSL::SSL::VERIFY_NONE
    response = http.request(Net::HTTP::Get.new(uri.request_uri))
    request_success = response.body.include?("Sign In") &&
      response.code.to_i == 200
  end

  # If instructed, write out a very basic JUnit XML file for Jenkins to parse.
  if options[:junit_xml]
    elapse_time = command_elapse.real.round(2)
    File.open(options[:junit_xml], "w") do |file|
      xml =<<-XML
<?xml version="1.0" encoding="UTF-8"?>
<testsuites tests="1" failures="#{request_success ? 0 : 1}" disabled="0" errors="0" time="#{elapse_time}" >
      XML

      if request_success
        xml << <<-XML
  <testcase classname="ctl.test.request.login" name="login page basic request test" time="#{elapse_time}"/>
        XML
      else
        xml << <<-XML
  <testcase classname="ctl.test.request.login" name="login page basic request test" time="#{elapse_time}">
    <failure type="RequestFailed">Request status: #{response.code}</failure>
  </testcase>
        XML
      end

      xml << "</testsuites>"
      file.write(xml)
    end
  end

  exit(request_success ? 0 : 1)
end
